Package: aRtsy
Title: Generative Art with 'ggplot2'
Description: Provides algorithms for creating artworks in the 'ggplot2' language that incorporate some form of randomness.
Version: 1.0.1
Date: 2025-05-16
Authors@R: 
    person(given = "Koen",
           family = "Derks",
           role = c("aut", "cre"),
           email = "koen-derks@hotmail.com")
BugReports: https://github.com/koenderks/aRtsy/issues
URL: https://koenderks.github.io/aRtsy/,
        https://github.com/koenderks/aRtsy,
        https://twitter.com/aRtsy_package,
        https://mastodon.social/@aRtsy_package
Suggests: testthat (>= 3.0.0)
Imports: ambient, e1071, ggplot2 (>= 3.4.0), FNN, randomForest, Rcpp,
        scales, stats
LinkingTo: Rcpp, RcppArmadillo
Language: en-US
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.2
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2025-05-16 06:06:17 UTC; koenderks
Author: Koen Derks [aut, cre]
Maintainer: Koen Derks <koen-derks@hotmail.com>
Repository: CRAN
Date/Publication: 2025-05-16 08:50:28 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 05:40:45 UTC; unix
Archs: aRtsy.so.dSYM

Package: aRtsy
Title: Generative Art with 'ggplot2'
Description: Provides algorithms for creating artworks in the 'ggplot2' language that incorporate some form of randomness.
Version: 1.0.0
Date: 2024-11-05
Authors@R: 
    person(given = "Koen",
           family = "Derks",
           role = c("aut", "cre"),
           email = "koen-derks@hotmail.com")
BugReports: https://github.com/koenderks/aRtsy/issues
URL: https://koenderks.github.io/aRtsy/, https://github.com/koenderks/aRtsy, https://twitter.com/aRtsy_package, https://botsin.space/web/@aRtsy_package
Suggests:
    testthat (>= 3.0.0)
Imports:
    ambient,
    e1071,
    ggplot2 (>= 3.4.0),
    kknn,
    randomForest,
    Rcpp,
    scales,
    stats
LinkingTo:
    Rcpp,
    RcppArmadillo
Language: en-US
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.2
Config/testthat/edition: 3

# Generated by roxygen2: do not edit by hand

export(canvas_ant)
export(canvas_blacklight)
export(canvas_chladni)
export(canvas_circlemap)
export(canvas_cobweb)
export(canvas_collatz)
export(canvas_diamonds)
export(canvas_flame)
export(canvas_flow)
export(canvas_forest)
export(canvas_function)
export(canvas_gemstone)
export(canvas_lissajous)
export(canvas_mandelbrot)
export(canvas_maze)
export(canvas_mesh)
export(canvas_mosaic)
export(canvas_nebula)
export(canvas_petri)
export(canvas_phyllotaxis)
export(canvas_planet)
export(canvas_polylines)
export(canvas_recaman)
export(canvas_ribbons)
export(canvas_segments)
export(canvas_smoke)
export(canvas_splits)
export(canvas_squares)
export(canvas_stripes)
export(canvas_strokes)
export(canvas_swirls)
export(canvas_tiles)
export(canvas_turmite)
export(canvas_watercolors)
export(colorPalette)
export(saveCanvas)
export(theme_canvas)
import(Rcpp)
importFrom(stats,predict)
useDynLib(aRtsy)

[![CRAN](https://img.shields.io/cran/v/aRtsy?color=yellow&label=CRAN&logo=r)](https://cran.r-project.org/package=aRtsy)
[![R_build_status](https://github.com/koenderks/aRtsy/workflows/Build/badge.svg)](https://github.com/koenderks/aRtsy/actions)
[![Codecov](https://codecov.io/gh/koenderks/aRtsy/branch/development/graph/badge.svg?token=ZoxIB8p8PW)](https://app.codecov.io/gh/koenderks/aRtsy)
[![Bugs](https://img.shields.io/github/issues/koenderks/aRtsy/bug?label=Bugs&logo=github&logoColor=%23FFF&color=brightgreen)](https://github.com/koenderks/aRtsy/issues?q=is%3Aopen+is%3Aissue+label%3Abug)
[![Total](https://cranlogs.r-pkg.org/badges/grand-total/aRtsy?color=blue)](https://cranlogs.r-pkg.org)
[![Licence](https://img.shields.io/badge/licence-GPL--3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0.en.html)

# aRtsy: Generative Art with `R` and `ggplot2`

<img src='https://github.com/koenderks/aRtsy/raw/development/man/figures/logo.png' width='149' height='173' alt='logo' align='right' margin-left='20' margin-right='20'/>

*"If you laugh at a joke, what difference does it make if subsequently you are told that the joke was created by an algorithm?" - Marcus du Sautoy, The Creative Code*

`aRtsy` aims to make generative art accessible to the general public in a straightforward and standardized manner. The package provides algorithms for creating artworks that incorporate some form of randomness and are dependent on the set `seed`. Each algorithm is implemented in a separate function with its own set of parameters that can be tweaked.

Good luck hunting for some good `seed`'s!

## Artwork of the day

Every 24 hours this repository randomly generates and tweets an artwork from the `aRtsy` library. The full collection of daily artworks is available on the [twitter](https://twitter.com/aRtsy_package) feed and the [mastodon](https://botsin.space/web/@aRtsy_package) feed. This is today's artwork:

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/daily.png' width='50%'>
</p>

## Installation

The most recently released version of `aRtsy` can be downloaded from [CRAN](https://cran.r-project.org/package=aRtsy) by running the following command in R:

```r
install.packages("aRtsy")
```

Alternatively, you can download the development version from GitHub using:

```r
devtools::install_github("koenderks/aRtsy")
```

After installation, the `aRtsy` package can be loaded with:

```r
library(aRtsy)
```

**Note:** Render times in RStudio can be quite long for some artworks. It is therefore recommended that you save the artwork to a file (e.g., `.png` or `.jpg`) before viewing it. You can save the artwork in an appropriate size and quality using the `saveCanvas()` function.

```r
artwork <- canvas_strokes(colors = c("black", "white"))
saveCanvas(artwork, filename = "myArtwork.png")
```

## Available artworks

*The Iterative collection*

* [`canvas_ant()`](#langtons-ant)
* [`canvas_chladni()`](#chladni-figures)
* [`canvas_cobweb()`](#cobwebs)
* [`canvas_collatz()`](#collatz-conjecture)
* [`canvas_flame()`](#fractal-flames)
* [`canvas_flow()`](#flow-fields)
* [`canvas_lissajous()`](#lissajous-curves)
* [`canvas_maze()`](#mazes)
* [`canvas_mesh()`](#meshes)
* [`canvas_petri()`](#petri-dishes)
* [`canvas_phyllotaxis()`](#phyllotaxis)
* [`canvas_planet()`](#planets)
* [`canvas_recaman()`](#recamáns-sequence)
* [`canvas_smoke()`](#rainbow-smoke)
* [`canvas_splits()`](#split-lines)
* [`canvas_stripes()`](#stripes)
* [`canvas_strokes()`](#paint-strokes)
* [`canvas_swirls()`](#swirls)
* [`canvas_tiles()`](#portuguese-tiles)
* [`canvas_turmite()`](#turmite)
* [`canvas_watercolors()`](#watercolors)

*The Geometric collection*

* [`canvas_diamonds()`](#diamonds)
* [`canvas_function()`](#functions)
* [`canvas_polylines()`](#polylines)
* [`canvas_ribbons()`](#ribbons)
* [`canvas_segments()`](#segments)
* [`canvas_squares()`](#squares-and-rectangles)

*The Supervised collection*

* [`canvas_blacklight()`](#blacklights)
* [`canvas_forest()`](#forests)
* [`canvas_gemstone()`](#gemstones)
* [`canvas_mosaic()`](#mosaics)
* [`canvas_nebula()`](#Nebula)

*The Static collection*

* [`canvas_circlemap()`](#circle-maps)
* [`canvas_mandelbrot()`](#the-mandelbrot-set)

### The Iterative collection

The Iterative collection implements algorithms whose state depend on the previous state. These algorithms mostly use a grid based canvas to draw on. On this grid, each point represents a pixel of the final image. By assigning a color to these points according to certain rules, one can create the images in this collection.

#### Langton's ant

According to [Wikipedia](https://en.wikipedia.org/wiki/Langtons_ant), Langton's ant is a turmite with a very specific set of rules. In particular, after choosing a starting position the algorithm involves repeating the following three rules:

1. On a non-colored block: turn 90 degrees clockwise, un-color the block, move forward one block,
1. On a colored block: turn 90 degrees counter-clockwise, color the block, move forward one block,
1. If a certain number of iterations has passed, choose a different color which corresponds to a different combination of these rules.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/ants/2021-03-03.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/ants/2021-03-02.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/ants/2021-03-01.png' width='30%'>
</p>

You can use the `canvas_ant()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_ant(colors = colorPalette("house"))
# see ?canvas_ant for more input parameters of this function
```

#### Chladni figures

This function draws [Chladni](https://en.wikipedia.org/wiki/Ernst_Chladni) figures on the canvas. It works by generating one or multiple sine waves on a square matrix. You can provide the waves to be added yourself. After generating the waves it is possible to warp them using a [domain warping](https://iquilezles.org/articles/warp/) technique. The angles and distances for the warp can be set manually or according to a type of noise.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/chladnis/2021-11-12.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/chladnis/2021-11-13.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/chladnis/2021-11-14.png' width='30%'>
</p>

You can use the `canvas_chladni()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_chladni(colors = colorPalette("tuscany1"))
# see ?canvas_chladni for more input parameters of this function
```

#### Cobwebs

This function draws a lines in a structure that resemble cobwebs. The algorithm creates many [Fibonacci spirals](https://en.wikipedia.org/wiki/Golden_spiral) shifted by random noise from a normal distribution.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/cobwebs/2021-11-05.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/cobwebs/2021-11-07.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/cobwebs/2021-11-06.png' width='30%'>
</p>

You can use the `canvas_cobweb()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_cobweb(colors = colorPalette("tuscany1"))
# see ?canvas_cobweb for more input parameters of this function
```

#### Collatz conjecture

The Collatz conjecture is also known as the `3x+1` equation. The algorithm draws lines according to a simple rule set:

1. Take a random positive number.
2. If the number is even, divide it by 2.
3. If the number is odd, multiply the number by 3 and add 1.
4. Repeat to get a sequence of numbers.

By visualizing the sequence for each number, overlaying sequences that are the same, and bending the edges differently for even and odd numbers in the sequence, organic looking structures can occur.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/collatzs/2021-08-09.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/collatzs/2021-08-08.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/collatzs/2021-08-10.png' width='30%'>
</p>

You can use the `canvas_collatz()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_collatz(colors = colorPalette("tuscany3"))
# see ?canvas_collatz for more input parameters of this function
```

#### Fractal flames

This function implements the Fractal Flame algorithm described in [this article](https://flam3.com/flame_draves.pdf) by Scott Draves and Erik Reckase. It iterates a set of randomly determined function systems following one or multiple specific variations to determine a set of points. You can specify which variations from the article to include in the flame, what type of symmetry to include, whether to blend the variations using weights or to pick a single variation for each iteration, whether to apply a post transformation and whether to apply a final transformation (optionally including an additional posttransformation). The final image can either be based on a the origin of the attractors or on the log density of the hit count of each pixel (for a more rigid look).

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/flames/2022-10-06.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/flames/2022-10-05.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/flames/2022-10-04.png' width='30%'>
</p>

You can use the `canvas_flame()` function to make your own artwork using this algorithm.

```r
set.seed(2)
canvas_flame(colors = colorPalette("dark2"))
# see ?canvas_flame for more input parameters of this function
```

#### Flow fields

This artwork implements a version of the algorithm described in the blog post [Flow Fields](https://tylerxhobbs.com/essays/2020/flow-fields) by Tyler Hobbs. It works by creating a grid of angles and determining how certain points will flow through this field. The angles in the field can be set manually or according to the predictions of a supervised learning method trained on randomly generated data.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/flows/2021-09-24.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/flows/2021-09-23.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/flows/2021-09-22.png' width='30%'>
</p>

You can use the `canvas_flow()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_flow(colors = colorPalette("dark2"))
# see ?canvas_flow for more input parameters of this function
```

#### Lissajous curves

This function draws [Lissajous curves](https://en.wikipedia.org/wiki/Lissajous_curve) and subsequently connects the points on the curve to its *k*-nearest neighbors. The function is inspired by the Lissajous curves implemented in Marcus Volz's [mathart](https://github.com/marcusvolz/mathart) package but adds colors into the mix.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/lissajous/2023-08-17.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/lissajous/2023-08-18.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/lissajous/2023-08-19.png' width='30%'>
</p>

You can use the `canvas_lissajous()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_lissajous(colors = colorPalette("blossom"))
# see ?canvas_lissajous for more input parameters of this function
```

#### Mazes

This artwork creates mazes. The mazes are created using a random walk algorithm (described in the [mazegenerator](https://github.com/matfmc/mazegenerator) repository). The mazes can also be displayed with polar coordinates, creating some pretty cool effects.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mazes/2021-10-03.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mazes/2021-10-02.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mazes/2021-10-04.png' width='30%'>
</p>

You can use the `canvas_maze()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_maze(color = "#fafafa")
# see ?canvas_maze for more input parameters of this function
```

#### Meshes

This artwork creates one or more rotating circular morphing meshes on the canvas. The idea behind this artwork is described in [this blogpost](https://web.archive.org/web/20231130151102/http://rectangleworld.com/blog/archives/462) by Dan Gries with the simple words: *"deformed circles move across the canvas, and trace out these shapes"*. The circle has a three random components at each time step: the center, the radius, and the increase in the radius.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/meshes/2022-10-03.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/meshes/2022-10-02.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/meshes/2022-10-04.png' width='30%'>
</p>

You can use the `canvas_mesh()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_mesh(color = "#000000")
# see ?canvas_mesh for more input parameters of this function
```

#### Petri dishes

This artwork uses a space colonization algorithm (excellently described in [this blogpost](https://medium.com/@jason.webb/space-colonization-algorithm-in-javascript-6f683b743dc5) by Jason Webb) to draw Petri dish colonies. If you add a hole in the middle of the Petri dish, the colony grows around the hole.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/petris/2022-04-04.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/petris/2022-04-06.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/petris/2022-04-05.png' width='30%'>
</p>

You can use the `canvas_petri()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_petri(colors = colorPalette("sooph"))
# see ?canvas_petri for more input parameters of this function
```

#### Phyllotaxis

This function draws a [Phyllotaxis](https://en.wikipedia.org/wiki/Phyllotaxis) on the canvas. This structure represents the arrangement of leaves on a plant stem.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/phyllotaxis/2021-11-05.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/phyllotaxis/2021-11-03.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/phyllotaxis/2021-11-04.png' width='30%'>
</p>

You can use the `canvas_phyllotaxis()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_phyllotaxis(colors = colorPalette("tuscany1"))
# see ?canvas_phyllotaxis for more input parameters of this function
```

#### Planets

We all love space, and this type of artwork puts you right between the planets. The algorithm creates one or multiple planets in space and uses a cellular automata (described in the blog post [Neighborhoods: Experimenting with Cyclic Cellular Automata](https://fronkonstin.com/2021/01/02/neighborhoods-experimenting-with-cyclic-cellular-automata/) by Antonio Sánchez Chinchón) to fill in their surfaces. The color and placement of the planets can be set manually.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/planets/2021-02-26.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/planets/2021-02-27.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/planets/2021-02-28.png' width='30%'>
</p>

You can use the `canvas_planet()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_planet(colors = colorPalette("retro3"))
# see ?canvas_planet for more input parameters of this function
```

#### Recamán's sequence

This function draws Recamán's sequence on a canvas. The algorithm takes increasingly large steps backwards on the positive number line, but takes a step forward if it is unable to perform the step backwards.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/recamans/2021-11-02.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/recamans/2021-11-03.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/recamans/2021-11-04.png' width='30%'>
</p>

You can use the `canvas_recaman()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_recaman(colors = colorPalette("random", n = 10))
# see ?canvas_recaman for more input parameters of this function
```

#### Rainbow smoke

This function implements the [rainbow smoke](http://rainbowsmoke.hu/home) algorithm, an original idea from Jozsef Fejes. The algorithm works as follows: First, we select an initial color and apply it to a randomly chosen pixel. Next, we proceed to use the remaining colors one at the time and color pixels in a manner where colors that are alike are positioned near each other.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/smokes/2023-01-10.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/smokes/2023-01-08.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/smokes/2023-01-09.png' width='30%'>
</p>

You can use the `canvas_smoke()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_smoke(colors = colorPalette("random", 1024))
# see ?canvas_smoke for more input parameters of this function
```

#### Split lines

This function generates a [fractal curve](https://en.wikipedia.org/wiki/Fractal_curve). It starts with four simple lines and proceeds to split each line in four new line segments. If this action is repeated for some time, and each time the same split is made, the end product is a fractal curve. The fractal curve in this function (optionally) uses some noise to create random distortions in the curve.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/splits/2022-05-03.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/splits/2022-05-01.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/splits/2022-05-02.png' width='30%'>
</p>

You can use the `canvas_splits()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_splits(colors = colorPalette("origami"))
# see ?canvas_splits for more input parameters of this function
```

#### Stripes

This type of artwork is based on the concept of [Brownian motion](https://en.wikipedia.org/wiki/Brownian_motion). The algorithm generates a sequence of slightly increasing and decreasing values for each row on the canvas. Next, it fills these according to their generated value. More colors usually make this artwork more interesting.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/stripes/2021-08-23.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/stripes/2021-08-24.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/stripes/2021-08-25.png' width='30%'>
</p>

You can use the `canvas_stripes()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_stripes(colors = colorPalette("random", n = 10))
# see ?canvas_stripes for more input parameters of this function
```

#### Paint strokes

When you think of the act of painting, you probably imagine stroking paint on a canvas. This type of artwork tries to mimic that activity. The algorithm is based on the simple idea that each next point on a grid-based canvas has a chance to take over the color of an adjacent colored point, but also has a minor chance of generating a new color. Going over the canvas like this results in something that looks like strokes of paint.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/strokes/2021-03-21.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/strokes/2021-03-20.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/strokes/2021-03-10.png' width='30%'>
</p>

You can use the `canvas_strokes()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_strokes(colors = colorPalette("tuscany1"))
# see ?canvas_strokes for more input parameters of this function
```

#### Swirls

In this artwork, inspired by Matt Deslauriers' blog post on [generative art with Node.js and canvas](https://mattdesl.svbtle.com/generative-art-with-nodejs-and-canvas), a particle system takes center stage, giving birth to enchanting swirling lines. The individual particles in the system are subjected to velocities in two directions, dictating the paths they traverse while shaping the swirls on the canvas.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/swirls/2023-05-18.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/swirls/2023-05-20.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/swirls/2023-05-19.png' width='30%'>
</p>

You can use the `canvas_swirls()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_swirls(colors = colorPalette("tuscany1"))
# see ?canvas_swirls for more input parameters of this function
```

#### Portuguese tiles

This function attempts to recreate the intricate designs on Portuguese-style decorative tiles. The artwork uses a reaction-diffusion algorithm, which simulates the process of chemicals reacting and diffusing on a surface (excellently described in these three blogposts: [link](https://itp.uni-frankfurt.de/~gros/StudentProjects/Projects_2020/projekt_schulz_kaefer/), [link](https://www.karlsims.com/rd.html) and [link](https://fronkonstin.com/2019/12/28/reaction-diffusion/)), to generate an initial quarter of each tile. Next, the initial quarter tile is mirrored twice to get the symmetric look that is reminiscent of traditional Portuguese tiles. You can fill the wall with as many unique tiles as you want and specify a custom layout.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/tiles/2023-04-29.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/tiles/2023-04-28.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/tiles/2023-04-27.png' width='30%'>
</p>

You can use the `canvas_tiles()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_tiles(colors = colorPalette("azul"))
# see ?canvas_tiles for more input parameters of this function
```

#### Turmite

According to [Wikipedia](https://en.wikipedia.org/wiki/Turmite), a turmite is *"a Turing machine which has an orientation in addition to a current state and a "tape" that consists of an infinite two-dimensional grid of cells"*. The classic algorithm consists of repeating the following three simple steps:

1. Turn on the spot (left, right, up, or down),
2. Change the color of the block,
3. Move forward one block.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/turmites/2021-03-06.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/turmites/2021-03-09.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/turmites/2021-03-08.png' width='30%'>
</p>

You can use the `canvas_turmite()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_turmite(colors = colorPalette("dark2"))
# see ?canvas_turmite for more input parameters of this function
```

#### Watercolors

This artwork implements a version of the algorithm described in the blog post [A Guide to Simulating Watercolor Paint with Generative Art](https://tylerxhobbs.com/essays/2017/a-generative-approach-to-simulating-watercolor-paints) by Tyler Hobbs. It works by layering several geometric shapes and deforming each shape by repeatedly splitting its edges.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/watercolors/2021-09-23.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/watercolors/2021-09-22.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/watercolors/2021-09-21.png' width='30%'>
</p>

You can use the `canvas_watercolors()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_watercolors(colors = colorPalette("tuscany2"))
# see ?canvas_watercolors for more input parameters of this function
```

### The Geometric collection

The Geometric collection mostly implements algorithms that draw a geometric shape and apply a random color to it.

#### Diamonds

This function creates a set of diamonds on a canvas. The diamonds are filled in (or left out) using a random color assignment.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/diamonds/2021-08-06.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/diamonds/2021-08-04.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/diamonds/2021-08-05.png' width='30%'>
</p>

You can use the `canvas_diamonds()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_diamonds(colors = colorPalette("tuscany1"))
# see ?canvas_diamonds for more input parameters of this function
```

#### Functions

The idea for this type of artwork is taken over from the [`generativeart`](https://github.com/cutterkom/generativeart) package. In this algorithm, the position of every single point is calculated by a formula which has random parameters. You can supply your own formula.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/functions/2021-03-17.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/functions/2021-04-08.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/functions/2021-04-04.png' width='30%'>
</p>

You can use the `canvas_function()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_function(colors = colorPalette("tuscany1"))
# see ?canvas_function for more input parameters of this function
```

#### Polylines

This function draws many points on the canvas and connects these points into a polygon. After repeating this for all the colors, the edges of all polygons are drawn on top of the artwork.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/polylines/2021-07-22.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/polylines/2021-07-23.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/polylines/2021-07-21.png' width='30%'>
</p>

You can use the `canvas_polylines()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_polylines(colors = colorPalette("retro1"))
# see ?canvas_polylines for more input parameters of this function
```

#### Ribbons

This function creates colored ribbons with (or without) a triangle that breaks their paths. This path of the ribbon polygon is creating by picking one point on the left side of the triangle and one point on the right side at random and using these points as nodes.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/ribbons/2021-07-16.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/ribbons/2021-07-15.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/ribbons/2021-07-14.png' width='30%'>
</p>

You can use the `canvas_ribbons()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_ribbons(colors = colorPalette("retro1")
# see ?canvas_ribbons for more input parameters of this function
```

#### Segments

This type of artwork is inspired by the style of the well-known paintings by the Dutch artist [Piet Mondriaan](https://nl.wikipedia.org/wiki/Piet_Mondriaan). The position and direction of each line segment is determined randomly.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/segments/2021-08-07.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/segments/2021-08-08.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/segments/2021-08-06.png' width='30%'>
</p>

You can use the `canvas_segments()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_segments(colors = colorPalette("dark1"))
# see ?canvas_segments for more input parameters of this function
```

#### Squares and rectangles

This artwork uses a variety of squares and rectangles to fill the canvas. It works by repeatedly cutting into the canvas at random locations and coloring the area that these cuts create.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/squares/2021-03-01.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/squares/2021-02-28.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/squares/2021-02-29.png' width='30%'>
</p>

You can use the `canvas_squares()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_squares(colors = colorPalette("retro2"))
# see ?canvas_squares for more input parameters of this function
```

### The Supervised collection

The artworks in the Supervised collection are inspired by decision boundary plots in machine learning tasks. The algorithms in this collection work by generating random data points on a two dimensional surface (with either a continuous or a categorical response variable), which they then try to model using the supervised learning algorithm. Next, they try to predict the color of each pixel on the canvas.

#### Blacklights

This artwork is inspired by a supervised machine learning method called support vector machines. It applies the principle as described above using a continuous response variable to fill the color of the pixels.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/blacklights/2021-08-22.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/blacklights/2021-08-21.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/blacklights/2021-08-20.png' width='30%'>
</p>

You can use the `canvas_blacklight()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_blacklight(colors = colorPalette("random", n = 5))
# see ?canvas_blacklight for more input parameters of this function
```

#### Forests

This artwork is inspired by a supervised learning method called random forest. It applies the principle as described above using a continuous response variable to fill the color of the pixels.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/forests/2021-08-20.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/forests/2021-08-21.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/forests/2021-08-19.png' width='30%'>
</p>

You can use the `canvas_forest()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_forest(colors = colorPalette("jungle"))
# see ?canvas_forest for more input parameters of this function
```

#### Gemstones

This artwork is inspired by a supervised learning method called k-nearest neighbors. It applies the principle as described above using a continuous response variable to fill the color of the pixels. In short, the k-nearest neighbors algorithm computes the distance of each pixel on the canvas to each randomly generated data point and assigns it the color of the value of that data point.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/gemstones/2021-08-20.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/gemstones/2021-08-21.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/gemstones/2021-08-22.png' width='30%'>
</p>

You can use the `canvas_gemstone()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_gemstone(colors = colorPalette("dark3"))
# see ?canvas_gemstone for more input parameters of this function
```

#### Mosaics

This artwork also uses a k-nearest neighbors method but instead of a continuous response variable a categorical one is used, making it a classification problem. If you considers fewer neighbors the artwork looks like a mosaic, while higher values make the artwork look more smooth.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mosaics/2021-08-17.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mosaics/2021-08-19.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mosaics/2021-08-18.png' width='30%'>
</p>

You can use the `canvas_mosaic()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_mosaic(colors = colorPalette("retro2"))
# see ?canvas_mosaic for more input parameters of this function
```

#### Nebula

Based on the very same principle as described in the artwork above is this next type of artwork. However, it produces slightly different pictures as it uses different code to create a form of k-nearest neighbors noise. Some of these artworks can resemble nebulas in outer space.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/nebulas/2021-08-29.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/nebulas/2021-08-28.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/nebulas/2021-08-27.png' width='30%'>
</p>

You can use the `canvas_nebula()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_nebula(colors = colorPalette("tuscany1"))
# see ?canvas_nebula for more input parameters of this function
```

### The Static collection

The Static collection implements static images that produce nice pictures.

#### Circle maps

This type of artwork is based on the concept of an [Arnold tongue](https://en.wikipedia.org/wiki/Arnold_tongue). According to Wikipedia, Arnold tongues *"are a pictorial phenomenon that occur when visualizing how the rotation number of a dynamical system, or other related invariant property thereof, changes according to two or more of its parameters"*.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/circlemaps/2021-04-22b.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/circlemaps/2021-04-22c.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/circlemaps/2021-04-22a.png' width='30%'>
</p>

You can use the `canvas_circlemap()` function to make your own artwork using this algorithm.

```r
canvas_circlemap(colors = colorPalette("dark2"))
# see ?canvas_circlemap for more input parameters of this function
```

#### The Mandelbrot set

This type of artwork visualizes the [Mandelbrot set](https://en.wikipedia.org/wiki/Mandelbrot_set) and other related fractals (e.g., the [Julia set](https://en.wikipedia.org/wiki/Julia_set), the [Multibrot set](https://en.wikipedia.org/wiki/Multibrot_set) and the [Burning ship fractal](https://en.wikipedia.org/wiki/Burning_Ship_fractal)). These fractals are well-known examples of a complex structure arising from the application of a simple rule set. You can zoom in on the set and apply some color to create these images below.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mandelbrots/2021-08-07.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mandelbrots/2021-08-09.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mandelbrots/2021-08-08.png' width='30%'>
</p>

You can use the `canvas_mandelbrot()` function to make your own artwork using this algorithm.

```r
canvas_mandelbrot(colors = colorPalette("tuscany1"))
# see ?canvas_mandelbrot for more input parameters of this function
```

## Color palettes

The function `colorPalette()` can be used to generate a (semi-)random color palette, or pick a pre-implemented color palette. Currently, the color palettes displayed below are implemented in `aRtsy`. Feel free to suggest or add a new palette by making an [issue](https://github.com/koenderks/aRtsy/issues) on GitHub!

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/man/figures/colors.svg' width='90%'>
</p>

## Contributing to the `aRtsy` package

Contributions to the `aRtsy` package are very much appreciated and you are free to suggest or add your own algorithms! If you want to add your own artwork to the package so that others can create unique versions of it, feel free to make a pull request to the [GitHub repository](https://github.com/koenderks/aRtsy). Don't forget to also adjust [generate-artwork.R](https://github.com/koenderks/aRtsy/blob/development/.github/workflows/generate_artwork.R) if you want the artwork to show up in the 'Artwork of the day' category and the twitter feed.

[![CRAN](https://img.shields.io/cran/v/aRtsy?color=yellow&label=CRAN&logo=r)](https://cran.r-project.org/package=aRtsy)
[![R_build_status](https://github.com/koenderks/aRtsy/workflows/Build/badge.svg)](https://github.com/koenderks/aRtsy/actions)
[![Codecov](https://codecov.io/gh/koenderks/aRtsy/branch/development/graph/badge.svg?token=ZoxIB8p8PW)](https://app.codecov.io/gh/koenderks/aRtsy)
[![Bugs](https://img.shields.io/github/issues/koenderks/aRtsy/bug?label=Bugs&logo=github&logoColor=%23FFF&color=brightgreen)](https://github.com/koenderks/aRtsy/issues?q=is%3Aopen+is%3Aissue+label%3Abug)
[![Total](https://cranlogs.r-pkg.org/badges/grand-total/aRtsy?color=blue)](https://cranlogs.r-pkg.org)
[![Licence](https://img.shields.io/badge/licence-GPL--3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0.en.html)

# aRtsy: Generative Art with `R` and `ggplot2`

<img src='https://github.com/koenderks/aRtsy/raw/development/man/figures/logo.png' width='149' height='173' alt='logo' align='right' margin-left='20' margin-right='20'/>

*"If you laugh at a joke, what difference does it make if subsequently you are told that the joke was created by an algorithm?" - Marcus du Sautoy, The Creativity Code*

`aRtsy` aims to make generative art accessible to the general public in a straightforward and standardized manner. The package provides algorithms for creating artworks that incorporate some form of randomness and are dependent on the set `seed`. Each algorithm is implemented in a separate function with its own set of parameters that can be tweaked.

Good luck hunting for some good `seed`'s!

## Artwork of the day

Every 24 hours this repository randomly generates and tweets an artwork from the `aRtsy` library. The full collection of daily artworks is available on the [twitter](https://twitter.com/aRtsy_package) feed and the [mastodon](https://mastodon.social/@aRtsy_package) feed. This is today's artwork:

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/daily.png' width='50%'>
</p>

## Installation

The most recently released version of `aRtsy` can be downloaded from [CRAN](https://cran.r-project.org/package=aRtsy) by running the following command in R:

```r
install.packages("aRtsy")
```

Alternatively, you can download the development version from GitHub using:

```r
devtools::install_github("koenderks/aRtsy")
```

After installation, the `aRtsy` package can be loaded with:

```r
library(aRtsy)
```

**Note:** Render times in RStudio can be quite long for some artworks. It is therefore recommended that you save the artwork to a file (e.g., `.png` or `.jpg`) before viewing it. You can save the artwork in an appropriate size and quality using the `saveCanvas()` function.

```r
artwork <- canvas_strokes(colors = c("black", "white"))
saveCanvas(artwork, filename = "myArtwork.png")
```

## Available artworks

*The Iterative collection*

* [`canvas_ant()`](#langtons-ant)
* [`canvas_chladni()`](#chladni-figures)
* [`canvas_cobweb()`](#cobwebs)
* [`canvas_collatz()`](#collatz-conjecture)
* [`canvas_flame()`](#fractal-flames)
* [`canvas_flow()`](#flow-fields)
* [`canvas_lissajous()`](#lissajous-curves)
* [`canvas_maze()`](#mazes)
* [`canvas_mesh()`](#meshes)
* [`canvas_petri()`](#petri-dishes)
* [`canvas_phyllotaxis()`](#phyllotaxis)
* [`canvas_planet()`](#planets)
* [`canvas_recaman()`](#recamáns-sequence)
* [`canvas_slime()`](#slime-mold)
* [`canvas_smoke()`](#rainbow-smoke)
* [`canvas_splits()`](#split-lines)
* [`canvas_stripes()`](#stripes)
* [`canvas_strokes()`](#paint-strokes)
* [`canvas_swirls()`](#swirls)
* [`canvas_tiles()`](#portuguese-tiles)
* [`canvas_turmite()`](#turmite)
* [`canvas_watercolors()`](#watercolors)

*The Geometric collection*

* [`canvas_diamonds()`](#diamonds)
* [`canvas_function()`](#functions)
* [`canvas_polylines()`](#polylines)
* [`canvas_ribbons()`](#ribbons)
* [`canvas_segments()`](#segments)
* [`canvas_squares()`](#squares-and-rectangles)

*The Supervised collection*

* [`canvas_blacklight()`](#blacklights)
* [`canvas_forest()`](#forests)
* [`canvas_gemstone()`](#gemstones)
* [`canvas_mosaic()`](#mosaics)
* [`canvas_nebula()`](#Nebula)

*The Static collection*

* [`canvas_circlemap()`](#circle-maps)
* [`canvas_mandelbrot()`](#the-mandelbrot-set)

### The Iterative collection

The Iterative collection implements algorithms whose state depend on the previous state. These algorithms mostly use a grid based canvas to draw on. On this grid, each point represents a pixel of the final image. By assigning a color to these points according to certain rules, one can create the images in this collection.

#### Langton's ant

According to [Wikipedia](https://en.wikipedia.org/wiki/Langtons_ant), Langton's ant is a turmite with a very specific set of rules. In particular, after choosing a starting position the algorithm involves repeating the following three rules:

1. On a non-colored block: turn 90 degrees clockwise, un-color the block, move forward one block,
1. On a colored block: turn 90 degrees counter-clockwise, color the block, move forward one block,
1. If a certain number of iterations has passed, choose a different color which corresponds to a different combination of these rules.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/ants/2021-03-03.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/ants/2021-03-02.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/ants/2021-03-01.png' width='30%'>
</p>

You can use the `canvas_ant()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_ant(colors = colorPalette("house"))
# see ?canvas_ant for more input parameters of this function
```

#### Chladni figures

This function draws [Chladni](https://en.wikipedia.org/wiki/Ernst_Chladni) figures on the canvas. It works by generating one or multiple sine waves on a square matrix. You can provide the waves to be added yourself. After generating the waves it is possible to warp them using a [domain warping](https://iquilezles.org/articles/warp/) technique. The angles and distances for the warp can be set manually or according to a type of noise.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/chladnis/2021-11-12.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/chladnis/2021-11-13.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/chladnis/2021-11-14.png' width='30%'>
</p>

You can use the `canvas_chladni()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_chladni(colors = colorPalette("tuscany1"))
# see ?canvas_chladni for more input parameters of this function
```

#### Cobwebs

This function draws a lines in a structure that resemble cobwebs. The algorithm creates many [Fibonacci spirals](https://en.wikipedia.org/wiki/Golden_spiral) shifted by random noise from a normal distribution.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/cobwebs/2021-11-05.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/cobwebs/2021-11-07.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/cobwebs/2021-11-06.png' width='30%'>
</p>

You can use the `canvas_cobweb()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_cobweb(colors = colorPalette("tuscany1"))
# see ?canvas_cobweb for more input parameters of this function
```

#### Collatz conjecture

The Collatz conjecture is also known as the `3x+1` equation. The algorithm draws lines according to a simple rule set:

1. Take a random positive number.
2. If the number is even, divide it by 2.
3. If the number is odd, multiply the number by 3 and add 1.
4. Repeat to get a sequence of numbers.

By visualizing the sequence for each number, overlaying sequences that are the same, and bending the edges differently for even and odd numbers in the sequence, organic looking structures can occur.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/collatzs/2021-08-09.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/collatzs/2021-08-08.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/collatzs/2021-08-10.png' width='30%'>
</p>

You can use the `canvas_collatz()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_collatz(colors = colorPalette("tuscany3"))
# see ?canvas_collatz for more input parameters of this function
```

#### Fractal flames

This function implements the Fractal Flame algorithm described in [this article](https://flam3.com/flame_draves.pdf) by Scott Draves and Erik Reckase. It iterates a set of randomly determined function systems following one or multiple specific variations to determine a set of points. You can specify which variations from the article to include in the flame, what type of symmetry to include, whether to blend the variations using weights or to pick a single variation for each iteration, whether to apply a post transformation and whether to apply a final transformation (optionally including an additional posttransformation). The final image can either be based on a the origin of the attractors or on the log density of the hit count of each pixel (for a more rigid look).

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/flames/2022-10-06.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/flames/2022-10-05.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/flames/2022-10-04.png' width='30%'>
</p>

You can use the `canvas_flame()` function to make your own artwork using this algorithm.

```r
set.seed(2)
canvas_flame(colors = colorPalette("dark2"))
# see ?canvas_flame for more input parameters of this function
```

#### Flow fields

This artwork implements a version of the algorithm described in the blog post [Flow Fields](https://tylerxhobbs.com/essays/2020/flow-fields) by Tyler Hobbs. It works by creating a grid of angles and determining how certain points will flow through this field. The angles in the field can be set manually or according to the predictions of a supervised learning method trained on randomly generated data.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/flows/2021-09-24.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/flows/2021-09-23.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/flows/2021-09-22.png' width='30%'>
</p>

You can use the `canvas_flow()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_flow(colors = colorPalette("dark2"))
# see ?canvas_flow for more input parameters of this function
```

#### Lissajous curves

This function draws [Lissajous curves](https://en.wikipedia.org/wiki/Lissajous_curve) and subsequently connects the points on the curve to its *k*-nearest neighbors. The function is inspired by the Lissajous curves implemented in Marcus Volz's [mathart](https://github.com/marcusvolz/mathart) package but adds colors into the mix.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/lissajous/2023-08-17.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/lissajous/2023-08-18.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/lissajous/2023-08-19.png' width='30%'>
</p>

You can use the `canvas_lissajous()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_lissajous(colors = colorPalette("blossom"))
# see ?canvas_lissajous for more input parameters of this function
```

#### Mazes

This artwork creates mazes. The mazes are created using a random walk algorithm (described in the [mazegenerator](https://github.com/matfmc/mazegenerator) repository). The mazes can also be displayed with polar coordinates, creating some pretty cool effects.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mazes/2021-10-03.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mazes/2021-10-02.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mazes/2021-10-04.png' width='30%'>
</p>

You can use the `canvas_maze()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_maze(color = "#fafafa")
# see ?canvas_maze for more input parameters of this function
```

#### Meshes

This artwork creates one or more rotating circular morphing meshes on the canvas. The idea behind this artwork is described in [this blogpost](https://web.archive.org/web/20231130151102/http://rectangleworld.com/blog/archives/462) by Dan Gries with the simple words: *"deformed circles move across the canvas, and trace out these shapes"*. The circle has a three random components at each time step: the center, the radius, and the increase in the radius.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/meshes/2022-10-03.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/meshes/2022-10-02.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/meshes/2022-10-04.png' width='30%'>
</p>

You can use the `canvas_mesh()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_mesh(color = "#000000")
# see ?canvas_mesh for more input parameters of this function
```

#### Petri dishes

This artwork uses a space colonization algorithm (excellently described in [this blogpost](https://medium.com/@jason.webb/space-colonization-algorithm-in-javascript-6f683b743dc5) by Jason Webb) to draw Petri dish colonies. If you add a hole in the middle of the Petri dish, the colony grows around the hole.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/petris/2022-04-04.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/petris/2022-04-06.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/petris/2022-04-05.png' width='30%'>
</p>

You can use the `canvas_petri()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_petri(colors = colorPalette("sooph"))
# see ?canvas_petri for more input parameters of this function
```

#### Phyllotaxis

This function draws a [Phyllotaxis](https://en.wikipedia.org/wiki/Phyllotaxis) on the canvas. This structure represents the arrangement of leaves on a plant stem.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/phyllotaxis/2021-11-05.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/phyllotaxis/2021-11-03.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/phyllotaxis/2021-11-04.png' width='30%'>
</p>

You can use the `canvas_phyllotaxis()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_phyllotaxis(colors = colorPalette("tuscany1"))
# see ?canvas_phyllotaxis for more input parameters of this function
```

#### Planets

We all love space, and this type of artwork puts you right between the planets. The algorithm creates one or multiple planets in space and uses a cellular automata (described in the blog post [Neighborhoods: Experimenting with Cyclic Cellular Automata](https://fronkonstin.com/2021/01/02/neighborhoods-experimenting-with-cyclic-cellular-automata/) by Antonio Sánchez Chinchón) to fill in their surfaces. The color and placement of the planets can be set manually.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/planets/2021-02-26.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/planets/2021-02-27.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/planets/2021-02-28.png' width='30%'>
</p>

You can use the `canvas_planet()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_planet(colors = colorPalette("retro3"))
# see ?canvas_planet for more input parameters of this function
```

#### Recamán's sequence

This function draws Recamán's sequence on a canvas. The algorithm takes increasingly large steps backwards on the positive number line, but takes a step forward if it is unable to perform the step backwards.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/recamans/2021-11-02.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/recamans/2021-11-03.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/recamans/2021-11-04.png' width='30%'>
</p>

You can use the `canvas_recaman()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_recaman(colors = colorPalette("random", n = 10))
# see ?canvas_recaman for more input parameters of this function
```

#### Slime mold

This function implements the [Physarium](https://doi.org/10.1162/artl.2010.16.2.16202) model, which is neatly described in [this blogpost](https://cargocollective.com/sagejenson/physarum) by Sage Jenson and [this blogpost](https://fronkonstin.com/2020/08/11/abstractions/) by Antonio Sánchez Chinchón. The algorithm simulates particles on a two-dimensional grid that move towards areas on the grid with a high intensity.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/slimes/2024-11-19.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/slimes/2024-11-18.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/slimes/2024-11-20.png' width='30%'>
</p>

You can use the `canvas_slime()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_slime(colors = colorPalette("neon1"))
# see ?canvas_slime for more input parameters of this function
```

#### Rainbow smoke

This function implements the [rainbow smoke](http://rainbowsmoke.hu/home) algorithm, an original idea from Jozsef Fejes. The algorithm works as follows: First, we select an initial color and apply it to a randomly chosen pixel. Next, we proceed to use the remaining colors one at the time and color pixels in a manner where colors that are alike are positioned near each other.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/smokes/2023-01-10.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/smokes/2023-01-08.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/smokes/2023-01-09.png' width='30%'>
</p>

You can use the `canvas_smoke()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_smoke(colors = colorPalette("random", 1024))
# see ?canvas_smoke for more input parameters of this function
```

#### Split lines

This function generates a [fractal curve](https://en.wikipedia.org/wiki/Fractal_curve). It starts with four simple lines and proceeds to split each line in four new line segments. If this action is repeated for some time, and each time the same split is made, the end product is a fractal curve. The fractal curve in this function (optionally) uses some noise to create random distortions in the curve.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/splits/2022-05-03.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/splits/2022-05-01.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/splits/2022-05-02.png' width='30%'>
</p>

You can use the `canvas_splits()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_splits(colors = colorPalette("origami"))
# see ?canvas_splits for more input parameters of this function
```

#### Stripes

This type of artwork is based on the concept of [Brownian motion](https://en.wikipedia.org/wiki/Brownian_motion). The algorithm generates a sequence of slightly increasing and decreasing values for each row on the canvas. Next, it fills these according to their generated value. More colors usually make this artwork more interesting.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/stripes/2021-08-23.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/stripes/2021-08-24.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/stripes/2021-08-25.png' width='30%'>
</p>

You can use the `canvas_stripes()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_stripes(colors = colorPalette("random", n = 10))
# see ?canvas_stripes for more input parameters of this function
```

#### Paint strokes

When you think of the act of painting, you probably imagine stroking paint on a canvas. This type of artwork tries to mimic that activity. The algorithm is based on the simple idea that each next point on a grid-based canvas has a chance to take over the color of an adjacent colored point, but also has a minor chance of generating a new color. Going over the canvas like this results in something that looks like strokes of paint.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/strokes/2021-03-21.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/strokes/2021-03-20.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/strokes/2021-03-10.png' width='30%'>
</p>

You can use the `canvas_strokes()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_strokes(colors = colorPalette("tuscany1"))
# see ?canvas_strokes for more input parameters of this function
```

#### Swirls

In this artwork, inspired by Matt Deslauriers' blog post on [generative art with Node.js and canvas](https://mattdesl.svbtle.com/generative-art-with-nodejs-and-canvas), a particle system takes center stage, giving birth to enchanting swirling lines. The individual particles in the system are subjected to velocities in two directions, dictating the paths they traverse while shaping the swirls on the canvas.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/swirls/2023-05-18.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/swirls/2023-05-20.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/swirls/2023-05-19.png' width='30%'>
</p>

You can use the `canvas_swirls()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_swirls(colors = colorPalette("tuscany1"))
# see ?canvas_swirls for more input parameters of this function
```

#### Portuguese tiles

This function attempts to recreate the intricate designs on Portuguese-style decorative tiles. The artwork uses a reaction-diffusion algorithm, which simulates the process of chemicals reacting and diffusing on a surface (excellently described in these three blogposts: [link](https://itp.uni-frankfurt.de/~gros/StudentProjects/Projects_2020/projekt_schulz_kaefer/), [link](https://www.karlsims.com/rd.html) and [link](https://fronkonstin.com/2019/12/28/reaction-diffusion/)), to generate an initial quarter of each tile. Next, the initial quarter tile is mirrored twice to get the symmetric look that is reminiscent of traditional Portuguese tiles. You can fill the wall with as many unique tiles as you want and specify a custom layout.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/tiles/2023-04-29.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/tiles/2023-04-28.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/tiles/2023-04-27.png' width='30%'>
</p>

You can use the `canvas_tiles()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_tiles(colors = colorPalette("azul"))
# see ?canvas_tiles for more input parameters of this function
```

#### Turmite

According to [Wikipedia](https://en.wikipedia.org/wiki/Turmite), a turmite is *"a Turing machine which has an orientation in addition to a current state and a "tape" that consists of an infinite two-dimensional grid of cells"*. The classic algorithm consists of repeating the following three simple steps:

1. Turn on the spot (left, right, up, or down),
2. Change the color of the block,
3. Move forward one block.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/turmites/2021-03-06.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/turmites/2021-03-09.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/turmites/2021-03-08.png' width='30%'>
</p>

You can use the `canvas_turmite()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_turmite(colors = colorPalette("dark2"))
# see ?canvas_turmite for more input parameters of this function
```

#### Watercolors

This artwork implements a version of the algorithm described in the blog post [A Guide to Simulating Watercolor Paint with Generative Art](https://tylerxhobbs.com/essays/2017/a-generative-approach-to-simulating-watercolor-paints) by Tyler Hobbs. It works by layering several geometric shapes and deforming each shape by repeatedly splitting its edges.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/watercolors/2021-09-23.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/watercolors/2021-09-22.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/watercolors/2021-09-21.png' width='30%'>
</p>

You can use the `canvas_watercolors()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_watercolors(colors = colorPalette("tuscany2"))
# see ?canvas_watercolors for more input parameters of this function
```

### The Geometric collection

The Geometric collection mostly implements algorithms that draw a geometric shape and apply a random color to it.

#### Diamonds

This function creates a set of diamonds on a canvas. The diamonds are filled in (or left out) using a random color assignment.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/diamonds/2021-08-06.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/diamonds/2021-08-04.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/diamonds/2021-08-05.png' width='30%'>
</p>

You can use the `canvas_diamonds()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_diamonds(colors = colorPalette("tuscany1"))
# see ?canvas_diamonds for more input parameters of this function
```

#### Functions

The idea for this type of artwork is taken over from the [`generativeart`](https://github.com/cutterkom/generativeart) package. In this algorithm, the position of every single point is calculated by a formula which has random parameters. You can supply your own formula.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/functions/2021-03-17.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/functions/2021-04-08.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/functions/2021-04-04.png' width='30%'>
</p>

You can use the `canvas_function()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_function(colors = colorPalette("tuscany1"))
# see ?canvas_function for more input parameters of this function
```

#### Polylines

This function draws many points on the canvas and connects these points into a polygon. After repeating this for all the colors, the edges of all polygons are drawn on top of the artwork.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/polylines/2021-07-22.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/polylines/2021-07-23.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/polylines/2021-07-21.png' width='30%'>
</p>

You can use the `canvas_polylines()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_polylines(colors = colorPalette("retro1"))
# see ?canvas_polylines for more input parameters of this function
```

#### Ribbons

This function creates colored ribbons with (or without) a triangle that breaks their paths. This path of the ribbon polygon is creating by picking one point on the left side of the triangle and one point on the right side at random and using these points as nodes.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/ribbons/2021-07-16.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/ribbons/2021-07-15.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/ribbons/2021-07-14.png' width='30%'>
</p>

You can use the `canvas_ribbons()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_ribbons(colors = colorPalette("retro1")
# see ?canvas_ribbons for more input parameters of this function
```

#### Segments

This type of artwork is inspired by the style of the well-known paintings by the Dutch artist [Piet Mondriaan](https://nl.wikipedia.org/wiki/Piet_Mondriaan). The position and direction of each line segment is determined randomly.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/segments/2021-08-07.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/segments/2021-08-08.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/segments/2021-08-06.png' width='30%'>
</p>

You can use the `canvas_segments()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_segments(colors = colorPalette("dark1"))
# see ?canvas_segments for more input parameters of this function
```

#### Squares and rectangles

This artwork uses a variety of squares and rectangles to fill the canvas. It works by repeatedly cutting into the canvas at random locations and coloring the area that these cuts create.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/squares/2021-03-01.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/squares/2021-02-28.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/squares/2021-02-29.png' width='30%'>
</p>

You can use the `canvas_squares()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_squares(colors = colorPalette("retro2"))
# see ?canvas_squares for more input parameters of this function
```

### The Supervised collection

The artworks in the Supervised collection are inspired by decision boundary plots in machine learning tasks. The algorithms in this collection work by generating random data points on a two dimensional surface (with either a continuous or a categorical response variable), which they then try to model using the supervised learning algorithm. Next, they try to predict the color of each pixel on the canvas.

#### Blacklights

This artwork is inspired by a supervised machine learning method called support vector machines. It applies the principle as described above using a continuous response variable to fill the color of the pixels.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/blacklights/2021-08-22.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/blacklights/2021-08-21.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/blacklights/2021-08-20.png' width='30%'>
</p>

You can use the `canvas_blacklight()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_blacklight(colors = colorPalette("random", n = 5))
# see ?canvas_blacklight for more input parameters of this function
```

#### Forests

This artwork is inspired by a supervised learning method called random forest. It applies the principle as described above using a continuous response variable to fill the color of the pixels.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/forests/2021-08-20.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/forests/2021-08-21.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/forests/2021-08-19.png' width='30%'>
</p>

You can use the `canvas_forest()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_forest(colors = colorPalette("jungle"))
# see ?canvas_forest for more input parameters of this function
```

#### Gemstones

This artwork is inspired by a supervised learning method called k-nearest neighbors. It applies the principle as described above using a continuous response variable to fill the color of the pixels. In short, the k-nearest neighbors algorithm computes the distance of each pixel on the canvas to each randomly generated data point and assigns it the color of the value of that data point.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/gemstones/2021-08-20.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/gemstones/2021-08-21.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/gemstones/2021-08-22.png' width='30%'>
</p>

You can use the `canvas_gemstone()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_gemstone(colors = colorPalette("dark3"))
# see ?canvas_gemstone for more input parameters of this function
```

#### Mosaics

This artwork also uses a k-nearest neighbors method but instead of a continuous response variable a categorical one is used, making it a classification problem. If you considers fewer neighbors the artwork looks like a mosaic, while higher values make the artwork look more smooth.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mosaics/2021-08-17.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mosaics/2021-08-19.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mosaics/2021-08-18.png' width='30%'>
</p>

You can use the `canvas_mosaic()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_mosaic(colors = colorPalette("retro2"))
# see ?canvas_mosaic for more input parameters of this function
```

#### Nebula

Based on the very same principle as described in the artwork above is this next type of artwork. However, it produces slightly different pictures as it uses different code to create a form of k-nearest neighbors noise. Some of these artworks can resemble nebulas in outer space.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/nebulas/2021-08-29.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/nebulas/2021-08-28.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/nebulas/2021-08-27.png' width='30%'>
</p>

You can use the `canvas_nebula()` function to make your own artwork using this algorithm.

```r
set.seed(1)
canvas_nebula(colors = colorPalette("tuscany1"))
# see ?canvas_nebula for more input parameters of this function
```

### The Static collection

The Static collection implements static images that produce nice pictures.

#### Circle maps

This type of artwork is based on the concept of an [Arnold tongue](https://en.wikipedia.org/wiki/Arnold_tongue). According to Wikipedia, Arnold tongues *"are a pictorial phenomenon that occur when visualizing how the rotation number of a dynamical system, or other related invariant property thereof, changes according to two or more of its parameters"*.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/circlemaps/2021-04-22b.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/circlemaps/2021-04-22c.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/circlemaps/2021-04-22a.png' width='30%'>
</p>

You can use the `canvas_circlemap()` function to make your own artwork using this algorithm.

```r
canvas_circlemap(colors = colorPalette("dark2"))
# see ?canvas_circlemap for more input parameters of this function
```

#### The Mandelbrot set

This type of artwork visualizes the [Mandelbrot set](https://en.wikipedia.org/wiki/Mandelbrot_set) and other related fractals (e.g., the [Julia set](https://en.wikipedia.org/wiki/Julia_set), the [Multibrot set](https://en.wikipedia.org/wiki/Multibrot_set) and the [Burning ship fractal](https://en.wikipedia.org/wiki/Burning_Ship_fractal)). These fractals are well-known examples of a complex structure arising from the application of a simple rule set. You can zoom in on the set and apply some color to create these images below.

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mandelbrots/2021-08-07.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mandelbrots/2021-08-09.png' width='30%'>
  <img src='https://github.com/koenderks/aRtsy/raw/development/png/mandelbrots/2021-08-08.png' width='30%'>
</p>

You can use the `canvas_mandelbrot()` function to make your own artwork using this algorithm.

```r
canvas_mandelbrot(colors = colorPalette("tuscany1"))
# see ?canvas_mandelbrot for more input parameters of this function
```

## Color palettes

The function `colorPalette()` can be used to generate a (semi-)random color palette, or pick a pre-implemented color palette. Currently, the color palettes displayed below are implemented in `aRtsy`. Feel free to suggest or add a new palette by making an [issue](https://github.com/koenderks/aRtsy/issues) on GitHub!

<p align="center">
  <img src='https://github.com/koenderks/aRtsy/raw/development/man/figures/colors.svg' width='90%'>
</p>

## Contributing to the `aRtsy` package

Contributions to the `aRtsy` package are very much appreciated and you are free to suggest or add your own algorithms! If you want to add your own artwork to the package so that others can create unique versions of it, feel free to make a pull request to the [GitHub repository](https://github.com/koenderks/aRtsy). Don't forget to also adjust [generate-artwork.R](https://github.com/koenderks/aRtsy/blob/development/.github/workflows/generate_artwork.R) if you want the artwork to show up in the 'Artwork of the day' category and the twitter feed.

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Recaman's Sequence
#'
#' @description This function draws Recaman's sequence on a canvas. The algorithm takes increasingly large steps backward on the positive number line, but if it is unable to it takes a step forward.
#'
#' @usage canvas_recaman(
#'   colors,
#'   background = "#fafafa",
#'   iterations = 100,
#'   start = 0,
#'   increment = 1,
#'   curvature = 1,
#'   angle = 0,
#'   size = 0.1,
#'   closed = FALSE
#' )
#'
#' @param colors         a string or character vector specifying the color(s) used for the artwork.
#' @param background     a character specifying the color used for the background.
#' @param iterations     the number of iterations of the algorithm.
#' @param start          the starting point of the algorithm.
#' @param increment      the increment of each step.
#' @param curvature      the curvature of each line.
#' @param angle          the angle at which to place the artwork.
#' @param size           the size of the lines.
#' @param closed         logical. Whether to plot a curve from the end of the sequence back to the starting point.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @references \url{https://mathworld.wolfram.com/RecamansSequence.html}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_recaman(colors = colorPalette("tuscany1"))
#' }
#'
#' @export

canvas_recaman <- function(colors,
                           background = "#fafafa",
                           iterations = 100,
                           start = 0,
                           increment = 1,
                           curvature = 1,
                           angle = 0,
                           size = 0.1,
                           closed = FALSE) {
  .checkUserInput(background = background, iterations = iterations)
  if (!(angle %in% c(0, 45))) {
    stop("'angle' must be either 0 or 45 degrees")
  }
  iterations <- max(2, iterations)
  if (start < 0) {
    stop("'start' must be a single number >= 0")
  }
  if (increment < 1) {
    stop("'increment' must be a single integer >= 0")
  }
  x <- cpp_recaman(iterations, start, increment)
  if (closed) {
    xend <- c(x[-1], x[1])
  } else {
    xend <- x[-1]
    x <- x[-length(x)]
  }
  canvas <- data.frame(z = seq_along(x), x = x, xend = xend)
  canvas[["y"]] <- x * sin(angle %% 360)
  canvas[["yend"]] <- canvas[["y"]][match(canvas[["xend"]], canvas[["x"]])]
  minx <- apply(canvas[, 2:3], 1, min, na.rm = TRUE)
  maxx <- apply(canvas[, 2:3], 1, max, na.rm = TRUE)
  miny <- apply(canvas[, 4:5], 1, min, na.rm = TRUE)
  maxy <- apply(canvas[, 4:5], 1, max, na.rm = TRUE)
  for (i in seq_len(nrow(canvas))) {
    if (i %% 2 == 1) {
      canvas[i, 2:5] <- c(minx[i], maxx[i], miny[i], maxy[i])
    } else {
      canvas[i, 2:5] <- c(maxx[i], minx[i], maxy[i], miny[i])
    }
  }
  if (!closed && angle == 45) {
    canvas <- canvas[-nrow(canvas), ]
  }
  artwork <- ggplot2::ggplot() +
    ggplot2::geom_curve(
      mapping = ggplot2::aes(x = x, y = y, xend = xend, yend = yend, color = z),
      data = canvas, curvature = curvature, ncp = 25, linewidth = size
    ) +
    ggplot2::scale_color_gradientn(colors = colors)
  artwork <- theme_canvas(artwork, background = background)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

test_that("colorPalette()", {
  set.seed(1)
  .runtime(expression = {
    palette <- colorPalette(name = "random", n = 4)
  }, name = "Palettes")
  expect_equal(length(palette), 4)
})

test_that("canvas_strokes()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_strokes(colors = c("black", "white"))
  }, name = "Strokes")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_ribbons()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_ribbons(colors = c("black", "white"))
  }, name = "Ribbons")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_turmite()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_turmite(color = "black")
  }, name = "Turmite")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_ant()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_ant(colors = "black")
  }, name = "Ant")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_planet()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_planet(colors = c("black", "white"))
  }, name = "Planet")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_squares()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_squares(colors = c("black", "white"))
  }, name = "Squares")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_circlemap()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_circlemap(colors = c("black", "white"))
  }, name = "Circlemap")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_function()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_function(color = "black")
  }, name = "Function")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_polylines()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_polylines(colors = c("black", "gray", "white"))
  }, name = "Polylines")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_diamonds()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_diamonds(colors = c("black", "gray", "white"))
  }, name = "Diamonds")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_segments()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_segments(colors = c("black", "gray", "white"))
  }, name = "Segments")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_mandelbrot()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_mandelbrot(colors = c("black", "gray", "white"))
  }, name = "Mandelbrot")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_collatz()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_collatz(colors = c("black", "gray", "white"))
  }, name = "Collatz")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_mosaic()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_mosaic(colors = c("black", "gray", "white"))
  }, name = "Mosaic")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_forest()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_forest(colors = c("black", "gray", "white"))
  }, name = "Forest")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_gemstone()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_gemstone(colors = c("black", "gray", "white"))
  }, name = "Gemstone")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_blacklight()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_blacklight(colors = c("black", "gray", "white"))
  }, name = "Blacklight")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_stripes()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_stripes(colors = c("black", "gray", "white"))
  }, name = "Stripes")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_nebula()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_nebula(colors = c("black", "gray", "white"))
  }, name = "Nebula")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_watercolors()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_watercolors(colors = c("black", "gray", "white"))
  }, name = "Watercolors")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_flow()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_flow(colors = c("black", "gray", "white"))
  }, name = "Flow")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_maze()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_maze(color = "black")
  }, name = "Maze")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_recaman()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_recaman(colors = "black")
  }, name = "Recaman")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_phyllotaxis()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_phyllotaxis(colors = "black")
  }, name = "Phyllotaxis")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_cobweb()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_cobweb(colors = "black")
  }, name = "Cobweb")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_chladni()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_chladni(colors = "black")
  }, name = "Chladni")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_petri()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_petri(colors = "black")
  }, name = "Petri")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_splits()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_splits(colors = "black")
  }, name = "Splits")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_mesh()", {
  set.seed(1)
  .runtime(expression = {
    artwork <- canvas_mesh(colors = "black")
  }, name = "Mesh")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_flame()", {
  set.seed(2)
  .runtime(expression = {
    artwork <- canvas_flame(colors = c("dodgerblue", "green"))
  }, name = "Flame")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_smoke()", {
  set.seed(2)
  .runtime(expression = {
    artwork <- canvas_smoke(colors = c("red", "blue", "green"), resolution = 10)
  }, name = "Smoke")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_tiles()", {
  set.seed(2)
  .runtime(expression = {
    artwork <- canvas_tiles(colors = c("red", "blue", "green"), iterations = 100)
  }, name = "Tiles")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_swirls()", {
  set.seed(2)
  .runtime(expression = {
    artwork <- canvas_swirls(colors = c("red", "blue", "green"), n = 10, iterations = 10)
  }, name = "Swirls")
  expect_equal(!is.null(artwork), TRUE)
})

test_that("canvas_lissajous()", {
  set.seed(2)
  .runtime(expression = {
    artwork <- canvas_lissajous(colors = c("red", "blue", "green"))
  }, name = "Lissajous")
  expect_equal(!is.null(artwork), TRUE)
})

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Planets
#'
#' @description This function paints one or multiple planets and uses a cellular automata to fill their surfaces.
#'
#' @usage canvas_planet(
#'   colors,
#'   threshold = 4,
#'   iterations = 200,
#'   starprob = 0.01,
#'   fade = 0.2,
#'   radius = NULL,
#'   center.x = NULL,
#'   center.y = NULL,
#'   light.right = TRUE,
#'   resolution = 1500
#' )
#'
#' @param colors      a character specifying the colors used for a single planet. Can also be a list where each entry is a vector of colors for a planet.
#' @param threshold   a character specifying the threshold for a color take.
#' @param starprob    a value specifying the probability of drawing a star in outer space.
#' @param fade        a value specifying the amount of fading to apply.
#' @param iterations  a positive integer specifying the number of iterations of the algorithm.
#' @param radius      a numeric (vector) specifying the radius of the planet(s).
#' @param center.x    the x-axis coordinate(s) for the center(s) of the planet(s).
#' @param center.y    the y-axis coordinate(s) for the center(s) of the planet(s).
#' @param light.right whether to draw the light from the right or the left.
#' @param resolution  resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.
#'
#' @references \url{https://fronkonstin.com/2021/01/02/neighborhoods-experimenting-with-cyclic-cellular-automata/}
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_planet(colors = colorPalette("lava"), threshold = 3)
#'
#' # Advanced example
#' colors <- list(
#'   c("khaki1", "lightcoral", "lightsalmon"),
#'   c("dodgerblue", "forestgreen", "white"),
#'   c("gray", "darkgray", "beige")
#' )
#' canvas_planet(colors,
#'   radius = c(800, 400, 150),
#'   center.x = c(1, 500, 1100),
#'   center.y = c(1400, 500, 1000),
#'   starprob = 0.005
#' )
#' }
#'
#' @keywords artwork canvas
#'
#' @export

canvas_planet <- function(colors,
                          threshold = 4,
                          iterations = 200,
                          starprob = 0.01,
                          fade = 0.2,
                          radius = NULL,
                          center.x = NULL,
                          center.y = NULL,
                          light.right = TRUE,
                          resolution = 1500) {
  .checkUserInput(resolution = resolution, iterations = iterations)
  if (is.list(colors)) {
    palette <- list()
    for (i in seq_along(colors)) {
      palette[[i]] <- c("#000000", "#787878", "#fafafa", colors[[i]])
    }
  } else {
    palette <- list(c("#000000", "#787878", "#fafafa", colors))
    colors <- list(colors)
  }
  canvas <- matrix(0, nrow = resolution, ncol = resolution)
  if (is.null(radius)) {
    radius <- ceiling(resolution / 2 / 1.5)
  }
  if (is.null(center.x)) {
    center.x <- ceiling(resolution / 2)
  }
  if (is.null(center.y)) {
    center.y <- ceiling(resolution / 2)
  }
  if (length(unique(c(length(radius), length(center.y), length(center.x)))) != 1) {
    stop("Radius, center.y, and center.x do not have equal length.")
  }
  planets <- length(radius)
  colorsused <- 0
  for (i in 1:planets) {
    canvas <- cpp_planet(
      canvas = canvas,
      resolution = resolution,
      radius = radius[i],
      xcenter = center.x[i],
      ycenter = center.y[i],
      threshold = threshold,
      iterations = ceiling(iterations / i),
      ncolors = length(palette[[i]]),
      colorsused = colorsused,
      starprob = starprob,
      fade = fade,
      lightright = light.right
    )
    colorsused <- colorsused + length(colors[[i]])
  }
  full_canvas <- .unraster(canvas, names = c("y", "x", "z")) # Convert 2D matrix to data frame
  full_palette <- c("#000000", "#787878", "#fafafa", unlist(colors))
  artwork <- ggplot2::ggplot(data = full_canvas, ggplot2::aes(x = x, y = y, fill = z)) +
    ggplot2::geom_raster(interpolate = TRUE, alpha = 0.9) +
    ggplot2::xlim(c(0, resolution + 1)) +
    ggplot2::ylim(c(0, resolution + 1)) +
    ggplot2::scale_fill_gradientn(colours = full_palette)
  artwork <- theme_canvas(artwork)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Canvas Theme for ggplot2 Objects
#'
#' @description Add a canvas theme to the plot. The canvas theme by default has no margins and fills any empty canvas with a background color.
#'
#' @usage theme_canvas(x, background = NULL, margin = 0)
#'
#' @param x            a ggplot2 object.
#' @param background   a character specifying the color used for the empty canvas.
#' @param margin       margins of the canvas.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords canvas theme
#'
#' @export

theme_canvas <- function(x, background = NULL, margin = 0) {
  x <- x + ggplot2::theme(
    axis.title = ggplot2::element_blank(),
    axis.text = ggplot2::element_blank(),
    axis.ticks = ggplot2::element_blank(),
    axis.line = ggplot2::element_blank(),
    legend.position = "none",
    plot.background = ggplot2::element_rect(fill = background, colour = background),
    panel.border = ggplot2::element_blank(),
    panel.grid = ggplot2::element_blank(),
    plot.margin = ggplot2::unit(rep(margin, 4), "lines"),
    strip.background = ggplot2::element_blank(),
    strip.text = ggplot2::element_blank()
  )
  if (is.null(background)) {
    x <- x + ggplot2::theme(panel.background = ggplot2::element_rect(fill = NA, colour = NA))
  } else {
    x <- x + ggplot2::theme(panel.background = ggplot2::element_rect(fill = background, colour = background))
  }
  return(x)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw a Lissajous Curve
#'
#' @description This function draws lissajous curves with points connected via a k-nearest neighbor approach.
#'
#' @usage canvas_lissajous(
#'   colors,
#'   background = "#000000",
#'   iterations = 2,
#'   neighbors = 50,
#'   noise = FALSE
#' )
#'
#' @param colors      a string or character vector specifying the color(s) used for the artwork.
#' @param background  a character specifying the color used for the background.
#' @param iterations  a positive integer specifying the number of iterations of the algorithm.
#' @param neighbors   a positive integer specifying the number of neighbors a block considers when drawing the connections.
#' @param noise       logical. Whether to add perlin noise to the coordinates of the nodes.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @references \url{https://en.wikipedia.org/wiki/Lissajous_curve}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(13)
#'
#' # Simple example
#' canvas_lissajous(colors = colorPalette("blossom"))
#' }
#'
#' @export

canvas_lissajous <- function(colors,
                             background = "#000000",
                             iterations = 2,
                             neighbors = 50,
                             noise = FALSE) {
  .checkUserInput(iterations = iterations, background = background)
  stopifnot("'neighbors' must be > 0" = neighbors > 0)
  d <- stats::runif(1, 1, 100)
  e <- stats::runif(1, 1, 100)
  a <- stats::runif(1, 1, 10)
  b <- stats::runif(1, 1, 10)
  c <- stats::runif(1, 1, 10)
  A <- stats::runif(1, 1, 10)
  B <- stats::runif(1, 1, 10)
  C <- stats::runif(1, 1, 10)
  npoints <- 1000
  t <- seq(from = 0, to = iterations * pi, length.out = npoints)
  x <- A * sin(a * t + d)
  y <- B * sin(b * t)
  z <- C * sin(c * t + e)
  if (noise) {
    x <- x + as.numeric(.noise(dims = c(1, npoints), type = "perlin", limits = c(-1, 1)))
    y <- y + as.numeric(.noise(dims = c(1, npoints), type = "perlin", limits = c(-1, 1)))
    z <- z + as.numeric(.noise(dims = c(1, npoints), type = "perlin", limits = c(-1, 1)))
  }
  df <- data.frame(t = t, x = x, y = y, z = z)
  n <- nrow(df)
  all_neighbors <- lapply(1:n, function(ptnum) {
    xi <- df$x[ptnum]
    yi <- df$y[ptnum]
    zi <- df$z[ptnum]
    dists <- sqrt((df$x - xi)^2 + (df$y - yi)^2 + (df$z - zi)^2)
    order <- order(dists)
    nn <- order[2:(neighbors + 1)]
    xend <- rep(xi, neighbors)
    yend <- rep(yi, neighbors)
    dftmp <- data.frame(
      x = df$x[nn],
      y = df$y[nn],
      xend = xend,
      yend = yend,
      z = df$z[nn]
    )
    return(dftmp)
  })
  canvas <- do.call(rbind, all_neighbors)
  breaks <- pretty(c(canvas$x, canvas$y, canvas$xend, canvas$yend))
  if (length(colors) == 1) {
    cols <- colors
  } else {
    cols <- colors[cut(canvas$z, breaks = length(colors))]
  }
  alphas <- 1 - scales::rescale(canvas$z, to = c(0.5, 0.8), from = range(canvas$z))
  artwork <- ggplot2::ggplot(data = canvas, mapping = ggplot2::aes(x = x, y = y, xend = xend, yend = yend)) +
    ggplot2::geom_segment(linewidth = 0.01, colour = cols, alpha = alphas) +
    ggplot2::scale_x_continuous(limits = range(breaks)) +
    ggplot2::scale_y_continuous(limits = range(breaks))
  artwork <- theme_canvas(artwork, background = background)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Cobwebs
#'
#' @description This function draws a cobweb on the canvas. The cobweb consists
#' of many Fibonacci spirals shifted by random noise from a normal distribution.
#' A Fibonacci spiral is a logarithmic spiral that is derived from the
#' Fibonacci sequence, a mathematical sequence where each number is the sum of
#' the two preceding ones. The spiral is created by connecting the corners of
#' squares that are sized according to the Fibonacci sequence. Specifically, if
#' we draw a sequence of squares with side lengths of 1, 1, 2, 3, 5, 8, 13, and
#' so on, each square can be arranged so that it is tangent to the previous
#' square at a corner. When we connect these corners with a smooth curve, the
#' resulting shape is the Fibonacci spiral.
#'
#' @usage canvas_cobweb(
#'   colors,
#'   background = "#fafafa",
#'   lines = 300,
#'   iterations = 100
#' )
#'
#' @param colors      a string or character vector specifying the color(s) used for the artwork.
#' @param background  a character specifying the color used for the background.
#' @param lines       the number of lines to draw.
#' @param iterations  the number of iterations of the algorithm.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_cobweb(colors = colorPalette("neon1"), background = "black")
#' }
#'
#' @export

canvas_cobweb <- function(colors,
                          background = "#fafafa",
                          lines = 300,
                          iterations = 100) {
  .checkUserInput(background = background, iterations = iterations)
  fibonacci <- .cobweb_fibonacci(n = iterations)
  seqn <- seq_len(iterations)
  nrows <- length(seqn - 3) * lines
  canvas <- data.frame(x = rep(NA, nrows), xend = rep(NA, nrows), y = rep(NA, nrows), yend = rep(NA, nrows), z = rep(NA, nrows), lwd = rep(NA, nrows), col = rep(NA, nrows))
  for (i in 1:lines) {
    x1 <- ifelse(seqn %% 2 == 1, yes = fibonacci, no = 0)
    y1 <- ifelse(seqn %% 2 == 0, yes = fibonacci, no = 0)
    x2 <- ifelse(seqn %% 4 == 1, yes = x1, no = -x1)
    y2 <- ifelse(seqn %% 4 == 2, yes = y1, no = -y1)
    suppressWarnings({
      x <- ifelse(x2 > 0, log(x2), ifelse(x2 < 0, -log(-x2), 0)) * stats::rnorm(length(x2))
      y <- ifelse(y2 > 0, log(y2), ifelse(y2 < 0, -log(-y2), 0)) * stats::rnorm(length(x2))
    })
    xend <- c(x[-1], x[1])
    yend <- c(y[-1], y[1])
    subdata <- data.frame(x = x, xend = xend, y = y, yend = yend, z = i, lwd = stats::runif(1, 0, 0.1), col = sample(colors, size = 1))
    subdata <- subdata[-which(subdata[["x"]] == subdata[["xend"]] | subdata[["y"]] == subdata[["yend"]]), ]
    ind <- which(is.na(canvas[["x"]]))[1]
    canvas[ind:(ind + nrow(subdata) - 1), ] <- subdata[stats::complete.cases(subdata), ]
  }
  canvas <- canvas[stats::complete.cases(canvas), ]
  artwork <- ggplot2::ggplot() +
    ggplot2::geom_curve(
      data = canvas, mapping = ggplot2::aes(x = x, y = y, xend = xend, yend = yend, group = z),
      color = canvas[["col"]], curvature = stats::runif(1, 0, 0.8), ncp = 25, linewidth = canvas[["lwd"]], alpha = 0.01
    ) +
    ggplot2::coord_cartesian(
      xlim = c(min(canvas[["x"]]) / 4, max(canvas[["x"]]) / 4),
      ylim = c(min(canvas[["y"]]) / 4, max(canvas[["y"]]) / 4)
    )
  artwork <- theme_canvas(artwork, background = background)
  return(artwork)
}

.cobweb_fibonacci <- function(n) {
  x <- c(0, 1)
  for (i in 3:n) {
    x[i] <- x[i - 1] + x[i - 2]
  }
  x <- x[-1]
  return(x)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Squares and Rectangles
#'
#' @description This function paints random squares and rectangles. It works by repeatedly cutting into the canvas at random locations and coloring the area that these cuts create.
#'
#' @usage canvas_squares(
#'   colors,
#'   background = "#000000",
#'   cuts = 50,
#'   ratio = 1.618,
#'   resolution = 200,
#'   noise = FALSE
#' )
#'
#' @param colors      a string or character vector specifying the color(s) used for the artwork.
#' @param background  a character specifying the color used for the borders of the squares.
#' @param cuts        a positive integer specifying the number of cuts to make.
#' @param ratio       a value specifying the \code{1:1} ratio for each cut.
#' @param resolution  resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.
#' @param noise       logical. Whether to add k-nn noise to the artwork. Note that adding noise increases computation time significantly in large dimensions.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_squares(colors = colorPalette("retro2"))
#' }
#'
#' @export

canvas_squares <- function(colors,
                           background = "#000000",
                           cuts = 50,
                           ratio = 1.618,
                           resolution = 200,
                           noise = FALSE) {
  .checkUserInput(resolution = resolution, background = background)
  stopifnot("'cuts' must be a single value > 0" = cuts > 0)
  palette <- c(background, colors)
  neighbors <- expand.grid(-1:1, -1:1)
  colnames(neighbors) <- c("x", "y")
  canvas <- matrix(0, nrow = resolution, ncol = resolution)
  full_canvas <- cpp_squares(canvas, neighbors, length(colors), cuts, ratio)
  if (noise) {
    full_canvas <- full_canvas - .noise(dims = c(resolution, resolution))
  }
  full_canvas <- .unraster(full_canvas, names = c("x", "y", "z"))
  artwork <- ggplot2::ggplot(data = full_canvas, ggplot2::aes(x = x, y = y, fill = z)) +
    ggplot2::geom_raster(interpolate = FALSE, alpha = 1) +
    ggplot2::xlim(c(0, resolution + 1)) +
    ggplot2::ylim(c(0, resolution + 1)) +
    ggplot2::scale_fill_gradientn(colours = palette)
  artwork <- theme_canvas(artwork)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Collatz Sequences
#'
#' @description This function draws the Collatz conjecture on a canvas. The
#'   conjecture of the Collatz sequence is that no matter what positive integer
#'   is chosen as the starting point of the sequence, the sequence will
#'   eventually reach the number 1. This conjecture has been verified for all
#'   starting integers up to very large numbers, but it has not been proven
#'   mathematically. Despite its simple rule, the sequence can produce long and
#'   complicated chains of numbers before eventually reaching 1. See the
#'   \code{Details} section for more specific information about the algorithm
#'   used in this function.
#'
#' @usage canvas_collatz(
#'   colors,
#'   background = "#fafafa",
#'   n = 200,
#'   angle.even = 0.0075,
#'   angle.odd = 0.0145,
#'   side = FALSE
#' )
#'
#' @param colors     a string or character vector specifying the color(s) used for the artwork.
#' @param background a character specifying the color used for the background.
#' @param n          a positive integer specifying the number of random starting integers to use for the lines. Can also be a vector of numbers to use as starting numbers.
#' @param angle.even a value specifying the angle (in radials) to use in bending the sequence at each odd number.
#' @param angle.odd  a value specifying the angle (in radials) to use in bending the sequence at each even number.
#' @param side       logical. Whether to put the artwork on its side.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @details The Collatz sequence, also known as the \code{3n+1} problem, is a
#'   sequence of numbers generated by the following rule:
#'
#' \itemize{
#'   \item{Start with any positive integer \code{n}.}
#'   \item{If \code{n} is even, divide it by 2.}
#'   \item{If \code{n} is odd, multiply it by 3 and add 1.}
#'   \item{Repeat this process with the new value of \code{n}, generating a new
#'     number in the sequence.}
#' }
#'
#' @references \url{https://nl.wikipedia.org/wiki/Collatz_Conjecture}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_collatz(colors = colorPalette("tuscany3"))
#' }
#'
#' @export

canvas_collatz <- function(colors,
                           background = "#fafafa",
                           n = 200,
                           angle.even = 0.0075,
                           angle.odd = 0.0145,
                           side = FALSE) {
  .checkUserInput(background = background)
  canvas <- data.frame(x = numeric(), y = numeric(), col = numeric(), z = numeric())
  if (length(n) == 1) {
    n <- sample.int(1000000, size = n, replace = FALSE)
  }
  for (i in n) {
    series <- rev(cpp_collatz_sequence(i))
    line <- matrix(0, nrow = length(series), ncol = 2)
    line <- cpp_collatz(line, series, angle.even, angle.odd)
    line <- data.frame(
      x = line[, 1],
      y = line[, 2],
      col = rep(sample(colors, size = 1), nrow(line)),
      z = i,
      size = nrow(line),
      alpha = nrow(line)
    )
    canvas <- rbind(canvas, line)
  }
  canvas[["z"]] <- as.factor(canvas[["z"]])
  canvas[["size"]] <- 1 - (canvas[["size"]] / max(canvas[["size"]]))
  canvas[["alpha"]] <- 1 - canvas[["size"]]
  artwork <- ggplot2::ggplot(data = canvas, mapping = ggplot2::aes(x = x, y = y, group = z)) +
    ggplot2::geom_path(linewidth = canvas[["size"]], color = canvas[["col"]], alpha = canvas[["alpha"]], lineend = "round") +
    ggplot2::xlim(range(canvas[["x"]])) +
    ggplot2::ylim(range(canvas[["y"]]))
  if (side) {
    artwork <- artwork + ggplot2::coord_flip()
  }
  artwork <- theme_canvas(artwork, background)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Petri Dish Colonies
#'
#' @description This function uses a space colony algorithm to draw Petri dish colonies.
#'
#' @usage canvas_petri(
#'   colors,
#'   background = "#fafafa",
#'   dish = "black",
#'   attractors = 1000,
#'   iterations = 15,
#'   hole = 0
#' )
#'
#' @param colors         a string or character vector specifying the color(s) used for the artwork.
#' @param background     a character specifying the color used for the background (and the hole).
#' @param dish           a character specifying the color used for the Petri dish.
#' @param attractors     an integer specifying the number of attractors.
#' @param iterations     a positive integer specifying the number of iterations of the algorithm.
#' @param hole           a value between 0 and 0.9 specifying the hole size in proportion to the dish.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @references \url{https://medium.com/@jason.webb/space-colonization-algorithm-in-javascript-6f683b743dc5}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(2)
#'
#' # Simple example
#' canvas_petri(colors = colorPalette("origami"))
#'
#' # Advanced example
#' canvas_petri(colors = "white", hole = 0.8, attractors = 5000)
#' }
#'
#' @export

canvas_petri <- function(colors,
                         background = "#fafafa",
                         dish = "black",
                         attractors = 1000,
                         iterations = 15,
                         hole = 0) {
  .checkUserInput(iterations = iterations)
  if (hole < 0 || hole > 0.9 || length(hole) != 1 || !is.numeric(hole)) {
    stop("'hole' must be a single value > 0 and < 0.9")
  }
  attraction_distance <- pi * (1 + hole)
  kill_distance <- attraction_distance / 50
  nodes <- length(colors)
  r <- pi * sqrt(stats::runif(attractors, min = hole, max = 1))
  theta <- stats::runif(attractors) * 2 * pi
  attractor_data <- data.frame(x = r * cos(theta), y = r * sin(theta))
  r <- pi * sqrt(stats::runif(nodes, min = hole * 1.01, max = 0.99))
  theta <- stats::runif(nodes) * 2 * pi
  node_data <- data.frame(x = r * cos(theta), y = r * sin(theta), z = 1:nodes, t = 0)
  node_data[["xend"]] <- node_data[["x"]] # Node x-location
  node_data[["yend"]] <- node_data[["y"]] # Node y-location
  for (i in 1:iterations) {
    closest_nodes <- cpp_petri_closest(attractor_data[["x"]], attractor_data[["y"]], node_data[["xend"]], node_data[["yend"]], attraction_distance)
    if (all(closest_nodes == 0)) {
      break
    }
    directions <- cpp_petri_directions(attractor_data[["x"]], attractor_data[["y"]], node_data[["xend"]], node_data[["yend"]], closest_nodes)
    directions[["xend"]] <- (directions[["xend"]] / sqrt(sum(directions[["xend"]]^2, na.rm = TRUE))) / 1.5
    directions[["yend"]] <- (directions[["yend"]] / sqrt(sum(directions[["yend"]]^2, na.rm = TRUE))) / 1.5
    new_nodes <- data.frame(
      x = node_data[["xend"]], y = node_data[["yend"]],
      z = node_data[["z"]], t = i,
      xend = node_data[["xend"]] + directions[["xend"]],
      yend = node_data[["yend"]] + directions[["yend"]]
    )
    new_nodes <- new_nodes[stats::complete.cases(new_nodes), ]
    node_data[nrow(node_data) + seq_len(nrow(new_nodes)), ] <- new_nodes
    attractor_data <- cpp_petri_kill(attractor_data[["x"]], attractor_data[["y"]], node_data[["x"]], node_data[["y"]], kill_distance)
    if (nrow(attractor_data) < 1) {
      break
    }
  }
  circle_points <- cpp_draw_circle(center_x = 0, center_y = 0, diameter = 2 * pi, n = 100)
  hole_points <- cpp_draw_circle(center_x = 0, center_y = 0, diameter = 2 * pi * hole, n = 100)
  limits <- range(pretty(c(node_data[["x"]], node_data[["xend"]], node_data[["y"]], node_data[["yend"]], circle_points[["x"]], circle_points[["y"]])))
  node_data[["size"]] <- (max(node_data[["t"]]) - node_data[["t"]]) / max(node_data[["t"]])
  artwork <- ggplot2::ggplot(data = node_data, mapping = ggplot2::aes(x = x, y = y, xend = xend, yend = yend, group = factor(z))) +
    ggplot2::geom_polygon(data = circle_points, mapping = ggplot2::aes(x = x, y = y), inherit.aes = FALSE, fill = dish) +
    ggplot2::geom_polygon(data = hole_points, mapping = ggplot2::aes(x = x, y = y), inherit.aes = FALSE, fill = background) +
    ggplot2::geom_segment(mapping = ggplot2::aes(color = factor(z)), linewidth = node_data[["size"]], linejoin = "round", lineend = "round") +
    ggplot2::scale_color_manual(values = colors) +
    ggplot2::coord_equal(xlim = limits, ylim = limits)
  artwork <- theme_canvas(artwork, background = background)
  return(artwork)
}

# Packages required for artworks
library(aRtsy)

.tile_helper <- function(ntiles) {
  if (ntiles > 1) {
    out <- list()
    for (i in 1:ntiles) {
      proposal <- colorPalette("random-palette")
      while (list(proposal) %in% out) {
        proposal <- colorPalette("random-palette")
      }
      out[[i]] <- proposal
    }
  } else {
    out <- colorPalette("random-palette")
  }
  return(out)
}

# Name of the artwork
filename <- paste0("png/daily.png")

# Artwork seed dependent on the current date
seed <- as.numeric(Sys.Date())
set.seed(seed)

# Select artwork type
type <- sample.int(34, 1)

# Create artwork with random palette, feel free to suggest a new palette at https://github.com/koenderks/aRtsy/issues
artwork <- switch(type,
  "1" = canvas_turmite(colors = colorPalette("random-palette"), background = "#050505", p = runif(1, 0.2, 0.5), resolution = 2000, noise = TRUE, iterations = 1e7),
  "2" = canvas_strokes(colors = colorPalette("random-palette"), neighbors = sample.int(4, 1), p = runif(1, 0.0001, 0.01), iterations = sample.int(3, 1), resolution = 1500, side = sample(c(TRUE, FALSE), 1)),
  "3" = canvas_function(color = colorPalette("random-palette"), background = sample(c("#fafafa", "#1a3657", "#343434", "#cc7722", "#a9d2c3", "#fc7c7c"), 1)),
  "4" = canvas_ant(colors = colorPalette("random-palette"), background = sample(c("#fafafa", "#cc7722", "#a9d2c3", "#fc7c7c", colorPalette("random", 1)), 1), resolution = 1000, iterations = 1e7),
  "5" = canvas_squares(colors = colorPalette("random-palette"), background = "#000000", cuts = sample(10:200, 1), noise = TRUE),
  "6" = canvas_planet(colors = colorPalette("random-palette", n = 3), iterations = 30, starprob = runif(1, 0.001, 0.05)),
  "7" = canvas_forest(colors = colorPalette("random-palette"), resolution = 1500),
  "8" = canvas_circlemap(colors = colorPalette("random-palette"), left = runif(1, -14, 1), right = runif(1, 1, 14), bottom = runif(1, -2, -1), top = runif(1, 1, 2), iterations = sample.int(30, 1), resolution = 1500),
  "9" = canvas_polylines(colors = colorPalette("random-palette"), background = sample(c("#fafafa", "black", colorPalette("random", 1)), 1)),
  "10" = canvas_diamonds(colors = colorPalette("random-palette"), background = sample(c("#fafafa", "black", colorPalette("random", 1)), 1), col.line = sample(c(NA, sample(c("#fafafa", "black", colorPalette("random", 1)), 1)), 1), radius = sample(c(1, 2, 2.5, 5, 7, 7.5), 1), p = sample(seq(0.4, 0.8, 0.05), 1)),
  "11" = canvas_segments(colors = colorPalette("random-palette"), n = sample(seq(300, 700, 100), 1), H = 0.1, p = sample(seq(0.3, 0.7, 0.1), 1)),
  "12" = canvas_mandelbrot(colors = colorPalette("random-palette"), set = sample(c("mandelbrot", "multibrot", "julia"), 1), zoom = sample(seq(1, 2, 0.1), 1), resolution = 2000),
  "13" = canvas_nebula(colors = colorPalette("random-palette"), k = sample(50:100, 1), resolution = 2000),
  "14" = canvas_mosaic(colors = colorPalette("random-palette"), resolution = 1500),
  "15" = canvas_stripes(colors = colorPalette("random-palette"), burnin = sample.int(200, 1)),
  "16" = canvas_gemstone(colors = colorPalette("random-palette"), resolution = 1500),
  "17" = canvas_blacklight(colors = colorPalette("random-palette"), resolution = 1500),
  "18" = canvas_ribbons(colors = colorPalette("random-palette"), background = colorPalette("random", 1)),
  "19" = canvas_collatz(colors = colorPalette("random-palette"), background = sample(c("black", "#fdf5e6", "#fafafa"), 1), n = sample(200:2000, 1), side = sample(c(TRUE, FALSE), 1)),
  "20" = canvas_watercolors(colors = colorPalette("random-palette"), background = sample(c("#fafafa", "black", "#ebd5b3", "darkgoldenrod3", "lavenderblush2", "salmon1"), 1), layers = 50, depth = 3),
  "21" = canvas_flow(colors = colorPalette("random-palette"), background = sample(c("#fafafa", "firebrick", "#f9f0e0", "black", "lavenderblush2", "#215682"), 1), lines = sample(2000:5000, 1), lwd = sample(seq(0.05, 0.2, 0.01), 1), iterations = sample(10:500, 1), polar = sample(c(TRUE, FALSE), 1), outline = sample(c("none", "circle", "square"), 1)),
  "22" = canvas_maze(color = colorPalette("random", 1), walls = colorPalette("random", 1), background = colorPalette("random", 1), resolution = sample(50:100, 1)),
  "23" = canvas_recaman(colors = colorPalette("random-palette"), background = colorPalette("random", 1), iterations = sample(100:1000, 1), curvature = runif(1, 1, 20), start = sample.int(100, 1), angle = sample(c(0, 45), 1)),
  "24" = canvas_phyllotaxis(colors = colorPalette("random-palette"), iterations = sample(1000:100000, 1), p = runif(1, 0.5, 0.9), background = colorPalette("random", 1), angle = runif(1, 0, 1000), size = 0.01, alpha = runif(1, 0.3, 1)),
  "25" = canvas_cobweb(colors = colorPalette("random-palette"), background = colorPalette("random", 1), lines = sample(500:1000, 1), iterations = sample(50:200, 1)),
  "26" = canvas_chladni(colors = colorPalette("random-palette"), waves = sample(3:10, 1), resolution = sample(c(500, 1000), 1), warp = runif(1, 0, 1.5)),
  "27" = canvas_petri(colors = colorPalette("random-palette"), background = colorPalette("random", 1), dish = colorPalette("random", 1), attractors = 5000, iterations = sample(10:20, 1), hole = sample(c(0, 0.7, 0.8), 1)),
  "28" = canvas_splits(colors = colorPalette("random-palette"), background = colorPalette("random", 1), iterations = sample(6:8, 1), sd = abs(rnorm(1, 0, 0.5))),
  "29" = canvas_mesh(colors = colorPalette("random-palette", n = sample.int(5, 1)), background = sample(c("#fafafa", "firebrick", "#f9f0e0", "black", "lavenderblush2", "#215682"), 1)),
  "30" = canvas_flame(colors = colorPalette("random-palette"), background = sample(c("#fafafa", "firebrick", "#f9f0e0", "black", "lavenderblush2", "#215682"), 1), variations = sample(0:48, 3), symmetry = sample(-1:2, 1), iterations = 1e9, weighted = TRUE, post = sample(c(FALSE, TRUE), 1)),
  "31" = canvas_smoke(colors = if (runif(1) < 2 / 3) colorPalette(sample(c("divergent", "random", "complement"), 1), n = 1024) else "all", algorithm = sample(c("minimum", "average"), 1), init = sample.int(3, 1), shape = sample(c("bursts", "clouds"), 1), resolution = 500),
  "32" = canvas_tiles(colors = .tile_helper(sample.int(7, 1)), iterations = sample(2000:10000, 1), size = sample(4:7, 1)),
  "33" = canvas_swirls(colors = colorPalette("random-palette"), background = sample(c("black", "#fafafa", "#fc7c7c", "#cc7722", "#a9d2c3"), 1), iterations = sample(250:2000, 1), n = sample(50:2000, 1), curvature = runif(1, 0.001, 0.01), resolution = sample(100:2000, 1)),
  "34" = canvas_lissajous(colors = colorPalette("random-palette"), background = sample(c("#fafafa", "#f9f0e0", "black", "lavenderblush2", "salmon"), 1), iterations = sample(2:5, 1), noise = sample(c(TRUE, FALSE), 1))
)

saveCanvas(artwork, filename, width = ifelse(type == 19, yes = NA, no = 7), height = ifelse(type == 19, yes = NA, no = 7))

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Moisaics
#'
#' @description This function draws the predictions from a k-nearest neighbors algorithm trained on randomly generated categorical data.
#'
#' @usage canvas_mosaic(
#'   colors,
#'   n = 1000,
#'   resolution = 500
#' )
#'
#' @param colors      a string or character vector specifying the color(s) used for the artwork.
#' @param n           a positive integer specifying the number of random data points to generate.
#' @param resolution  resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @references \url{https://en.wikipedia.org/wiki/K-nearest_neighbors_algorithm}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_mosaic(colors = colorPalette("retro2"))
#' }
#'
#' @export

canvas_mosaic <- function(colors,
                          n = 1000,
                          resolution = 500) {
  .checkUserInput(resolution = resolution)
  train <- data.frame(
    x = stats::runif(n, 0, resolution),
    y = stats::runif(n, 0, resolution),
    z = factor(sample(colors, size = n, replace = TRUE))
  )
  fit <- kknn::train.kknn(formula = z ~ x + y, data = train, kmax = 1)
  sequence <- seq(0, resolution, by = 1)
  canvas <- expand.grid(sequence, sequence)
  colnames(canvas) <- c("x", "y")
  suppressWarnings({
    z <- predict(fit, newdata = canvas)
  })
  full_canvas <- data.frame(x = canvas[["x"]], y = canvas[["y"]], z = z)
  artwork <- ggplot2::ggplot(data = full_canvas, mapping = ggplot2::aes(x = x, y = y, fill = z)) +
    ggplot2::geom_raster(interpolate = TRUE) +
    ggplot2::xlim(c(-1, resolution + 1)) +
    ggplot2::ylim(c(-1, resolution + 1)) +
    ggplot2::scale_fill_manual(values = colors)
  artwork <- theme_canvas(artwork)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Turmites
#'
#' @description This function paints a turmite. A turmite is a Turing machine which has an orientation in addition to a current state and a "tape" that consists of a two-dimensional grid of cells.
#'
#' @usage canvas_turmite(
#'   colors,
#'   background = "#fafafa",
#'   p = 0,
#'   iterations = 1000000,
#'   resolution = 500,
#'   noise = FALSE
#' )
#'
#' @param colors       a character specifying the color used for the artwork. The number of colors determines the number of turmites.
#' @param background  a character specifying the color used for the background.
#' @param p           a value specifying the probability of a state switch within the turmite.
#' @param iterations  a positive integer specifying the number of iterations of the algorithm.
#' @param resolution  resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.
#' @param noise       logical. Whether to add k-nn noise to the artwork. Note that adding noise increases computation time significantly in large dimensions.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @details The turmite algorithm consists of the following steps: 1) turn on the spot (left, right, up, down) 2) change the color of the square 3) move forward one square.
#'
#' @references \url{https://en.wikipedia.org/wiki/Turmite}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_turmite(colors = colorPalette("dark2"))
#' }
#'
#' @export

canvas_turmite <- function(colors,
                           background = "#fafafa",
                           p = 0,
                           iterations = 1000000,
                           resolution = 500,
                           noise = FALSE) {
  .checkUserInput(
    resolution = resolution, background = background, iterations = iterations
  )
  palette <- c(background, colors)
  canvas <- matrix(0, nrow = resolution, ncol = resolution)
  for (i in seq_along(colors)) {
    k <- sample(x = 0:1, size = 1)
    turmite <- cpp_turmite(
      canvas = matrix(0, nrow = resolution, ncol = resolution),
      iters = iterations,
      row = if (k == 0) sample(0:(resolution - 1), size = 1) else 0,
      col = if (k == 1) sample(0:(resolution - 1), size = 1) else 0,
      p = p
    )
    if (noise) {
      turmite[which(turmite == 0)] <- NA
      turmite <- turmite - .noise(dims = c(resolution, resolution), type = "perlin")
      turmite[which(is.na(turmite))] <- 0
    }
    canvas <- canvas + turmite
  }
  full_canvas <- .unraster(canvas, names = c("x", "y", "z"))
  artwork <- ggplot2::ggplot(data = full_canvas, ggplot2::aes(x = x, y = y, fill = z)) +
    ggplot2::geom_raster(interpolate = TRUE, alpha = 0.9) +
    ggplot2::xlim(c(0, resolution + 1)) +
    ggplot2::ylim(c(0, resolution + 1)) +
    ggplot2::scale_fill_gradientn(colours = palette)
  artwork <- theme_canvas(artwork, background)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Langton's Ant
#'
#' @description This function draws Langton's Ant on a canvas. Langton's Ant is
#'   a two-dimensional cellular automaton that is named after its creator, Chris
#'   Langton. See the \code{Details} section for more specific information about
#'   the algorithm used in this function.
#'
#' @usage canvas_ant(
#'   colors,
#'   background = "#fafafa",
#'   iterations = 1000000,
#'   resolution = 500
#' )
#'
#' @param colors      a character (vector) specifying the color(s) used for the
#'   artwork.
#' @param background  a character specifying the color used for the background.
#' @param iterations  a positive integer specifying the number of iterations of
#'   the algorithm.
#' @param resolution  resolution of the artwork in pixels per row/column.
#'   Increasing the resolution increases the quality of the artwork but also
#'   increases the computation time exponentially.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @details The algorithm for Langton's Ant involves the following steps:
#'
#' \itemize{
#'   \item{Set up a two-dimensional grid of cells, where each cell can either be
#'     "colored" or "non-colored." The initial state of the grid is usually a
#'     single non-colored cell in the center of the grid.}
#'   \item{Place an "ant" on the grid at the position of the initial non-colored
#'     cell. The ant can move in four directions: up, down, left, or right.}
#'   \item{At each step of the algorithm, the ant examines the color of the cell
#'     it is currently on. If the cell is non-colored, the ant turns 90 degrees
#'     clockwise, colors the cell, and moves forward one unit.}
#'   \item{If the cell is colored, the ant turns 90 degrees counterclockwise,
#'     uncolors the cell, and moves forward one unit.}
#'   \item{The ant continues to move around the grid, following these rules at
#'     each step. If a certain number of iterations has passed, the ant chooses
#'     a different color which corresponds to a different combination of these
#'     rules.}
#' }
#'
#' @references \url{https://en.wikipedia.org/wiki/Langtons_ant}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_ant(colors = colorPalette("house"))
#' }
#'
#' @export

canvas_ant <- function(colors,
                       background = "#fafafa",
                       iterations = 1000000,
                       resolution = 500) {
  .checkUserInput(
    background = background, resolution = resolution, iterations = iterations
  )
  if (iterations < length(colors)) {
    stop(paste0("'iterations' must be >= ", length(colors)))
  }
  palette <- c(background, colors)
  full_canvas <- cpp_ant(
    canvas = matrix(0, nrow = resolution, ncol = resolution),
    directions = .ant_directions(length(colors)),
    iterations = iterations,
    resolution = resolution
  )
  full_canvas <- .unraster(full_canvas, names = c("x", "y", "z"))
  artwork <- ggplot2::ggplot(data = full_canvas, ggplot2::aes(x = x, y = y, fill = z)) +
    ggplot2::geom_raster(interpolate = TRUE, alpha = 0.9) +
    ggplot2::xlim(c(0, resolution + 1)) +
    ggplot2::ylim(c(0, resolution + 1)) +
    ggplot2::scale_fill_gradientn(colours = palette)
  artwork <- theme_canvas(artwork, background)
  return(artwork)
}

.ant_directions <- function(n) {
  # Create a sequence of 0 (L) and 1 (R) positions
  sequence <- rep(0:1, n)
  # Create a matrix that holds all possible combinations of 0 (L) and 1 (R)
  pos <- expand.grid(sequence, sequence)
  # Remove all combinations that have two 0s or two 1s
  pos <- pos[which(pos[, 1] == pos[, 2]), ]
  # Mix the possible positions randomly
  pos[2:nrow(pos), ] <- pos[sample(2:nrow(pos), size = length(2:nrow(pos))), ]
  # Select only as many positions as there are colors given by the user
  pos <- pos[1:n, ]
  return(as.matrix(pos))
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw A Flow Field
#'
#' @description This function draws flow fields on a canvas. The algorithm simulates the flow of points through a field of angles which can be set manually or generated from the predictions of a supervised learning method (i.e., knn, svm, random forest) trained on randomly generated data.
#'
#' @usage canvas_flow(
#'   colors,
#'   background = "#fafafa",
#'   lines = 500,
#'   lwd = 0.05,
#'   iterations = 100,
#'   stepmax = 0.01,
#'   outline = c("none", "circle", "square"),
#'   polar = FALSE,
#'   angles = NULL
#' )
#'
#' @param colors         a string or character vector specifying the color(s) used for the artwork.
#' @param background     a character specifying the color used for the background.
#' @param lines          the number of lines to draw.
#' @param lwd            expansion factor for the line width.
#' @param iterations     the maximum number of iterations for each line.
#' @param stepmax        the maximum proportion of the canvas covered in each iteration.
#' @param outline        character. Which outline to use for the artwork. Possible options are \code{none} (default), \code{circle} or \code{square}.
#' @param polar          logical. Whether to draw the flow field with polar coordinates.
#' @param angles         optional, a 200 x 200 matrix containing the angles in the flow field, or a character indicating the type of noise to use (\code{svm}, \code{knn}, \code{rf}, \code{perlin}, \code{cubic}, \code{simplex}, or \code{worley}). If \code{NULL} (the default), the noise type is chosen randomly.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @references \url{https://tylerxhobbs.com/essays/2020/flow-fields}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_flow(colors = colorPalette("dark2"))
#'
#' # Outline example
#' canvas_flow(
#'   colors = colorPalette("vrolik1"), lines = 10000,
#'   outline = "circle", iterations = 10, angles = "svm"
#' )
#'
#' # Polar example
#' canvas_flow(
#'   colors = colorPalette("vrolik2"), lines = 300,
#'   lwd = 0.5, polar = TRUE
#' )
#'
#' # Advanced example
#' angles <- matrix(0, 200, 200)
#' angles[1:100, ] <- seq(from = 0, to = 2 * pi, length = 100)
#' angles[101:200, ] <- seq(from = 2 * pi, to = 0, length = 100)
#' angles <- angles + rnorm(200 * 200, sd = 0.1)
#' canvas_flow(
#'   colors = colorPalette("tuscany1"), background = "black",
#'   angles = angles, lwd = 0.4, lines = 1000, stepmax = 0.001
#' )
#' }
#'
#' @export

canvas_flow <- function(colors,
                        background = "#fafafa",
                        lines = 500,
                        lwd = 0.05,
                        iterations = 100,
                        stepmax = 0.01,
                        outline = c("none", "circle", "square"),
                        polar = FALSE,
                        angles = NULL) {
  .checkUserInput(
    background = background, iterations = iterations
  )
  outline <- match.arg(outline)
  sequence <- seq(0, 100, length = 100)
  grid <- expand.grid(sequence, sequence)
  grid <- data.frame(x = grid[, 1], y = grid[, 2], z = 0)
  left <- -100
  right <- 100
  bottom <- -100
  top <- 100
  ncols <- right - left
  nrows <- top - bottom
  if (is.null(angles)) {
    angles <- .noise(
      dims = c(nrows, ncols), n = sample(100:300, 1),
      type = sample(c("knn", "svm", "perlin", "cubic", "simplex", "worley"), 1),
      limits = c(-pi, pi)
    )
  } else if (is.character(angles)) {
    angles <- .noise(
      dims = c(nrows, ncols), n = sample(100:300, 1),
      type = angles,
      limits = c(-pi, pi)
    )
  } else {
    if (!is.matrix(angles)) {
      stop("'angles' must be a matrix")
    }
    if (nrow(angles) != nrows || ncol(angles) != ncols) {
      stop(paste0("'angles' must be a ", nrows, " x ", ncols, " matrix"))
    }
  }
  canvas <- matrix(NA, nrow = iterations * lines, ncol = 5)
  ncolors <- length(colors)
  canvas <- cpp_flow(canvas, angles, lines, iterations, ncolors, left, right, top, bottom, stepmax)
  canvas <- canvas[!is.na(canvas[, 1]), ]
  for (j in seq_len(lines)) {
    index <- which(canvas[, 3] == j)
    canvas[index, 5] <- .bmline(n = length(index), lwd)
  }
  canvas <- as.data.frame(canvas)
  colnames(canvas) <- c("x", "y", "z", "color", "size")
  canvas[["color"]] <- colors[canvas[["color"]]]
  if (outline == "circle") {
    canvas[which(sqrt(canvas[["x"]]^2 + canvas[["y"]]^2) > 175 / 2), "color"] <- background
  } else if (outline == "square") {
    canvas[which(canvas[["x"]] < -75 & canvas[["x"]] > 75 & canvas[["y"]] < -75 & canvas[["y"]] > 75), "color"] <- background
  }
  artwork <- ggplot2::ggplot(data = canvas, mapping = ggplot2::aes(x = x, y = y, group = factor(z))) +
    ggplot2::geom_path(linewidth = canvas[["size"]], color = canvas[["color"]], lineend = "round")
  if (polar) {
    artwork <- artwork + ggplot2::coord_polar()
  } else {
    artwork <- artwork + ggplot2::coord_cartesian(xlim = c(left, right), ylim = c(bottom, top))
  }
  artwork <- theme_canvas(artwork, background = background)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Segments
#'
#' @description This function draws line segments on a canvas. The length and direction of the line segments is determined randomly.
#'
#' @usage canvas_segments(
#'   colors,
#'   background = "#fafafa",
#'   n = 250,
#'   p = 0.5,
#'   H = 0.1,
#'   size = 0.2
#' )
#'
#' @param colors      a string or character vector specifying the color(s) used for the artwork.
#' @param background  a character specifying the color used for the background.
#' @param n           a positive integer specifying the number of line segments to draw.
#' @param p           a value specifying the probability of drawing a vertical line segment.
#' @param H           a positive value specifying the scaling factor for the line segments.
#' @param size        a positive value specifying the size of the line segments.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_segments(colors = colorPalette("dark1"))
#' }
#'
#' @export

canvas_segments <- function(colors,
                            background = "#fafafa",
                            n = 250,
                            p = 0.5,
                            H = 0.1,
                            size = 0.2) {
  .checkUserInput(background = background)
  full_canvas <- data.frame(
    x = numeric(), xend = numeric(),
    y = numeric(), yend = numeric()
  )
  for (i in 1:n) {
    x <- 0.8 * stats::runif(1, 0, 1) + 0.1
    y <- 0.8 * stats::runif(1, 0, 1) + 0.1
    k <- H * (1 - sqrt(stats::runif(1, 0, 1)))
    if (stats::runif(1, 0, 1) > p) {
      row <- data.frame(x = x - k, xend = x + k, y = y, yend = y, col = sample(colors, size = 1))
    } else {
      row <- data.frame(x = x, xend = x, y = y - k, yend = y + k, col = sample(colors, size = 1))
    }
    full_canvas <- rbind(full_canvas, row)
  }
  sizes <- sample(size, size = n, replace = TRUE)
  artwork <- ggplot2::ggplot(data = full_canvas, mapping = ggplot2::aes(x = x, y = y, xend = xend, yend = yend)) +
    ggplot2::xlim(c(0, 1)) +
    ggplot2::ylim(c(0, 1)) +
    ggplot2::geom_segment(color = full_canvas[["col"]], linewidth = sizes)
  artwork <- theme_canvas(artwork, background)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Ribbons
#'
#' @description This function paints random ribbons and (optionally) a triangle in the middle.
#'
#' @usage canvas_ribbons(
#'   colors,
#'   background = "#fdf5e6",
#'   triangle = TRUE
#' )
#'
#' @param colors      a string or character vector specifying the color(s) used for the artwork. The number of colors determines the number of ribbons.
#' @param background  a character specifying the color of the background.
#' @param triangle    logical. Whether to draw the triangle that breaks the ribbon polygons.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_ribbons(colors = colorPalette("retro1"))
#' }
#'
#' @export

canvas_ribbons <- function(colors,
                           background = "#fdf5e6",
                           triangle = TRUE) {
  .checkUserInput(background = background)
  # Create an empty figure
  artwork <- ggplot2::ggplot() +
    ggplot2::xlim(c(0, 100)) +
    ggplot2::ylim(0, 100)
  # Determine points on the triangle
  y_max_top <- 75 - 7
  tpl <- data.frame(x = 16:49, y = seq(from = 16, to = 75, length.out = 34))
  tpl <- tpl[which(tpl[["y"]] < y_max_top), ]
  tpr <- data.frame(x = 51:84, y = seq(from = 74, to = 16, length.out = 34))
  tpr <- tpr[which(tpr[["y"]] < y_max_top), ]
  for (i in seq_along(colors)) {
    # Determine points on left side of triangle
    bpb <- data.frame(x = 0, y = sample(10:90, size = 1))
    fpb <- tpl[sample(seq_len(nrow(tpl)), size = 1), ]
    spb <- tpr[sample(seq_len(nrow(tpr)), size = 1), ]
    epb <- data.frame(x = 100, y = sample(10:90, size = 1))
    # Determine points on right side of triangle
    bpt <- data.frame(x = 0, y = bpb[["y"]] + 5)
    fpt <- data.frame(x = fpb[["x"]] + 2.5, y = fpb[["y"]] + 5)
    spt <- data.frame(x = spb[["x"]] - 2.5, y = spb[["y"]] + 5)
    ept <- data.frame(x = 100, y = epb[["y"]] + 5)
    # Combine polygon points
    polygon <- rbind(bpb, fpb, spb, epb, ept, spt, fpt, bpt)
    artwork <- artwork + ggplot2::geom_polygon(
      data = polygon, mapping = ggplot2::aes(x = x, y = y),
      fill = colors[i], color = NA,
      stat = "identity", alpha = 1
    )
  }
  # (Optionally) draw the triangle
  if (triangle) {
    artwork <- artwork + ggplot2::geom_polygon(
      data = data.frame(x = c(15, 50, 85), y = c(15, 75, 15)), mapping = ggplot2::aes(x = x, y = y),
      fill = NA, color = "black",
      stat = "identity", linewidth = 1
    )
  }
  artwork <- theme_canvas(artwork, background)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Portuguese Tiles
#'
#' @description This function uses a reaction diffusion algorithm in an attempt to draw a Portuguese-styled tiling pattern.
#'
#' @usage canvas_tiles(
#'   colors,
#'   background = "#ffffff",
#'   iterations = 1000,
#'   size = 5,
#'   col.line = "#000000",
#'   resolution = 100,
#'   layout = NULL
#' )
#'
#' @param colors      a string or character vector specifying the color(s) used for the artwork, or a list containing a set of colors for each unique tile on the wall.
#' @param background  a character specifying the color of the background.
#' @param size        a positive integer specifying how many tiles should be in each row of the wall.
#' @param col.line    a character specifying the color of the joints between the tiles.
#' @param iterations  a positive integer specifying the number of iterations of the algorithm.
#' @param resolution  resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.
#' @param layout      optional. A matrix containing integers ranging from 1 to the maximum number of unique tiles (i.e., \code{length(colors)}) specifying the placement of the tiles on the wall.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @references \url{https://en.wikipedia.org/wiki/Reaction–diffusion_system}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(3)
#'
#' # Simple example
#' canvas_tiles(colors = colorPalette("azul"), iterations = 5000)
#'
#' # Advanced example
#' canvas_tiles(colors = list(
#'   colorPalette("blossom"),
#'   colorPalette("neon1"),
#'   colorPalette("dark1")
#' ))
#'
#' # Custom layout
#' layout <- matrix(c(
#'   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
#'   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
#'   1, 1, 1, 2, 1, 1, 1, 2, 1, 1, 1,
#'   1, 1, 2, 2, 2, 1, 2, 2, 2, 1, 1,
#'   1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1,
#'   1, 1, 2, 2, 2, 2, 2, 2, 2, 1, 1,
#'   1, 1, 1, 2, 2, 2, 2, 2, 1, 1, 1,
#'   1, 1, 1, 1, 2, 2, 2, 1, 1, 1, 1,
#'   1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1,
#'   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
#'   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
#' ), nrow = 11, byrow = TRUE)
#' canvas_tiles(
#'   colors = list(colorPalette("azul"), colorPalette("blossom")),
#'   size = nrow(layout), layout = layout
#' )
#'
#' # Another custom layout
#' set.seed(11)
#' layout <- matrix(c(
#'   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
#'   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
#'   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
#'   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
#'   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
#'   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
#'   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
#'   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
#'   2, 2, 1, 2, 2, 3, 3, 2, 2, 4, 4, 4, 2, 5, 5, 5, 2, 6, 2, 6, 2,
#'   2, 1, 2, 1, 2, 3, 2, 3, 2, 2, 4, 2, 2, 5, 2, 2, 2, 6, 2, 6, 2,
#'   2, 1, 1, 1, 2, 3, 3, 2, 2, 2, 4, 2, 2, 2, 5, 2, 2, 2, 6, 2, 2,
#'   2, 1, 2, 1, 2, 3, 2, 3, 2, 2, 4, 2, 5, 5, 5, 2, 2, 2, 6, 2, 2,
#'   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
#'   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
#'   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
#'   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
#'   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
#'   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
#'   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
#'   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
#'   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2
#' ), nrow = 21, byrow = TRUE)
#' canvas_tiles(
#'   colors = list(
#'     colorPalette("blossom"),
#'     colorPalette("azul"),
#'     colorPalette("neon1"),
#'     colorPalette("mixer4"),
#'     colorPalette("neon2"),
#'     colorPalette("vrolik1"),
#'     colorPalette("blackwhite")
#'   ),
#'   iterations = 2000,
#'   size = nrow(layout), layout = layout
#' )
#' }
#'
#' @export

canvas_tiles <- function(colors,
                         background = "#ffffff",
                         iterations = 1000,
                         size = 5,
                         col.line = "#000000",
                         resolution = 100,
                         layout = NULL) {
  .checkUserInput(
    resolution = resolution, background = background, iterations = iterations
  )
  stopifnot("'size' must be > 0" = size > 0)
  rate_a <- 1
  rate_b <- 0.5
  duplicates <- size - 1
  cex.line <- 7 / duplicates
  ones <- matrix(1, resolution, resolution)
  conv_mat <- matrix(c(
    0.05, 0.2, 0.05,
    0.2, -1, 0.2,
    0.05, 0.2, 0.05
  ), nrow = 3)
  if (is.list(colors)) {
    ntiles <- length(colors)
    colorvalues <- unlist(mapply(c, background, colors, USE.NAMES = FALSE))
  } else {
    ntiles <- 1
    colorvalues <- c(background, colors)
  }
  tiles <- list()
  for (i in 1:ntiles) {
    type <- sample.int(4, size = 1)
    feed_rate <- switch(type,
      "1" = 0.0545,
      "2" = 0.055,
      "3" = 0.029,
      "4" = 0.03
    )
    kill_rate <- switch(type,
      "1" = 0.062,
      "2" = 0.062,
      "3" = 0.057,
      "4" = 0.06
    )
    binary <- matrix(sample(c(0, 1), size = resolution * resolution, replace = TRUE, prob = c(0.99, 0.01)), ncol = resolution)
    tile <- array(c(ones, binary), dim = c(resolution, resolution, 2))
    tile <- cpp_tiles(tile, conv_mat, rate_a, rate_b, feed_rate, kill_rate, iterations)[, , 2]
    tile <- cbind(tile, tile[, rev(seq_len(ncol(tile)))])
    tile <- rbind(tile, tile[rev(seq_len(nrow(tile))), ])
    if (ntiles > 1) {
      tile <- scales::rescale(x = tile, from = range(tile), to = c(0, lengths(colors)[i]))
    }
    if (i > 1) {
      tile <- tile + i + sum(lengths(colors)[seq_len(i - 1)])
    }
    tiles[[i]] <- tile
  }
  if (size == 1) {
    canvas <- tiles[[1]]
  } else {
    if (is.null(layout)) {
      suppressWarnings({
        layout_matrix <- matrix(seq_len(ntiles), nrow = size, ncol = size)
      })
    } else {
      stopifnot("'layout' must be a matrix" = is.matrix(layout))
      stopifnot("'layout' must be a `size` x `size` matrix" = nrow(layout) == size && ncol(layout) == size)
      stopifnot("'layout' must contain integers from 1 to the number of tiles" = all(layout %% 1 == 0) && max(layout) <= ntiles && min(layout) == 1)
      layout_matrix <- layout
      layout_matrix <- layout_matrix[rev(seq_len(nrow(layout_matrix))), ]
    }
    for (i in 1:size) {
      row <- tiles[[layout_matrix[1, i]]]
      for (j in 2:size) {
        row <- cbind(row, tiles[[layout_matrix[j, i]]])
      }
      if (i == 1) {
        canvas <- row
      } else {
        canvas <- rbind(canvas, row)
      }
    }
  }
  rownames(canvas) <- colnames(canvas) <- seq_len(nrow(canvas))
  full_canvas <- .unraster(canvas, names = c("x", "y", "z"))
  if (ntiles > 1) {
    breaks <- NULL
    for (i in 1:ntiles) {
      breaks <- c(breaks, seq(floor(min(tiles[[i]])), max(tiles[[i]]), length.out = lengths(colors)[i] + 1))
    }
  } else {
    breaks <- seq(0, max(full_canvas[["z"]]), length.out = length(colorvalues))
  }
  artwork <- ggplot2::ggplot(data = full_canvas, ggplot2::aes(x = x, y = y, fill = z)) +
    ggplot2::geom_raster(interpolate = TRUE) +
    ggplot2::scale_fill_gradientn(colours = colorvalues, values = scales::rescale(breaks, from = range(breaks), to = c(0, 1))) +
    ggplot2::coord_cartesian(xlim = c(0, (resolution * 2) * (duplicates + 1)), ylim = c(0, (resolution * 2) * (duplicates + 1)))
  if (duplicates > 0 && !is.null(col.line)) {
    lineDataX <- data.frame(x = (resolution * 2) * (0:(duplicates + 1)), xend = (resolution * 2) * (0:(duplicates + 1)), y = rep(0, duplicates + 2), yend = rep((resolution * 2) * (duplicates + 1), duplicates + 2))
    lineDataY <- data.frame(y = (resolution * 2) * (0:(duplicates + 1)), yend = (resolution * 2) * (0:(duplicates + 1)), x = rep(0, duplicates + 2), xend = rep((resolution * 2) * (duplicates + 1), duplicates + 2))
    lineDataX2 <- data.frame(x = resolution * (0:((duplicates * 2) + 1)), xend = resolution * (0:((duplicates * 2) + 1)), y = rep(0, (duplicates * 2) + 2), yend = rep(resolution * ((duplicates * 2) + 2), (duplicates * 2) + 2))
    lineDataY2 <- data.frame(y = resolution * (0:((duplicates * 2) + 1)), yend = resolution * (0:((duplicates * 2) + 1)), x = rep(0, (duplicates * 2) + 2), xend = rep(resolution * ((duplicates * 2) + 2), (duplicates * 2) + 2))
    artwork <- artwork + ggplot2::geom_segment(data = lineDataX2, mapping = ggplot2::aes(x = x, xend = xend, y = y, yend = yend), col = col.line, inherit.aes = FALSE, linewidth = cex.line * 0.1) +
      ggplot2::geom_segment(data = lineDataY2, mapping = ggplot2::aes(x = x, xend = xend, y = y, yend = yend), col = col.line, inherit.aes = FALSE, linewidth = cex.line * 0.1) +
      ggplot2::geom_segment(data = lineDataX, mapping = ggplot2::aes(x = x, xend = xend, y = y, yend = yend), col = col.line, inherit.aes = FALSE, linewidth = cex.line * 0.75) +
      ggplot2::geom_segment(data = lineDataY, mapping = ggplot2::aes(x = x, xend = xend, y = y, yend = yend), col = col.line, inherit.aes = FALSE, linewidth = cex.line * 0.75) +
      ggplot2::geom_segment(data = lineDataX, mapping = ggplot2::aes(x = x, xend = xend, y = y, yend = yend), col = background, inherit.aes = FALSE, linewidth = cex.line * 0.3) +
      ggplot2::geom_segment(data = lineDataY, mapping = ggplot2::aes(x = x, xend = xend, y = y, yend = yend), col = background, inherit.aes = FALSE, linewidth = cex.line * 0.3)
    if (size < 8) {
      for (i in 0:size) {
        for (j in 1:10) {
          pointData <- data.frame(x = rep(i * resolution * 2, 2000) + stats::rnorm(2000, 0, cex.line * 0.4), y = seq(0, resolution * 2 * size, length.out = 2000) + stats::rnorm(2000, 0, cex.line * 0.4))
          artwork <- artwork + ggplot2::geom_point(data = pointData, mapping = ggplot2::aes(x = x, y = y), fill = background, inherit.aes = FALSE, size = 0, alpha = 0.1, shape = 21, col = background) +
            ggplot2::geom_point(data = pointData, mapping = ggplot2::aes(x = y, y = x), fill = background, inherit.aes = FALSE, size = 0, alpha = 0.1, shape = 21, col = background)
        }
      }
    }
  }
  artwork <- aRtsy::theme_canvas(artwork)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Functions
#'
#' @description This function paints functions with random parameters on a canvas.
#'
#' @usage canvas_function(
#'   colors,
#'   background = "#fafafa",
#'   by = 0.01,
#'   polar = TRUE,
#'   formula = NULL
#' )
#'
#' @param colors      a string specifying the color used for the artwork.
#' @param background  a character specifying the color used for the background.
#' @param by          a value specifying the step size between consecutive points.
#' @param polar       logical. Whether to draw the function with polar coordinates.
#' @param formula     optional, a named list with 'x' and 'y' as structured in the example. If \code{NULL} (default), chooses a function with random parameters.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @references \url{https://github.com/cutterkom/generativeart}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(10)
#'
#' # Simple example
#' canvas_function(colors = colorPalette("tuscany1"))
#'
#' # Advanced example
#' formula <- list(
#'   x = quote(x_i^2 - sin(y_i^2)),
#'   y = quote(y_i^3 - cos(x_i^2))
#' )
#' canvas_function(colors = "firebrick", formula = formula)
#' }
#'
#' @keywords artwork canvas
#'
#' @export

canvas_function <- function(colors,
                            background = "#fafafa",
                            by = 0.01,
                            polar = TRUE,
                            formula = NULL) {
  .checkUserInput(background = background)
  if (is.null(formula)) {
    painting_formulas <- list()
    painting_formulas[[1]] <- list(
      x = quote(runif(1, -10, 10) * x_i^sample(c(0.5, 1:6), 1) - sin(y_i^sample(c(0.5, 1:6), 1)) * runif(1, -100, 100)),
      y = quote(runif(1, -10, 10) * y_i^sample(c(0.5, 1:6), 1) - cos(x_i^sample(c(0.5, 1:6), 1)) * y_i^sample(1:6, 1) + runif(1, -100, 100))
    )
    painting_formulas[[2]] <- list(
      x = quote(runif(1, -1, 10) * x_i^sample(c(0.5, 1:6), 1) - sin(y_i^sample(c(0.5, 1:6), 1))),
      y = quote(runif(1, -1, 10) * y_i^sample(c(0.5, 1:6), 1) - cos(x_i^sample(c(0.5, 1:6), 1)) * y_i^sample(c(0.5, 1:6), 1))
    )
    painting_formulas[[3]] <- list(
      x = quote(runif(1, -5, 5) * x_i^sample(1:5, 1) - sin(y_i^sample(1:5, 1))),
      y = quote(runif(1, -5, 5) * y_i^sample(1:5, 1) - cos(x_i^sample(1:5, 1)))
    )
    painting_formula <- painting_formulas[[sample(seq_len(length(painting_formulas)), 1)]]
  } else {
    if (!is.list(formula) || !("x" %in% names(formula)) || !("y" %in% names(formula))) {
      stop("'formula' must be a named list containing 'x' and 'y'")
    }
    painting_formula <- list(x = formula[["x"]], y = formula[["y"]])
  }
  grid <- expand.grid(x_i = seq(from = -pi, to = pi, by = by), y_i = seq(from = -pi, to = pi, by = by))
  x_i <- grid[["x_i"]] # nolint
  y_i <- grid[["y_i"]]
  full_canvas <- data.frame(x = eval(painting_formula[["x"]]), y = eval(painting_formula[["y"]]))
  z <- y_i[stats::complete.cases(full_canvas)]
  full_canvas <- full_canvas[stats::complete.cases(full_canvas), ]
  artwork <- ggplot2::ggplot(data = full_canvas, ggplot2::aes(x = x, y = y, color = z)) +
    ggplot2::geom_point(alpha = 0.1, size = 0, shape = 20) +
    ggplot2::scale_color_gradientn(colors = colors)
  if (polar) {
    artwork <- artwork + ggplot2::coord_polar()
  }
  artwork <- theme_canvas(artwork, background)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Rainbow Smoke
#'
#' @description This function implements the rainbow smoke algorithm.
#'
#' @usage canvas_smoke(
#'   colors,
#'   init = 1,
#'   shape = c("bursts", "clouds"),
#'   algorithm = c("minimum", "average"),
#'   resolution = 150
#' )
#'
#' @param colors      a string or character vector specifying the color(s) used for the artwork.
#' @param init        an integer larger than zero and lower  than or equal to \code{resolution^2} specifying the initial number of pixels to color on the canvas.
#' @param shape       a character indicating the shape of the smoke. Possible options are \code{burst} and \code{clouds}.
#' @param resolution  resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.
#' @param algorithm   a character specifying how to select a new pixel. The default option \code{minimum} selects the pixel with the smallest color difference in a single neighbor and is relatively fast. The option \code{average} selects the pixel with the smallest average color difference in all the neighbors and is relatively slow.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @references \url{http://rainbowsmoke.hu}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_smoke(colors = "all", resolution = 500)
#'
#' # Advanced example
#' reds <- colorRampPalette(c("red", "black"))
#' blues <- colorRampPalette(c("goldenrod", "navyblue"))
#' palette <- c(reds(100), blues(100))
#' canvas_smoke(colors = palette, init = 3, shape = "clouds", resolution = 500)
#' }
#'
#' @export
canvas_smoke <- function(colors,
                         init = 1,
                         shape = c("bursts", "clouds"),
                         algorithm = c("minimum", "average"),
                         resolution = 150) {
  .checkUserInput(resolution = resolution)
  stopifnot("'init' must be > 0 and <= resolution^2" = init > 0 && init <= resolution^2)
  shape <- match.arg(shape)
  algorithm <- match.arg(algorithm)
  all_colors <- length(colors) == 1 && colors[1] == "all"
  color_mat <- .smoke_colors(colors, all_colors)
  shape <- switch(shape,
    "bursts" = 0,
    "clouds" = 1
  )
  algorithm <- switch(algorithm,
    "minimum" = 0,
    "average" = 1
  )
  canvas <- array(c(rep(-1, 3 * resolution^2), rep(0, 2 * resolution^2)), c(resolution, resolution, 5))
  coords <- as.matrix(expand.grid(0:(resolution - 1), 0:(resolution - 1)))
  canvas <- cpp_smoke(canvas, coords, color_mat, init, algorithm, shape, all_colors)
  full_canvas <- as.data.frame(expand.grid(x = 1:resolution, y = 1:resolution))
  full_canvas[["col"]] <- grDevices::rgb(red = canvas[, , 1], green = canvas[, , 2], blue = canvas[, , 3], maxColorValue = 255)
  artwork <- ggplot2::ggplot(data = full_canvas, mapping = ggplot2::aes(x = x, y = y)) +
    ggplot2::geom_raster(interpolate = TRUE, fill = full_canvas[["col"]])
  artwork <- theme_canvas(artwork)
  return(artwork)
}

.smoke_colors <- function(colors, all_colors) {
  if (all_colors) {
    return(matrix(NA, 1, 1))
  } else {
    return(t(as.matrix(grDevices::col2rgb(colors))))
  }
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Chladni Figures
#'
#' @description This function draws Chladni figures on a canvas. Named after
#'   Ernst Chladni, an 18th century physicist who first discovered them, Chladni
#'   figures are patterns that arise from the vibrations of a two-dimensional
#'   plate, typically covered with a thin layer of sand or powder. The Chladni
#'   figures are created by varying the frequency of vibration applied to the
#'   plate. In this implementation, the grid underneath the plate can be
#'   transformed using a domain warping technique. The basic idea behind domain
#'   warping is to apply a series of transformations to the input grid to create
#'   a more complex and interesting output.
#'
#' @usage canvas_chladni(
#'   colors,
#'   waves = 5,
#'   warp = 0,
#'   resolution = 500,
#'   angles = NULL,
#'   distances = NULL,
#'   flatten = FALSE
#' )
#'
#' @param colors      a string or character vector specifying the color(s) used
#'   for the artwork.
#' @param waves       a character specifying the number of randomly sampled
#'   waves, or an integer vector of waves to be summed.
#' @param resolution  resolution of the artwork in pixels per row/column.
#'   Increasing the resolution increases the quality of the artwork but also
#'   increases the computation time exponentially.
#' @param warp        a numeric value specifying the maximum warping distance
#'   for each point. If \code{warp = 0} (the default), no warping is performed.
#' @param angles      optional, a resolution x resolution matrix containing the
#'   angles for the warp, or a character indicating the type of noise to use
#'   (\code{svm}, \code{knn}, \code{rf}, \code{perlin}, \code{cubic},
#'   \code{simplex}, or \code{worley}). If \code{NULL} (the default), the noise
#'   type is chosen randomly.
#' @param distances   optional, a resolution x resolution matrix containing the
#'   distances for the warp, or a character indicating the type of noise to use
#'   (\code{svm}, \code{knn}, \code{rf}, \code{perlin}, \code{cubic},
#'   \code{simplex}, or \code{worley}). If \code{NULL} (the default), the noise
#'   type is chosen randomly.
#' @param flatten     logical, should colors be flattened after being assigned
#'   to a point.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(2)
#'
#' # Simple example
#' canvas_chladni(colors = colorPalette("origami"))
#'
#' # Advanced example
#' canvas_chladni(
#'   colors = colorPalette("lava"),
#'   waves = c(1, 2, 3, 9),
#'   warp = 1
#' )
#' }
#'
#' @export

canvas_chladni <- function(colors,
                           waves = 5,
                           warp = 0,
                           resolution = 500,
                           angles = NULL,
                           distances = NULL,
                           flatten = FALSE) {
  .checkUserInput(resolution = resolution)
  if (length(waves) == 1) {
    waves <- sample.int(50, size = waves, replace = TRUE)
  }
  x <- seq(0, 0.5 * pi, length.out = resolution)
  y <- seq(0, 0.5 * pi, length.out = resolution)
  canvas <- expand.grid(x, y)
  if (warp > 0) {
    inputCanvas <- .warp(canvas, warp, resolution, angles, distances)
  } else {
    inputCanvas <- as.matrix(canvas)
  }
  full_canvas <- data.frame(x = canvas[, 1], y = canvas[, 2], z = cpp_chladni(x = inputCanvas[, 1], y = inputCanvas[, 2], waves))
  if (flatten) {
    full_canvas[["z"]] <- round(full_canvas[["z"]], 0)
  }
  artwork <- ggplot2::ggplot(data = full_canvas, mapping = ggplot2::aes(x = x, y = y, fill = z)) +
    ggplot2::geom_raster(interpolate = TRUE) +
    ggplot2::scale_fill_gradientn(colours = colors)
  artwork <- theme_canvas(artwork)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Strokes
#'
#' @description This function creates an artwork that resembles paints strokes. The algorithm is based on the simple idea that each next point on the grid has a chance to take over the color of an adjacent colored point but also has a change of generating a new color.
#'
#' @usage canvas_strokes(
#'   colors,
#'   neighbors = 1,
#'   p = 0.01,
#'   iterations = 1,
#'   resolution = 500,
#'   side = FALSE
#' )
#'
#' @param colors     a string or character vector specifying the color(s) used for the artwork.
#' @param neighbors  a positive integer specifying the number of neighbors a block considers when taking over a color. More neighbors fades the artwork.
#' @param p          a value specifying the probability of selecting a new color at each block. A higher probability adds more noise to the artwork.
#' @param iterations a positive integer specifying the number of iterations of the algorithm. More iterations generally apply more fade to the artwork.
#' @param resolution resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.
#' @param side       logical. Whether to put the artwork on its side.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_strokes(colors = colorPalette("tuscany1"))
#' }
#'
#' @export

canvas_strokes <- function(colors,
                           neighbors = 1,
                           p = 0.01,
                           iterations = 1,
                           resolution = 500,
                           side = FALSE) {
  .checkUserInput(
    resolution = resolution, iterations = iterations
  )
  stopifnot("'neighbors' must be a single integer >= 1" = neighbors > 0 && neighbors %% 1 == 0 && length(neighbors) == 1)
  if (length(colors) == 1) {
    colors <- c("#fafafa", colors)
  }
  neighborsLocations <- as.matrix(expand.grid(-(neighbors):neighbors, -(neighbors):neighbors))
  canvas <- matrix(0, nrow = resolution, ncol = resolution)
  for (i in seq_len(iterations)) {
    canvas <- cpp_strokes(canvas, neighborsLocations, length(colors), p)
  }
  full_canvas <- .unraster(canvas, names = c("x", "y", "z"))
  artwork <- ggplot2::ggplot(data = full_canvas, ggplot2::aes(x = x, y = y, fill = z)) +
    ggplot2::geom_raster(interpolate = TRUE, alpha = 0.9) +
    ggplot2::xlim(c(0, resolution + 1)) +
    ggplot2::ylim(c(0, resolution + 1)) +
    ggplot2::scale_fill_gradientn(colours = colors)
  if (side) {
    artwork <- artwork + ggplot2::coord_flip()
  }
  artwork <- theme_canvas(artwork)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw a Fractal Flame
#'
#' @description This function implements the fractal flame algorithm.
#'
#' @usage canvas_flame(
#'   colors,
#'   background = "#000000",
#'   iterations = 1000000,
#'   variations = 0,
#'   symmetry = 0,
#'   blend = TRUE,
#'   weighted = FALSE,
#'   post = FALSE,
#'   final = FALSE,
#'   extra = FALSE,
#'   display = c("colored", "logdensity"),
#'   zoom = 1,
#'   resolution = 1000,
#'   gamma = 1
#' )
#'
#' @param colors      a string or character vector specifying the color(s) used for the artwork.
#' @param background  a character specifying the color used for the background.
#' @param iterations  a positive integer specifying the number of iterations of the algorithm. Using more iterations results in images of higher quality but also increases the computation time.
#' @param variations  an integer (vector) with a minimum of 0 and a maximum of 48 specifying the variations to be included in the flame. The default \code{0} includes only a linear variation. Including multiple variations (e.g., \code{c(1, 2, 3)}) increases the computation time. See the details section for more information about possible variations.
#' @param symmetry    an integer with a minimum of -6 and a maximum of 6 indicating the type of symmetry to include in the flame. The default \code{0} includes no symmetry. Including symmetry decreases the computation time as a function of the absolute \code{symmetry} value. See the details section for more information about possible symmetries.
#' @param blend       logical. Whether to blend the variations (\code{TRUE}) or pick a unique variation in each iteration (\code{FALSE}). \code{blend = TRUE} increases computation time as a function of the number of included variations.
#' @param weighted    logical. Whether to weigh the functions and the variations (\code{TRUE}) or pick a function at random and equally weigh all variations (\code{FALSE}). \code{weighted = TRUE} significantly increases the computation time.
#' @param post        logical. Whether to apply a post transformation in each iteration.
#' @param final       logical. Whether to apply a final transformation in each iteration.
#' @param extra       logical. Whether to apply an additional post transformation after the final transformation. Only has an effect when \code{final = TRUE}.
#' @param display     a character indicating how to display the flame. \code{colored} (the default) displays colors according to which function they originate from. \code{logdensity} plots a gradient using the log density of the pixel count.
#' @param zoom        a positive value specifying the amount of zooming.
#' @param resolution  resolution of the artwork in pixels per row/column. Increasing the resolution does not increases the computation time of this algorithm.
#' @param gamma       a numeric value specifying the gamma correction (only used when \code{display = "colored"}). Larger values result in brighter images and vice versa.
#'
#' @details           The \code{variation} argument can be used to include specific variations into the flame. See the appendix in the references for examples of all variations. Possible variations are:
#'
#' \itemize{
#'  \item{\code{0}: Linear (default)}
#'  \item{\code{1}: Sinusoidal}
#'  \item{\code{2}: Spherical}
#'  \item{\code{3}: Swirl}
#'  \item{\code{4}: Horsehoe}
#'  \item{\code{5}: Polar}
#'  \item{\code{6}: Handkerchief}
#'  \item{\code{7}: Heart}
#'  \item{\code{8}: Disc}
#'  \item{\code{9}: Spiral}
#'  \item{\code{10}: Hyperbolic}
#'  \item{\code{11}: Diamond}
#'  \item{\code{12}: Ex}
#'  \item{\code{13}: Julia}
#'  \item{\code{14}: Bent}
#'  \item{\code{15}: Waves}
#'  \item{\code{16}: Fisheye}
#'  \item{\code{17}: Popcorn}
#'  \item{\code{18}: Exponential}
#'  \item{\code{19}: Power}
#'  \item{\code{20}: Cosine}
#'  \item{\code{21}: Rings}
#'  \item{\code{22}: Fan}
#'  \item{\code{23}: Blob}
#'  \item{\code{24}: PDJ}
#'  \item{\code{25}: Fan2}
#'  \item{\code{26}: Rings2}
#'  \item{\code{27}: Eyefish}
#'  \item{\code{28}: Bubble}
#'  \item{\code{29}: Cylinder}
#'  \item{\code{30}: Perspective}
#'  \item{\code{31}: Noise}
#'  \item{\code{32}: JuliaN}
#'  \item{\code{33}: JuliaScope}
#'  \item{\code{34}: Blur}
#'  \item{\code{35}: Gaussian}
#'  \item{\code{36}: RadialBlur}
#'  \item{\code{37}: Pie}
#'  \item{\code{38}: Ngon}
#'  \item{\code{39}: Curl}
#'  \item{\code{40}: Rectangles}
#'  \item{\code{41}: Arch}
#'  \item{\code{42}: Tangent}
#'  \item{\code{43}: Square}
#'  \item{\code{44}: Rays}
#'  \item{\code{45}: Blade}
#'  \item{\code{46}: Secant}
#'  \item{\code{47}: Twintrian}
#'  \item{\code{48}: Cross}
#' }
#'
#' @details           The \code{symmetry} argument can be used to include symmetry into the flame. Possible options are:
#'
#' \itemize{
#'  \item{\code{0}: No symmetry (default)}
#'  \item{\code{-1}: Dihedral symmetry}
#'  \item{\code{1}: Two-way rotational symmetry}
#'  \item{\code{(-)2}: (Dihedral) Three-way rotational symmetry}
#'  \item{\code{(-)3}: (Dihedral) Four-way rotational symmetry}
#'  \item{\code{(-)4}: (Dihedral) Five-way rotational symmetry}
#'  \item{\code{(-)5}: (Dihedral) Six-way rotational symmetry}
#'  \item{\code{(-)6}: (Dihedral) Snowflake symmetry}
#' }
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @references \url{https://flam3.com/flame_draves.pdf}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(3)
#'
#' # Simple example, linear variation, relatively few iterations
#' canvas_flame(colors = c("dodgerblue", "green"), variations = 0)
#'
#' # Simple example, linear variation, dihedral symmetry
#' canvas_flame(colors = c("hotpink", "yellow"), variations = 0, symmetry = -1, iterations = 1e7)
#'
#' # Advanced example (no-blend, weighted, sinusoidal and spherical variations)
#' canvas_flame(
#'   colors = colorPalette("origami"), variations = c(1, 2),
#'   blend = FALSE, weighted = TRUE, iterations = 1e8
#' )
#'
#' # More iterations give much better images
#' set.seed(123)
#' canvas_flame(colors = c("red", "blue"), iterations = 1e8, variations = c(10, 17))
#' }
#'
#' @export

canvas_flame <- function(colors,
                         background = "#000000",
                         iterations = 1000000,
                         variations = 0,
                         symmetry = 0,
                         blend = TRUE,
                         weighted = FALSE,
                         post = FALSE,
                         final = FALSE,
                         extra = FALSE,
                         display = c("colored", "logdensity"),
                         zoom = 1,
                         resolution = 1000,
                         gamma = 1) {
  display <- match.arg(display)
  .checkUserInput(
    resolution = resolution, background = background
  )
  if (is.null(variations) || !all(variations %% 1 == 0) || min(variations) < 0 || max(variations) > 48) {
    stop("'variations' must be an integer (vector) with a minimum of 0 and a maximum of 48")
  }
  if (is.null(symmetry) || symmetry %% 1 != 0 || symmetry < -6 || symmetry > 6) {
    stop("'symmetry' must be an integer with a minimum of -6 and a maximum of 6")
  }
  iterations <- iterations + 20
  if (display == "logdensity") {
    n <- sample(x = 3:10, size = 1)
    color_mat <- matrix(stats::runif(n * 3), nrow = n, ncol = 3)
  } else {
    n <- sample(x = 3:max(10, length(colors)), size = 1)
    colors <- c(colors, sample(x = colors, size = n, replace = TRUE))
    color_mat <- matrix(t(grDevices::col2rgb(colors) / 255), nrow = length(colors), ncol = 3)
  }
  fs <- .flame_system(n, symmetry)
  w_i <- stats::runif(n = fs[["n"]], min = 0, max = 1)
  v_ij <- matrix(stats::runif(fs[["n"]] * length(variations), min = 0, max = 1), nrow = fs[["n"]], ncol = length(variations))
  for (i in seq_len(nrow(v_ij))) {
    v_ij[i, ] <- v_ij[i, ] / sum(v_ij[i, ])
  }
  canvas <- cpp_flame( # 1 = x, 2 = y, 3 = red, 4 = green, 5 = blue
    canvas = array(0, dim = c(resolution + 1, resolution + 1, 4)), iterations = iterations,
    resolution = resolution,
    edge = 2 * (1 / zoom),
    blend = blend,
    weighted = weighted,
    post = post,
    final = final,
    extra = extra,
    colors = color_mat,
    functions = 0:(fs[["n"]] - 1),
    funcWeights = w_i / sum(w_i),
    funcPars = fs[["parameters"]],
    variations = variations,
    varWeights = v_ij,
    varParams = .flame_variations(),
    postPars = matrix(stats::runif(fs[["n"]] * 6, min = -1, max = 1), nrow = fs[["n"]], ncol = 6),
    finalPars = stats::runif(6, min = -1, max = 1),
    extraPars = stats::runif(6, min = -1, max = 1),
    bsym = fs[["indicator"]]
  )
  if (length(which(canvas[, , 1] > 0)) <= 1) {
    stop("Too few points are drawn on the canvas")
  }
  full_canvas <- .unraster(canvas[, , 1], c("x", "y", "z"))
  full_canvas[["z"]][full_canvas[["z"]] != 0] <- log(full_canvas[["z"]][full_canvas[["z"]] != 0], base = 1.2589)
  full_canvas[["z"]][full_canvas[["z"]] == 0] <- NA
  if (display == "logdensity") {
    artwork <- ggplot2::ggplot(data = full_canvas, mapping = ggplot2::aes(x = x, y = y, fill = z)) +
      ggplot2::geom_raster(interpolate = TRUE) +
      ggplot2::scale_fill_gradientn(colors = colors, na.value = background)
  } else {
    canvas <- .flame_scaling(canvas)
    canvas[, , 2:4] <- canvas[, , 2:4]^(1 / gamma) # Gamma correction
    maxVal <- max(c(1, c(canvas[, , 2]), c(canvas[, , 3]), c(canvas[, , 4])))
    if (maxVal == 0) {
      stop("No points are drawn on the canvas")
    }
    full_canvas[["col"]] <- grDevices::rgb(red = canvas[, , 2], green = canvas[, , 3], blue = canvas[, , 4], maxColorValue = maxVal)
    full_canvas[["col"]][which(is.na(full_canvas[["z"]]))] <- background
    artwork <- ggplot2::ggplot(data = full_canvas, mapping = ggplot2::aes(x = x, y = y)) +
      ggplot2::geom_raster(interpolate = TRUE, fill = full_canvas[["col"]])
  }
  artwork <- theme_canvas(artwork, background = background)
  return(artwork)
}

.flame_variations <- function() {
  return(c(
    stats::runif(1, 1, 2), stats::runif(1, -2, -1), sample(3:7, size = 1), # blob.high, blob.low, blob.waves
    stats::runif(4, 0, 1), # padj.a, pdj.b, pdj.c, pdj.d
    stats::runif(1, 0, 1), # rings2.val
    stats::runif(1, 10, 80), stats::runif(1, 1, 10), # perspective.angle, perspective.dist
    stats::runif(1, 1, 5), stats::runif(1, 0.1, 1), # juliaN.power, juliaN.dist
    stats::runif(1, 1, 5), stats::runif(1, 0.1, 1), # juliaScope.power, juliaScope.dist
    stats::runif(1, 0, 360), stats::runif(1, 0.5, 10), # radialBlur.angle, v_36
    sample(x = 3:8, size = 1), stats::runif(1, 1, 360), stats::runif(1, 0, 1), # pie.slices, pie.rotation, pie.thickness
    stats::runif(1, 1, 2), sample(3:9, size = 1), sample(3:9, size = 1), stats::runif(1, 0, 1), # ngon.power, ngon.sides, ngon.corners, ngon.circle
    stats::runif(1, -1.5, -0.5), stats::runif(1, 0.5, 1.5), # curl.c1, curl.c2
    stats::runif(1, -1, 1), stats::runif(1, -1, 1), # rectangles.x, rectangles.y
    stats::runif(1, -1, 1), # v_41
    stats::runif(2, 0, 1), # v_44, # v_45
    stats::runif(1, 1, 7), # v_46
    stats::runif(1, 0, 10) # v_47
  ))
}

.flame_scaling <- function(canvas) {
  hits <- which(canvas[, , 1] > 0)
  scaling <- log(canvas[, , 1][hits]) / canvas[, , 1][hits]
  canvas[, , 2][hits] <- canvas[, , 2][hits] * scaling
  canvas[, , 3][hits] <- canvas[, , 3][hits] * scaling
  canvas[, , 4][hits] <- canvas[, , 4][hits] * scaling
  return(canvas)
}

.flame_system <- function(n, symmetry) {
  parameters <- matrix(stats::runif(n * 6, min = -1, max = 1), nrow = n, ncol = 6)
  indicator <- nrow(parameters)
  if (symmetry != 0) {
    if (symmetry != -1) { # Rotation functions
      nrotations <- abs(symmetry) + 1
      for (i in 1:(nrotations - 1)) {
        angle <- (2 * pi) / nrotations * i
        rotation <- matrix(rep(c(cos(angle), -sin(angle), 0, sin(angle), cos(angle), 0), times = n), byrow = TRUE, nrow = n, ncol = 6)
        parameters <- rbind(parameters, rotation)
      }
      n <- nrow(parameters)
    }
    if (symmetry < 0) { # Dihedral function
      dihedral <- matrix(rep(c(-1, 0, 0, 0, 1, 0), times = n), byrow = TRUE, nrow = n, ncol = 6)
      parameters <- rbind(parameters, dihedral)
      n <- nrow(parameters)
    }
  }
  out <- list()
  out[["n"]] <- n
  out[["parameters"]] <- parameters
  out[["indicator"]] <- indicator
  return(out)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Stripes
#'
#' @description This function creates a brownian motion on each row of the artwork and colors it according to the height of the motion.
#'
#' @usage canvas_stripes(
#'   colors,
#'   n = 300,
#'   H = 1,
#'   burnin = 1
#' )
#'
#' @param colors      a string or character vector specifying the color(s) used for the artwork.
#' @param n           a positive integer specifying the length of the brownian motion (effectively the width of the artwork).
#' @param H           a positive value specifying the square of the standard deviation of each step in the motion.
#' @param burnin      a positive integer specifying the number of steps to discard before filling each row.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_stripes(colors = colorPalette("random", n = 10))
#' }
#'
#' @export

canvas_stripes <- function(colors,
                           n = 300,
                           H = 1,
                           burnin = 1) {
  stopifnot("'burnin' must be a single integer >= 1" = burnin > 0 && burnin %% 1 == 0 && length(burnin) == 1)
  mat <- matrix(NA, nrow = n, ncol = n)
  for (i in seq_len(nrow(mat))) {
    t <- seq_len(n + burnin)
    x <- stats::rnorm(n = length(t) - 1, sd = sqrt(H))
    x <- c(0, cumsum(x))
    mat[i, ] <- rev(x[-(1:burnin)])
  }
  canvas <- .unraster(mat, names = c("x", "y", "z"))
  artwork <- ggplot2::ggplot(data = canvas, ggplot2::aes(x = x, y = y, fill = z)) +
    ggplot2::geom_raster(interpolate = TRUE) +
    ggplot2::scale_fill_gradientn(colours = colors)
  artwork <- theme_canvas(artwork)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw the Mandelbrot Set
#'
#' @description This function draws the Mandelbrot set and other related fractal sets on the canvas.
#'
#' @usage canvas_mandelbrot(
#'   colors,
#'   iterations = 100,
#'   zoom = 1,
#'   set = c("mandelbrot", "multibrot", "julia", "ship"),
#'   left = -2.16,
#'   right = 1.16,
#'   bottom = -1.66,
#'   top = 1.66,
#'   resolution = 500
#' )
#'
#' @param colors      a string or character vector specifying the color(s) used for the artwork.
#' @param iterations  a positive integer specifying the number of iterations of the algorithm.
#' @param zoom        a positive value specifying the amount of zoom to apply.
#' @param set         a character indicating which fractal set to draw. Possible options are \code{mandelbrot} for the Mandelbrot set, \code{multibrot} for variations of the Mandelbrot set (aka the Multibrot sets), \code{julia} for the Julia set and \code{ship} for the Burning ship set.
#' @param left        a value specifying the minimum location on the x-axis.
#' @param right       a value specifying the maximum location on the x-axis.
#' @param bottom      a value specifying the minimum location on the y-axis.
#' @param top         a value specifying the maximum location on the y-axis.
#' @param resolution  resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @references \url{https://en.wikipedia.org/wiki/Mandelbrot_set}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' canvas_mandelbrot(colors = colorPalette("tuscany1"), set = "mandelbrot")
#' canvas_mandelbrot(colors = colorPalette("flag"), set = "julia", zoom = 2)
#' }
#'
#' @export

canvas_mandelbrot <- function(colors,
                              iterations = 100,
                              zoom = 1,
                              set = c("mandelbrot", "multibrot", "julia", "ship"),
                              left = -2.16,
                              right = 1.16,
                              bottom = -1.66,
                              top = 1.66,
                              resolution = 500) {
  .checkUserInput(resolution = resolution, iterations = iterations)
  set <- match.arg(set)
  if (zoom > 1) {
    for (i in 1:(zoom - 1)) {
      xmin_tmp <- left
      xmax_tmp <- right
      ymin_tmp <- bottom
      ymax_tmp <- top
      left <- xmin_tmp + abs(xmin_tmp - xmax_tmp) / 4
      right <- xmax_tmp - abs(xmin_tmp - xmax_tmp) / 4
      bottom <- ymin_tmp + abs(ymin_tmp - ymax_tmp) / 4
      top <- ymax_tmp - abs(ymin_tmp - ymax_tmp) / 4
    }
  }
  x <- seq(left, right, length.out = resolution)
  y <- seq(bottom, top, length.out = resolution)
  mk <- sample(3:5, size = 1)
  if (set != "julia") {
    z <- matrix(0, nrow = length(x), ncol = length(y))
    c <- outer(x, y * 1i, FUN = "+")
  } else {
    z <- outer(x, y * 1i, FUN = "+")
    c <- -0.4 + 0.6 * 1i
  }
  canvas <- matrix(0, nrow = length(x), ncol = length(y))
  for (rep in 1:iterations) {
    index <- which(Mod(z) < 2)
    z[index] <- switch(set,
      "mandelbrot" = z[index]^2 + c[index],
      "multibrot" = z[index]^mk + c[index],
      "julia" = z[index]^2 + c,
      "ship" = (abs(Re(z[index])) + abs(Im(z[index])) * 1i)^2 + c[index]
    )
    canvas[index] <- canvas[index] + 1
  }
  full_canvas <- .unraster(canvas, names = c("x", "y", "z"))
  artwork <- ggplot2::ggplot(data = full_canvas, ggplot2::aes(x = x, y = y, fill = z)) +
    ggplot2::geom_raster(interpolate = TRUE, alpha = 0.9) +
    ggplot2::xlim(c(0, resolution + 1)) +
    ggplot2::ylim(c(0, resolution + 1)) +
    ggplot2::scale_fill_gradientn(colours = colors)
  artwork <- theme_canvas(artwork)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# This function generates noise
.noise <- function(dims, n = 100, type = c("artsy-knn", "knn", "svm", "rf", "perlin", "cubic", "simplex", "worley"),
                   k = 20, limits = c(0, 1)) {
  type <- match.arg(type)
  if (type == "artsy-knn") {
    if (length(dims) == 1) {
      vec <- expand.grid(limits[1], seq(limits[1], limits[2], length.out = dims))
    } else if (length(dims) == 2) {
      vec <- expand.grid(seq(limits[1], limits[2], length.out = dims[1]), seq(limits[1], limits[2], length.out = dims[2]))
    }
    z <- cpp_knn(stats::runif(n), stats::runif(n), stats::runif(n), vec[, 1], vec[, 2], k)
  } else if (type == "svm") {
    train <- data.frame(
      x = stats::runif(n, limits[1], limits[2]),
      y = stats::runif(n, limits[1], limits[2]),
      z = stats::runif(n, limits[1], limits[2])
    )
    fit <- e1071::svm(formula = z ~ x + y, data = train)
    xsequence <- seq(limits[1], limits[2], length = dims[1])
    ysequence <- seq(limits[1], limits[2], length = dims[2])
    canvas <- expand.grid(xsequence, ysequence)
    colnames(canvas) <- c("x", "y")
    z <- predict(fit, newdata = canvas)
  } else if (type == "knn") {
    train <- data.frame(
      x = stats::runif(n, limits[1], limits[2]),
      y = stats::runif(n, limits[1], limits[2]),
      z = stats::runif(n, limits[1], limits[2])
    )
    fit <- kknn::train.kknn(formula = z ~ x + y, data = train, kmax = k)
    xsequence <- seq(limits[1], limits[2], length = dims[1])
    ysequence <- seq(limits[1], limits[2], length = dims[2])
    canvas <- expand.grid(xsequence, ysequence)
    colnames(canvas) <- c("x", "y")
    z <- predict(fit, newdata = canvas)
  } else if (type == "rf") {
    train <- data.frame(
      x = stats::runif(n, limits[1], limits[2]),
      y = stats::runif(n, limits[1], limits[2]),
      z = stats::runif(n, limits[1], limits[2])
    )
    fit <- randomForest::randomForest(formula = z ~ x + y, data = train)
    xsequence <- seq(limits[1], limits[2], length = dims[1])
    ysequence <- seq(limits[1], limits[2], length = dims[2])
    canvas <- expand.grid(xsequence, ysequence)
    colnames(canvas) <- c("x", "y")
    z <- predict(fit, newdata = canvas)
  } else if (type == "perlin") {
    z <- ambient::noise_perlin(dims, frequency = stats::runif(1, 0.001, 0.04))
    z <- (z - range(z)[1]) / diff(range(z)) * diff(limits) + limits[1]
  } else if (type == "cubic") {
    z <- ambient::noise_cubic(dims, frequency = stats::runif(1, 0.001, 0.04))
    z <- (z - range(z)[1]) / diff(range(z)) * diff(limits) + limits[1]
  } else if (type == "simplex") {
    z <- ambient::noise_simplex(dims, frequency = stats::runif(1, 0.001, 0.04))
    z <- (z - range(z)[1]) / diff(range(z)) * diff(limits) + limits[1]
  } else if (type == "worley") {
    z <- ambient::noise_worley(dims)
    z <- (z - range(z)[1]) / diff(range(z)) * diff(limits) + limits[1]
  }
  return(matrix(z, nrow = dims[1], ncol = dims[2]))
}

# This function performs validation checks on the standardized input arguments of a function
.checkUserInput <- function(background = NULL, resolution = NULL, iterations = NULL) {
  if (!is.null(background) && length(background) != 1) {
    stop("'background' must be a single character")
  }
  if (!is.null(resolution) && (resolution < 1 || resolution %% 1 != 0)) {
    stop("'resolution' must be a single value > 0")
  }
  if (!is.null(iterations) && (iterations < 1 || iterations %% 1 != 0)) {
    stop("'iterations' must be a single integer > 0")
  }
}

# This function turns a matrix into a data frame with columns x, y, and z
.unraster <- function(x, names) {
  newx <- data.frame(x = rep(seq_len(ncol(x)), times = ncol(x)), y = rep(seq_len(nrow(x)), each = nrow(x)), z = c(x))
  colnames(newx) <- names
  return(newx)
}

# This function takes a point (x, y) and returns a warped point (x, y)
.warp <- function(p, warpDistance, resolution, angles = NULL, distances = NULL) {
  if (is.null(angles)) {
    angles <- .noise(c(resolution, resolution), type = sample(c("svm", "perlin", "cubic", "simplex"), size = 1), limits = c(-pi, pi))
  } else if (is.character(angles)) {
    angles <- .noise(c(resolution, resolution), type = angles, limits = c(-pi, pi))
  } else if (is.matrix(angles)) {
    if (nrow(angles) != resolution || ncol(angles) != resolution) {
      stop(paste0("'angles' should be a ", resolution, " x ", resolution, " matrix"))
    }
  }
  if (is.null(distances)) {
    distances <- .noise(c(resolution, resolution), type = sample(c("knn", "perlin", "cubic", "simplex"), size = 1), limits = c(0, warpDistance))
  } else if (is.character(distances)) {
    distances <- .noise(c(resolution, resolution), type = distances, limits = c(0, warpDistance))
  } else if (is.matrix(distances)) {
    if (nrow(distances) != resolution || ncol(distances) != resolution) {
      stop(paste0("'distances' should be a ", resolution, " x ", resolution, " matrix"))
    }
  }
  return(matrix(c(p[, 1] + c(cos(angles)) * c(distances), p[, 2] + c(sin(angles)) * c(distances)), ncol = 2))
}

# This function returns a brownian motion line
.bmline <- function(n, lwd) {
  x <- cumsum(stats::rnorm(n = n, sd = sqrt(1)))
  x <- abs(x / stats::sd(x) * lwd)
  return(x)
}

# This function returns the time elapsed
.runtime <- function(expression, name = NULL) {
  s1 <- Sys.time()
  eval(expression)
  s2 <- Sys.time()
  cat("\n", if (is.null(name)) NULL else paste0("[", name, "]"), "Runtime:", round(s2 - s1, 5), "seconds\n")
  return(invisible(as.numeric(s2 - s1)))
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Color Palette Generator
#'
#' @description This function creates a random color palette, or allows the user to select a pre-implemented palette.
#'
#' @usage colorPalette(
#'   name,
#'   n = NULL
#' )
#'
#' @param name   name of the color palette. Can be \code{random} for random colors, \code{complement} for complementing colors, \code{divergent} for equally spaced colors, or \code{random-palette} for a random palette, but can also be the name of a pre-implemented palette. See the \code{details} section for a list of pre-implemented palettes.
#' @param n      the number of colors to select from the palette. Required if \code{name = 'random'}, \code{name = 'complement'}, or \code{name = 'divergent'}. Otherwise, if \code{NULL}, automatically selects all colors from the chosen palette.
#'
#' @details The following color palettes are implemented:
#'
#' \if{html}{\figure{colors.svg}{options: width=600 alt="colors"}}
#' \if{latex}{\figure{colors.pdf}{options: width=5in}}
#'
#' @return A vector of colors.
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @examples
#' colorPalette("divergent", 5)
#' @keywords canvas palette
#'
#' @export

colorPalette <- function(name,
                         n = NULL) {
  if (!is.null(n) && (n < 1 || n %% 1 != 0)) {
    stop("'n' must be an integer > 0")
  }
  if (name == "random") {
    stopifnot("'n' is missing for palette = 'random'" = !is.null(n))
    palette <- character(n)
    for (i in seq_along(palette)) {
      palette[i] <- .hsl_to_rgb(h = stats::runif(1, 1, 360), stats::runif(1), stats::runif(1))
    }
  } else if (name == "complement") {
    stopifnot("'n' is missing for palette = 'random'" = !is.null(n))
    palette <- character(n)
    tmp <- stats::runif(1, 1, 360)
    palette[1] <- .hsl_to_rgb(h = tmp, stats::runif(1, .4, .8), stats::runif(1, .4, .8))
    for (i in 2:length(palette)) {
      if (i %% 2 == 0) {
        if (tmp > 180) {
          color <- tmp - 180
        } else if (tmp < 180) {
          color <- tmp + 180
        } else {
          color <- tmp
        }
        palette[i] <- .hsl_to_rgb(h = color, stats::runif(1, .4, .8), stats::runif(1, .4, .8))
      } else {
        tmp <- stats::runif(1, 1, 360)
        palette[i] <- .hsl_to_rgb(h = tmp, stats::runif(1, .4, .8), stats::runif(1, .4, .8))
      }
    }
  } else if (name == "divergent") {
    stopifnot("'n' is missing for palette = 'random'" = !is.null(n))
    palette <- character(n)
    start <- stats::runif(1, 0, 260)
    colSeq <- seq(from = 1, to = 360, length.out = n)
    h <- (colSeq + start) %% 360
    for (i in seq_along(palette)) {
      palette[i] <- .hsl_to_rgb(h = h[i], stats::runif(1, .4, .7), stats::runif(1, .4, .7))
    }
  } else {
    if (name == "random-palette") {
      name <- sample(c(
        "azul", "bell", "blackwhite",
        "blossom", "boogy1", "boogy2",
        "boogy3", "dark1", "dark2",
        "dark3", "flag", "flora",
        "gogh", "house", "jasp",
        "jfa", "jungle", "klimt",
        "kpd", "lava", "nature",
        "neo1", "neo2", "neo3",
        "mixer1", "mixer2", "mixer3",
        "mixer4", "neon1", "neon2",
        "origami", "retro1", "retro2",
        "retro3", "retro4", "shell1",
        "shell2", "shell3", "sooph",
        "sky", "tuscany1", "tuscany2",
        "tuscany3", "vrolik1", "vrolik2",
        "vrolik3", "vrolik4", "vrolik5"
      ), size = 1)
    }
    palette <- switch(name,
      "azul" = c("#ffffff", "#142ea5", "#3b74e4", "#a8d4f7", "#f1da52"),
      "bell" = c("#000000", "#ff0000", "#ffcc00"),
      "blackwhite" = c("black", "white"),
      "blossom" = c("#b89a9d", "#fcf9ee", "#f1acec", "#e0b8c0", "#e95bb1", "#c03962"),
      "boogy1" = c("#a2a07a", "#c49700", "#b4b9cc", "#c0c1b1"),
      "boogy2" = c("#204b9a", "#f1e60e", "#ffffff", "#e5181d", "#1d1d1b"),
      "boogy3" = c("#e9e489", "#995c25", "#5eb9f0", "#68b95d", "#69241f"),
      "dark1" = c("#161616", "#346751", "#C84B31", "#ECDBBA"),
      "dark2" = c("#1B262C", "#0F4C75", "#3282B8", "#BBE1FA"),
      "dark3" = c("#222831", "#393E46", "#00ADB5", "#EEEEEE"),
      "flag" = c("#002F6C", "#0067B9", "#A7C6ED", "#CFCDC9", "#BA0C2F", "#651D32"),
      "flora" = c("#000000", "#f2f2eb", "#ccb77c", "#523402"),
      "gogh" = c("#8699b5", "#161918", "#9d8018", "#232d8a", "#b4ad5a", "#c2ccd5"),
      "house" = c("#191919", "white", "#ab3920", "#cca222", "#036440"),
      "jasp" = c("#14a1e3", "#ffffff", "#f7971c", "#8cc63e", "#000000"),
      "jfa" = c("#2a94d1", "#e89643", "#233e4a"),
      "jungle" = c("#4e6349", "#614128", "#171812", "#698144", "#917861"),
      "klimt" = c("#847049", "#ba9d3e", "#29241f", "#a787b0", "#cd5627", "#7ea36e"),
      "kpd" = c("#c42f32", "#315b99", "#354741"),
      "lava" = c("#f59907", "#a42300", "#482a22", "#050000", "#6b0800"),
      "nature" = c("forestgreen", "dodgerblue", "brown", "white", "gray"),
      "neo1" = c("#de3070", "#89fb68", "#84e7fa", "#6b9cc4", "#090F22"),
      "neo2" = c("#7dfa6a", "#dafd6a", "#eeb556", "#e93d54", "#ad2456"),
      "neo3" = c("#de4a27", "#ed7a30", "#f7cf46", "#1735e4", "#0c259c"),
      "mixer1" = c("#fa9f29", "#15cab1", "#0a633d", "#2f3030"),
      "mixer2" = c("#a3a948", "#edb92e", "#f85931", "#ce1836", "#009989"),
      "mixer3" = c("#aacad5", "#c9e0e6", "#e0e9ee", "#dccda4", "#99886e"),
      "mixer4" = c("#20663F", "#259959", "#ABD406", "#FFD412", "#FF821C"),
      "neon1" = c("#F7FD04", "#F9B208", "#F98404", "#FC5404"),
      "neon2" = c("#F5F7B2", "#1CC5DC", "#890596", "#CF0000"),
      "origami" = c("#01364f", "#84231e", "#247c86", "#e8674d", "#dfdbbe", "#fdf4b4"),
      "retro1" = c("#0A1931", "#185ADB", "#FFC947", "#EFEFEF"),
      "retro2" = c("#DDDDDD", "#222831", "#30475E", "#F05454"),
      "retro3" = c("#111D5E", "#C70039", "#F37121", "#C0E218"),
      "retro4" = c("#80A8A8", "#909D9E", "#A88C8C", "#FF0D51"),
      "shell1" = c("#D6D0C0", "#1C2328", "#A1ABC4", "#8A847A"),
      "shell2" = c("#2B1E1E", "#6E3623", "#292D2F", "#E4F3F5", "#D8BBA2"),
      "shell3" = c("#B9CDDD", "#D4D5CE", "#44596F", "#495152", "#7CADD6"),
      "sooph" = c("#d6c398", "#cbabdb", "#139485", "#9e1710", "#414952"),
      "sky" = c("#CFF09E", "#A8DBA8", "#79BD9A", "#3B8686", "#0B486B"),
      "tuscany1" = c("firebrick", "goldenrod", "forestgreen", "navyblue"),
      "tuscany2" = c("#500342", "#023b59", "#f9efdd", "#deaa70", "#711308"),
      "tuscany3" = c("#b08653", "#f5daba", "#c9673c", "#f2ab4e", "#a1863b"),
      "vrolik1" = c("#4ca787", "#183867", "#ea8857", "#442a37", "#ffb747"),
      "vrolik2" = c("#d8c888", "#f4edf3", "#fa428c", "#1900b4", "#ecd07d"),
      "vrolik3" = c("#774f38", "#e08e79", "#f1d4af", "#ece5ce", "#c5e0dc"),
      "vrolik4" = c("#16E0F2", "#080E10", "#E69E02", "#CB1BA3", "#BA097F"),
      "vrolik5" = c("#FBCE03", "#03F7EC", "#4D0E98", "#3A3838", "#150326")
    )
    if (is.null(palette)) {
      stop(paste0("there is no palette called '", name, "'"))
    }
    if (is.null(n)) {
      n <- length(palette)
    }
    if (n > length(palette)) {
      palette <- c(palette, palette[sample.int(n - length(palette))])
    } else {
      palette <- palette[sample.int(n)]
    }
  }
  return(palette)
}

.hsl_to_rgb <- function(h, s, l) {
  h <- h / 360
  r <- g <- b <- 0.0
  if (s == 0) {
    r <- g <- b <- l
  } else {
    q <- ifelse(l < 0.5, l * (1.0 + s), l + s - (l * s))
    p <- 2.0 * l - q
    r <- .hue_to_rgb(p, q, h + 1 / 3)
    g <- .hue_to_rgb(p, q, h)
    b <- .hue_to_rgb(p, q, h - 1 / 3)
  }
  col <- grDevices::rgb(r, g, b)
  return(col)
}

.hue_to_rgb <- function(p, q, t) {
  if (t < 0) {
    t <- t + 1.0
  }
  if (t > 1) {
    t <- t - 1.0
  }
  if (t < 1 / 6) {
    return(p + (q - p) * 6.0 * t)
  }
  if (t < 1 / 2) {
    return(q)
  }
  if (t < 2 / 3) {
    return(p + ((q - p) * ((2 / 3) - t) * 6))
  }
  return(p)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Diamonds
#'
#' @description This function draws diamonds on a canvas and (optionally) places two lines behind them. The diamonds can be transparent or have a random color sampled from the input.
#'
#' @usage canvas_diamonds(
#'   colors,
#'   background = "#fafafa",
#'   col.line = "black",
#'   radius = 10,
#'   alpha = 1,
#'   p = 0.2,
#'   resolution = 500
#' )
#'
#' @param colors      a string or character vector specifying the color(s) used for the artwork.
#' @param background  a character specifying the color used for the background.
#' @param col.line    a character specifying the color of the diamond borders.
#' @param radius      a positive value specifying the radius of the diamonds.
#' @param alpha       a value specifying the transparency of the diamonds. If \code{NULL} (the default), added layers become increasingly more transparent.
#' @param p           a value specifying the probability of drawing an empty diamond.
#' @param resolution  resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_diamonds(colors = colorPalette("tuscany1"))
#' }
#'
#' @export

canvas_diamonds <- function(colors,
                            background = "#fafafa",
                            col.line = "black",
                            radius = 10,
                            alpha = 1,
                            p = 0.2,
                            resolution = 500) {
  .checkUserInput(background = background, resolution = resolution)
  x <- seq(from = resolution / 5, to = resolution / 5 * 4, by = radius)
  top <- seq(from = resolution / 2 + radius, to = resolution / 5 * 4, by = radius)
  top <- c(top, top[length(top)] + radius)
  top <- c(top, seq(from = top[length(top)] - radius, to = resolution / 2 + radius, by = -radius))
  bottom <- seq(from = resolution / 2 - radius, to = resolution / 5, by = -radius)
  bottom <- c(bottom, bottom[length(bottom)] - radius)
  bottom <- c(bottom, seq(from = bottom[length(bottom)] + radius, to = resolution / 2 - radius, by = radius))
  locs <- data.frame(x = x, bottom = bottom, top = top)
  palette <- NULL
  full_canvas <- data.frame(x = numeric(), y = numeric(), type = numeric())
  for (j in seq_len(nrow(locs))) {
    rs <- ceiling((top[j] - bottom[j]) / (radius * 2)) # required squares
    for (i in 1:rs) {
      xvec <- c(locs[["x"]][j], locs[["x"]][j] + radius, locs[["x"]][j], locs[["x"]][j] - radius, locs[["x"]][j])
      yvec <- c(locs[["top"]][j] - radius * ((i - 1) * 2), (locs[["top"]][j] - radius * ((i - 1) * 2)) - radius, (locs[["top"]][j] - radius * ((i - 1) * 2)) - (radius * 2), (locs[["top"]][j] - radius * ((i - 1) * 2)) - radius, locs[["top"]][j] - radius * ((i - 1) * 2))
      col <- sample(c(NA, sample(colors, size = 1)), size = 1, prob = c(p, 1 - p))
      if (!(col %in% palette)) {
        palette <- c(palette, col)
      }
      diamond <- data.frame(x = xvec, y = yvec, type = paste0(j, i), col = col)
      full_canvas <- rbind(full_canvas, diamond)
    }
  }
  artwork <- ggplot2::ggplot(data = full_canvas, mapping = ggplot2::aes(x = x, y = y, group = type)) +
    ggplot2::xlim(c(0, resolution)) +
    ggplot2::ylim(c(0, resolution)) +
    ggplot2::geom_curve(
      data = data.frame(x = 0, y = sample(0:resolution / 2, size = 1), xend = resolution, yend = sample((resolution / 2):resolution, size = 1), type = 999),
      mapping = ggplot2::aes(x = x, y = y, xend = xend, yend = yend),
      colour = col.line, linewidth = stats::runif(1, 5, 15), curvature = stats::runif(1, 0, 0.5)
    ) +
    ggplot2::geom_curve(
      data = data.frame(x = 0, y = sample((resolution / 2):resolution, size = 1), xend = resolution, yend = sample(0:resolution / 2, size = 1), type = 999),
      mapping = ggplot2::aes(x = x, y = y, xend = xend, yend = yend),
      colour = col.line, linewidth = stats::runif(1, 5, 15), curvature = stats::runif(1, -0.5, 0)
    ) +
    ggplot2::geom_polygon(fill = full_canvas[["col"]], color = NA, alpha = alpha) +
    ggplot2::scale_fill_manual(values = palette)
  artwork <- theme_canvas(artwork, background)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw a Circle Map
#'
#' @description This function draws a circle map on a canvas. A circle map is a
#' nonlinear dynamic system that can exhibit a phenomenon known as Arnold's
#' tongue: a visualization of the frequency-locking behavior of a nonlinear
#' oscillator with a periodic external force. The tongue is a region in the
#' parameter space of the oscillator where the frequency of the oscillator
#' matches the frequency of the external force. The tongue appears as a series
#' of tongues of varying widths and shapes that can extend into regions of the
#' parameter space where the frequency locking does not occur.
#'
#' @usage canvas_circlemap(
#'   colors,
#'   left = 0,
#'   right = 12.56,
#'   bottom = 0,
#'   top = 1,
#'   iterations = 10,
#'   resolution = 1500
#' )
#'
#' @param colors      a string or character vector specifying the color(s) used
#'   for the artwork.
#' @param left        a value specifying the minimum location on the x-axis.
#' @param right       a value specifying the maximum location on the x-axis.
#' @param bottom      a value specifying the minimum location on the y-axis.
#' @param top         a value specifying the maximum location on the y-axis.
#' @param iterations  a positive integer specifying the number of iterations of
#'   the algorithm.
#' @param resolution  resolution of the artwork in pixels per row/column.
#'   Increasing the resolution increases the quality of the artwork but also
#'   increases the computation time exponentially.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @references \url{https://en.wikipedia.org/wiki/Arnold_tongue}
#' @references \url{https://linas.org/art-gallery/circle-map/circle-map.html}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' canvas_circlemap(colors = colorPalette("dark2"))
#' }
#'
#' @export

canvas_circlemap <- function(colors,
                             left = 0,
                             right = 12.56,
                             bottom = 0,
                             top = 1,
                             iterations = 10,
                             resolution = 1500) {
  .checkUserInput(resolution = resolution)
  canvas <- matrix(1, nrow = resolution, ncol = resolution)
  canvas <- cpp_circlemap(canvas, left, right, bottom, top, iterations)
  canvas <- (canvas / iterations) / length(colors)
  full_canvas <- .unraster(canvas, names = c("y", "x", "z"))
  artwork <- ggplot2::ggplot(data = full_canvas, ggplot2::aes(x = x, y = y, fill = z)) +
    ggplot2::geom_raster(interpolate = TRUE, alpha = 0.9) +
    ggplot2::xlim(c(0, resolution + 1)) +
    ggplot2::ylim(c(0, resolution + 1)) +
    ggplot2::scale_fill_gradientn(colours = colors)
  artwork <- theme_canvas(artwork)
  return(artwork)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

cpp_ant <- function(canvas, directions, iterations, resolution) {
    .Call('_aRtsy_cpp_ant', PACKAGE = 'aRtsy', canvas, directions, iterations, resolution)
}

cpp_chladni <- function(x, y, waves) {
    .Call('_aRtsy_cpp_chladni', PACKAGE = 'aRtsy', x, y, waves)
}

cpp_circlemap <- function(canvas, left, right, bottom, top, iters) {
    .Call('_aRtsy_cpp_circlemap', PACKAGE = 'aRtsy', canvas, left, right, bottom, top, iters)
}

cpp_collatz_sequence <- function(x) {
    .Call('_aRtsy_cpp_collatz_sequence', PACKAGE = 'aRtsy', x)
}

cpp_collatz <- function(empty, series, even, odd) {
    .Call('_aRtsy_cpp_collatz', PACKAGE = 'aRtsy', empty, series, even, odd)
}

cpp_flame <- function(canvas, iterations, resolution, edge, blend, weighted, post, final, extra, colors, functions, funcWeights, funcPars, variations, varWeights, varParams, postPars, finalPars, extraPars, bsym) {
    .Call('_aRtsy_cpp_flame', PACKAGE = 'aRtsy', canvas, iterations, resolution, edge, blend, weighted, post, final, extra, colors, functions, funcWeights, funcPars, variations, varWeights, varParams, postPars, finalPars, extraPars, bsym)
}

cpp_flow <- function(canvas, angles, lines, iters, ncolors, left, right, top, bottom, stepmax) {
    .Call('_aRtsy_cpp_flow', PACKAGE = 'aRtsy', canvas, angles, lines, iters, ncolors, left, right, top, bottom, stepmax)
}

cpp_maze <- function(X, x, y) {
    .Call('_aRtsy_cpp_maze', PACKAGE = 'aRtsy', X, x, y)
}

cpp_mesh <- function(canvas, points, centers, iterations, start, order, radii, increase) {
    .Call('_aRtsy_cpp_mesh', PACKAGE = 'aRtsy', canvas, points, centers, iterations, start, order, radii, increase)
}

cpp_petri_closest <- function(attractor_x, attractor_y, nodes_x, nodes_y, attraction_distance) {
    .Call('_aRtsy_cpp_petri_closest', PACKAGE = 'aRtsy', attractor_x, attractor_y, nodes_x, nodes_y, attraction_distance)
}

cpp_petri_kill <- function(attractor_x, attractor_y, nodes_x, nodes_y, kill_distance) {
    .Call('_aRtsy_cpp_petri_kill', PACKAGE = 'aRtsy', attractor_x, attractor_y, nodes_x, nodes_y, kill_distance)
}

cpp_draw_circle <- function(center_x, center_y, diameter, n) {
    .Call('_aRtsy_cpp_draw_circle', PACKAGE = 'aRtsy', center_x, center_y, diameter, n)
}

cpp_petri_directions <- function(attractor_x, attractor_y, nodes_x, nodes_y, closest_nodes) {
    .Call('_aRtsy_cpp_petri_directions', PACKAGE = 'aRtsy', attractor_x, attractor_y, nodes_x, nodes_y, closest_nodes)
}

cpp_phyllotaxis <- function(iter, a, p) {
    .Call('_aRtsy_cpp_phyllotaxis', PACKAGE = 'aRtsy', iter, a, p)
}

cpp_planet <- function(canvas, resolution, radius, xcenter, ycenter, threshold, iterations, ncolors, colorsused, starprob, fade, lightright) {
    .Call('_aRtsy_cpp_planet', PACKAGE = 'aRtsy', canvas, resolution, radius, xcenter, ycenter, threshold, iterations, ncolors, colorsused, starprob, fade, lightright)
}

cpp_polylines <- function(canvas, ratio, iters, rows, cols) {
    .Call('_aRtsy_cpp_polylines', PACKAGE = 'aRtsy', canvas, ratio, iters, rows, cols)
}

cpp_recaman <- function(n, start, increment) {
    .Call('_aRtsy_cpp_recaman', PACKAGE = 'aRtsy', n, start, increment)
}

cpp_smoke <- function(canvas, coords, color_mat, init, algorithm, shape, all_colors) {
    .Call('_aRtsy_cpp_smoke', PACKAGE = 'aRtsy', canvas, coords, color_mat, init, algorithm, shape, all_colors)
}

cpp_splits <- function(x, xend, y, yend, col, sda, ncols, iterations) {
    .Call('_aRtsy_cpp_splits', PACKAGE = 'aRtsy', x, xend, y, yend, col, sda, ncols, iterations)
}

cpp_squares <- function(X, neighbors, s, cuts, ratio) {
    .Call('_aRtsy_cpp_squares', PACKAGE = 'aRtsy', X, neighbors, s, cuts, ratio)
}

cpp_strokes <- function(canvas, neighbors, s, p) {
    .Call('_aRtsy_cpp_strokes', PACKAGE = 'aRtsy', canvas, neighbors, s, p)
}

cpp_swirls <- function(heightMap, iterations, n, resolution, ncols, lwd, freq) {
    .Call('_aRtsy_cpp_swirls', PACKAGE = 'aRtsy', heightMap, iterations, n, resolution, ncols, lwd, freq)
}

cpp_tiles <- function(canvas, conv, diffa, diffb, feedrate, killrate, iterations) {
    .Call('_aRtsy_cpp_tiles', PACKAGE = 'aRtsy', canvas, conv, diffa, diffb, feedrate, killrate, iterations)
}

cpp_turmite <- function(canvas, iters, row, col, p) {
    .Call('_aRtsy_cpp_turmite', PACKAGE = 'aRtsy', canvas, iters, row, col, p)
}

cpp_watercolors <- function(x, y, s, maxdepth, resolution) {
    .Call('_aRtsy_cpp_watercolors', PACKAGE = 'aRtsy', x, y, s, maxdepth, resolution)
}

cpp_knn <- function(x, y, z, newx, newy, k) {
    .Call('_aRtsy_cpp_knn', PACKAGE = 'aRtsy', x, y, z, newx, newy, k)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Watercolors
#'
#' @description This function paints watercolors on a canvas.
#'
#' @usage canvas_watercolors(
#'   colors,
#'   background = "#fafafa",
#'   layers = 50,
#'   depth = 2,
#'   resolution = 250
#' )
#'
#' @param colors       a string specifying the color used for the artwork.
#' @param background   a character specifying the color used for the background.
#' @param layers       the number of layers of each color.
#' @param depth        the maximum depth of the recursive algorithm.
#' @param resolution   resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @references \url{https://tylerxhobbs.com/essays/2017/a-generative-approach-to-simulating-watercolor-paints}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_watercolors(colors = colorPalette("tuscany2"))
#' }
#'
#' @export

canvas_watercolors <- function(colors,
                               background = "#fafafa",
                               layers = 50,
                               depth = 2,
                               resolution = 250) {
  .checkUserInput(resolution = resolution, background = background)
  nlayers <- length(colors)
  plotData <- data.frame(x = numeric(), y = numeric(), s = numeric(), z = numeric())
  colorSequence <- rep(colors, times = ceiling(layers / 5), each = 5)
  labelSequence <- rep(seq_along(colors), times = ceiling(layers / 5), each = 5)
  corners <- sample(3:10, size = nlayers, replace = TRUE)
  basePolygons <- list()
  for (i in seq_len(nlayers)) {
    basePolygons[[i]] <- .watercolors_basepolygon(i, nlayers, corners[i], resolution)
  }
  for (i in seq_along(colorSequence)) {
    canvas <- basePolygons[[labelSequence[i]]]
    canvas <- cpp_watercolors(canvas[, 1], canvas[, 2], canvas[, 3], depth, resolution)
    canvas <- cbind(canvas, z = i)
    plotData <- rbind(plotData, canvas)
  }
  artwork <- ggplot2::ggplot(data = plotData, mapping = ggplot2::aes(x = x, y = y, fill = factor(z))) +
    ggplot2::geom_polygon(alpha = 0.04) +
    ggplot2::scale_fill_manual(values = colorSequence) +
    ggplot2::xlim(c(0, resolution)) +
    ggplot2::ylim(0, resolution)
  artwork <- theme_canvas(artwork, background)
  return(artwork)
}

.watercolors_basepolygon <- function(color, nlayers, corners, resolution) {
  if (nlayers == 1) {
    xmid <- (resolution / 2)
    ymid <- (resolution / 2)
  } else {
    xmid <- (resolution / 2) + (resolution / 3) * cos(2 * pi * color / nlayers) * stats::rnorm(1, mean = 0.75, sd = 0.25)
    ymid <- (resolution / 2) + (resolution / 3) * sin(2 * pi * color / nlayers) * stats::rnorm(1, mean = 0.75, sd = 0.25)
  }
  radiusx <- sample((resolution / 3):(resolution / 7.5), size = 1)
  radiusy <- sample((resolution / 3):(resolution / 7.5), size = 1)
  polyx <- xmid + radiusx * cos(2 * pi * 1:corners / corners)
  polyy <- ymid + radiusy * sin(2 * pi * 1:corners / corners)
  coords <- data.frame(x = polyx, y = polyy)
  coords[nrow(coords) + 1, ] <- coords[1, ]
  varsegments <- stats::rnorm(nrow(coords), mean = 6, sd = 1.5)
  canvas <- data.frame(x = coords[["x"]], y = coords[["y"]], s = varsegments)
  canvas <- cpp_watercolors(canvas[, 1], canvas[, 2], canvas[, 3], 5, resolution)
  return(canvas)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Blacklights
#'
#' @description This function draws Blacklights on a canvas using a Support
#'   Vector Machine (SVM) algorithm. SVM's are a type of supervised learning
#'   algorithm that can be used for classification and regression purposes. The
#'   main goal of the SVM technique is to find a hyperplane (decision boundary)
#'   that best separates the values in the training dataset. This function draws
#'   the predictions from the SVM algorithm fitted on a randomly generated
#'   continuous training data set.
#'
#' @usage canvas_blacklight(
#'   colors,
#'   n = 1000,
#'   resolution = 500
#' )
#'
#' @param colors      a string or character vector specifying the color(s) used
#'   for the artwork.
#' @param n           a positive integer specifying the number of random data
#'   points to generate.
#' @param resolution  resolution of the artwork in pixels per row/column.
#'   Increasing the resolution increases the quality of the artwork but also
#'   increases the computation time exponentially.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @references \url{https://en.wikipedia.org/wiki/Support-vector_machine}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_blacklight(colors = colorPalette("tuscany2"))
#' }
#'
#' @export

canvas_blacklight <- function(colors,
                              n = 1000,
                              resolution = 500) {
  .checkUserInput(resolution = resolution)
  canvas <- .noise(dims = c(resolution, resolution), n = n, type = "svm")
  canvas <- .unraster(canvas, names = c("x", "y", "z"))
  artwork <- ggplot2::ggplot(data = canvas, mapping = ggplot2::aes(x = x, y = y, fill = z)) +
    ggplot2::geom_raster(interpolate = TRUE) +
    ggplot2::xlim(c(0, resolution + 1)) +
    ggplot2::ylim(c(0, resolution + 1)) +
    ggplot2::scale_fill_gradientn(colours = colors)
  artwork <- theme_canvas(artwork)
  return(artwork)
}

library(ggpubr)
library(aRtsy)

palettes <- c(
  "azul",
  "bell",
  "blackwhite",
  "blossom",
  "boogy1",
  "boogy2",
  "boogy3",
  "dark1",
  "dark2",
  "dark3",
  "flag",
  "flora",
  "house",
  "gogh",
  "jasp",
  "jfa",
  "jungle",
  "klimt",
  "kpd",
  "lava",
  "origami",
  "mixer1",
  "mixer2",
  "mixer3",
  "mixer4",
  "nature",
  "neo1",
  "neo2",
  "neo3",
  "neon1",
  "neon2",
  "retro1",
  "retro2",
  "retro3",
  "retro4",
  "shell1",
  "shell2",
  "shell3",
  "sooph",
  "sky",
  "tuscany1",
  "tuscany2",
  "tuscany3",
  "vrolik1",
  "vrolik2",
  "vrolik3",
  "vrolik4",
  "vrolik5"
)

ncol <- 3
n <- length(palettes)
nrow <- ceiling(n / ncol)

plotList <- list()
for (i in seq_along(palettes)) {
  palette <- aRtsy::colorPalette(palettes[i])
  d <- data.frame(y = rep(1, length(palette)), col = palette)
  p <- ggplot2::ggplot(data = d, mapping = ggplot2::aes(y = y, group = col)) +
    ggplot2::geom_bar(fill = d[["col"]], col = "black", linewidth = 0.1) +
    ggplot2::xlab(palettes[i]) +
    ggplot2::theme(
      axis.title.x = ggplot2::element_text(size = 9),
      axis.title.y = ggplot2::element_blank(),
      axis.text = ggplot2::element_blank(),
      axis.ticks = ggplot2::element_blank(),
      axis.line = ggplot2::element_blank(),
      panel.background = ggplot2::element_rect(fill = NA, colour = NA),
      legend.position = "none",
      panel.border = ggplot2::element_blank(),
      panel.grid = ggplot2::element_blank(),
      strip.background = ggplot2::element_blank(),
      strip.text = ggplot2::element_blank()
    )
  plotList[[i]] <- p
}

plot <- do.call("ggarrange", c(plotList, nrow = nrow, ncol = ncol)) +
  ggplot2::theme(panel.background = ggplot2::element_rect(fill = NA, colour = NA))
ggplot2::ggsave(filename = "./man/figures/colors.svg", plot = plot, width = 12.5, height = nrow * 1.25, units = "cm")
ggplot2::ggsave(filename = "./man/figures/colors.pdf", plot = plot, width = 12.5, height = nrow * 1.25, units = "cm")

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw a Phyllotaxis
#'
#' @description This function draws a phyllotaxis which resembles the arrangement of leaves on a plant stem.
#'
#' @usage canvas_phyllotaxis(
#'   colors,
#'   background = "#fafafa",
#'   iterations = 10000,
#'   angle = 137.5,
#'   size = 0.01,
#'   alpha = 1,
#'   p = 0.5
#' )
#'
#' @param colors         a string or character vector specifying the color(s) used for the artwork.
#' @param background     a character specifying the color used for the background.
#' @param iterations     the number of iterations of the algorithm.
#' @param angle          the angle at which to place the artwork.
#' @param size           the size of the lines.
#' @param alpha          transparency of the points.
#' @param p              probability of drawing a point on each iteration.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @references \url{https://en.wikipedia.org/wiki/Phyllotaxis}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_phyllotaxis(colors = colorPalette("tuscany1"))
#' }
#'
#' @export

canvas_phyllotaxis <- function(colors,
                               background = "#fafafa",
                               iterations = 10000,
                               angle = 137.5,
                               size = 0.01,
                               alpha = 1,
                               p = 0.5) {
  .checkUserInput(background = background, iterations = iterations)
  canvas <- cpp_phyllotaxis(iterations, angle, p)
  canvas[["z"]] <- seq_len(nrow(canvas))
  artwork <- ggplot2::ggplot(data = canvas, mapping = ggplot2::aes(x = x, y = y, color = z)) +
    ggplot2::geom_point(size = size, alpha = alpha) +
    ggplot2::scale_color_gradientn(colors = colors)
  artwork <- theme_canvas(artwork, background = background)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Save a Canvas to an External Device
#'
#' @description This function is a wrapper around \code{ggplot2::ggsave}. It provides a suggested export with square dimensions for a canvas created using the \code{aRtsy} package.
#'
#' @usage saveCanvas(plot, filename, width = 7, height = 7, dpi = 300)
#'
#' @param plot       a ggplot2 object to be saved.
#' @param filename   the filename of the export.
#' @param width      the width of the artwork in cm.
#' @param height     the height of the artwork in cm.
#' @param dpi        the \code{dpi} (dots per inch) of the file.
#'
#' @return No return value, called for saving plots.
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords canvas save
#'
#' @export

saveCanvas <- function(plot, filename, width = 7, height = 7, dpi = 300) {
  plot <- plot + ggplot2::theme(plot.margin = ggplot2::unit(rep(-1.25, 4), "lines"))
  ggplot2::ggsave(plot = plot, filename = filename, width = width, height = height, units = "cm", dpi = dpi)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Swirls
#'
#' @description This function draws swirling stripes on a canvas by simulating a
#'   particle system.
#'
#' @usage canvas_swirls(
#'   colors,
#'   background = "#fafafa",
#'   iterations = 250,
#'   n = 250,
#'   curvature = 0.005,
#'   lwd = 0.1,
#'   resolution = 500
#' )
#'
#' @param colors      a character (vector) specifying the color(s) used for the
#'   artwork.
#' @param background  a character specifying the color used for the background.
#' @param iterations  a positive integer specifying the number of iterations of
#'   the algorithm.
#' @param n           a positive integer specifying the number of particles.
#' @param curvature   a positive number specifying the curvature of the lines.
#'   Larger values imply relatively curved lines, while lower values produce
#'   relatively straight lines.
#' @param lwd         expansion factor for the line width.
#' @param resolution  resolution of the artwork in pixels per row/column.
#'   Increasing the resolution increases the quality of the artwork but also
#'   increases the computation time exponentially.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @references \url{https://mattdesl.svbtle.com/generative-art-with-nodejs-and-canvas}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(2)
#'
#' # Simple example
#' canvas_swirls(colors = colorPalette("origami"))
#' }
#'
#' @export

canvas_swirls <- function(colors, background = "#fafafa", iterations = 250,
                          n = 250, curvature = 0.005, lwd = 0.1, resolution = 500) {
  .checkUserInput(
    resolution = resolution, background = background, iterations = iterations
  )
  stopifnot("'curvature' must be a single value > 0 and <= 0.01" = curvature > 0 && curvature <= 0.01)
  heightMap <- .noise(dims = c(resolution, resolution), type = "perlin", limits = c(0, 255))
  palette <- c(background, colors)
  canvas <- cpp_swirls(heightMap, iterations, n, resolution, length(palette), lwd, curvature)
  canvas <- as.data.frame(canvas)
  colnames(canvas) <- c("x", "y", "xend", "yend", "color", "z", "width")
  canvas <- canvas[((canvas[["xend"]] > 0 & canvas[["xend"]] < resolution) & (canvas[["yend"]] > 0 & canvas[["yend"]] < resolution)), ]
  palette <- palette[canvas[["color"]]]
  artwork <- ggplot2::ggplot(canvas) +
    ggplot2::geom_segment(mapping = ggplot2::aes(x = x, y = y, xend = xend, yend = yend, group = z), linewidth = canvas[["width"]], color = palette, lineend = "square") +
    ggplot2::scale_x_continuous(limits = c(0, resolution)) +
    ggplot2::scale_y_continuous(limits = c(0, resolution))
  artwork <- aRtsy::theme_canvas(artwork, background = background)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Split Lines
#'
#' @description This function draws split lines.
#'
#' @usage canvas_splits(
#'   colors,
#'   background = "#fafafa",
#'   iterations = 6,
#'   sd = 0.2,
#'   lwd = 0.05,
#'   alpha = 0.5
#' )
#'
#' @param colors         a string or character vector specifying the color(s) used for the artwork.
#' @param background     a character specifying the color used for the background (and the hole).
#' @param iterations     a positive integer specifying the number of iterations of the algorithm.
#' @param sd             a numeric value specifying the standard deviation of the angle noise.
#' @param lwd            a numeric value specifying the width of the lines.
#' @param alpha          a numeric value specifying the transparency of the lines.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(2)
#'
#' # Simple example
#' canvas_splits(colors = "black", sd = 0)
#'
#' # Simple example
#' canvas_splits(colors = colorPalette("dark2"), background = "black", sd = 1)
#' }
#'
#' @export

canvas_splits <- function(colors,
                          background = "#fafafa",
                          iterations = 6,
                          sd = 0.2,
                          lwd = 0.05,
                          alpha = 0.5) {
  .checkUserInput(iterations = iterations, background = background)
  stopifnot("'sd' must be >= 0" = sd >= 0)
  line <- data.frame(
    x = c(0, 1, 1, 0), xend = c(1, 1, 0, 0),
    y = c(0, 0, 1, 1), yend = c(0, 1, 1, 0),
    col = sample(seq_along(colors), size = 4, replace = TRUE)
  )
  canvas <- cpp_splits(line[["x"]], line[["xend"]], line[["y"]], line[["yend"]], line[["col"]], sd, length(colors), iterations)
  breaks <- range(c(canvas[["x"]], canvas[["xend"]], canvas[["y"]], canvas[["yend"]]))
  p <- ggplot2::ggplot(data = canvas) +
    ggplot2::geom_segment(mapping = ggplot2::aes(x = x, y = y, xend = xend, yend = yend, col = factor(col)), linewidth = lwd, alpha = alpha) +
    ggplot2::scale_x_continuous(limits = breaks) +
    ggplot2::scale_y_continuous(limits = breaks) +
    ggplot2::scale_color_manual(values = colors)
  p <- theme_canvas(p, background = background)
  return(p)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Meshes
#'
#' @description This function draws one or more rotating circular morphing meshes on the canvas.
#'
#' @usage canvas_mesh(
#'   colors,
#'   background = "#fafafa",
#'   transform = c("perlin", "fbm", "simplex", "cubic",
#'                 "worley", "knn", "rf", "svm"),
#'   lines = 500,
#'   iterations = 500,
#'   mixprob = 0
#' )
#'
#' @param colors         a string or character vector specifying the color(s) used for the artwork.
#' @param background     a character specifying the color used for the background (and the hole).
#' @param transform      a character specifying the type of transformation to use for the radius.
#' @param lines          an integer specifying the number of lines to darw.
#' @param iterations     a positive integer specifying the number of iterations of the algorithm.
#' @param mixprob        a value between 0 and 1 specifying the probability of a line segment getting another color.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @references \url{https://web.archive.org/web/20231130151102/http://rectangleworld.com/blog/archives/462}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(2)
#'
#' # Simple example
#' canvas_mesh(colors = colorPalette("origami"))
#' }
#'
#' @export

canvas_mesh <- function(colors,
                        background = "#fafafa",
                        transform = c(
                          "perlin", "fbm", "simplex", "cubic",
                          "worley", "knn", "rf", "svm"
                        ),
                        lines = 500,
                        iterations = 500,
                        mixprob = 0) {
  transform <- match.arg(transform)
  .checkUserInput(iterations = iterations, background = background)
  artwork <- ggplot2::ggplot()
  miny <- Inf
  maxy <- -Inf
  for (j in seq_along(colors)) {
    circle_points <- seq(from = -pi, to = pi, length.out = lines)
    start <- stats::runif(1, min = -10, max = 10)
    centers <- data.frame(x = 0:iterations, y = c(start, start + .bmline(n = iterations, lwd = stats::runif(1, min = 1, max = 10))))
    circle_centers <- predict(stats::loess(y ~ x, data = centers), newdata = centers)
    if (transform == "fbm") {
      radius <- data.frame(x = 1:lines, y = .bmline(n = lines, lwd = stats::runif(1, min = 0.5, max = 1)))
      circle_radius <- predict(stats::loess(y ~ x, data = radius), newdata = radius)
    } else {
      circle_radius <- c(.noise(c(1, lines), type = transform, limits = c(0.5, 1)))
    }
    radius_increase <- data.frame(x = 1:lines, y = stats::rnorm(lines, mean = 0, sd = stats::runif(1, min = 0.01, max = 0.5)))
    circle_radius_increase <- predict(stats::loess(y ~ x, data = radius_increase), newdata = radius_increase)
    x <- rep(0:iterations, each = lines) + 0.75 * cos(circle_points)
    mesh <- cpp_mesh(
      canvas = matrix(NA, nrow = lines * (iterations + 1), ncol = 2),
      points = circle_points,
      centers = circle_centers,
      iterations = iterations,
      start = start,
      order = seq_len(lines),
      radii = circle_radius,
      increase = circle_radius_increase
    )
    if (mixprob > 0) {
      probs <- rep(1, length(colors))
      probs[j] <- probs[j] + 1 / mixprob
      col <- sample(colors, size = length(mesh[, 1]), replace = TRUE, prob = probs)
    } else {
      col <- colors[j]
    }
    df <- data.frame(x = x, y = mesh[, 1], z = mesh[, 2], col = col)
    if (min(df[["y"]]) < miny) miny <- min(df[["y"]])
    if (max(df[["y"]]) > maxy) maxy <- max(df[["y"]])
    artwork <- artwork + ggplot2::geom_line(
      data = df,
      mapping = ggplot2::aes(x = x, y = y, group = z, col = col),
      alpha = 0.2, linewidth = 0.05
    )
  }
  artwork <- artwork + ggplot2::scale_color_manual(values = colors) +
    ggplot2::scale_x_continuous(limits = c(-1, iterations + 1)) +
    ggplot2::scale_y_continuous(limits = c(miny - abs(maxy - miny) / 10, maxy + abs(maxy - miny) / 10))
  artwork <- theme_canvas(artwork, background = background)
  return(artwork)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Mazes
#'
#' @description This function draws a maze on a canvas.
#'
#' @usage canvas_maze(
#'   color = "#fafafa",
#'   walls = "black",
#'   background = "#fafafa",
#'   resolution = 20,
#'   polar = FALSE
#' )
#'
#' @param color       a character specifying the color used for the artwork.
#' @param walls       a character specifying the color used for the walls of the maze.
#' @param background  a character specifying the color used for the background.
#' @param resolution  resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.
#' @param polar       logical, whether to use polar coordinates. Warning, this increases display and saving time dramatically.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @references \url{https://github.com/matfmc/mazegenerator}
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_maze(color = "#fafafa")
#' }
#'
#' @export

canvas_maze <- function(color = "#fafafa",
                        walls = "black",
                        background = "#fafafa",
                        resolution = 20,
                        polar = FALSE) {
  canvas <- matrix(0, resolution, resolution)
  x <- sample(2:resolution, size = 1)
  y <- sample(2:resolution, size = 1)
  maze <- cpp_maze(canvas, x, y)
  full_canvas <- .maze_connect(maze, canvas)
  full_canvas <- .unraster(full_canvas, names = c("x", "y", "z"))
  artwork <- ggplot2::ggplot(data = full_canvas, ggplot2::aes(x = x, y = y, fill = factor(z), color = factor(z))) +
    ggplot2::geom_tile() +
    ggplot2::scale_fill_manual(values = c(color, walls)) +
    ggplot2::scale_color_manual(values = c(color, walls)) +
    ggplot2::ylim(c(0, max(full_canvas[["y"]]) + 1))
  if (!polar) {
    artwork <- artwork + ggplot2::xlim(c(0, max(full_canvas[["x"]]) + 1))
  } else {
    artwork <- artwork + ggplot2::coord_polar(start = stats::runif(1, 0, 2 * pi), clip = "off") +
      ggplot2::xlim(c(0.5, max(full_canvas[["x"]]) + 0.5))
  }
  artwork <- theme_canvas(artwork, background = background)
  return(artwork)
}

.maze_connect <- function(maze, canvas) {
  # From https://github.com/matfmc/mazegenerator
  for (i in seq_len(nrow(maze))) {
    if (i == 1) {
      maze[i, 3:4] <- (maze[i, 1:2] - maze[i + 1, 1:2])
    } else if (i == nrow(maze)) {
      maze[i, 3:4] <- (maze[i, 1:2] - maze[i - 1, 1:2])
    } else {
      maze[i, 3:4] <- (maze[i, 1:2] - maze[i + 1, 1:2])
      maze[i, 5:6] <- (maze[i, 1:2] - maze[i - 1, 1:2])
    }
  }
  maze[maze[, 3] == -1 & maze[, 4] == 0, "start"] <- "A"
  maze[maze[, 3] == 0 & maze[, 4] == -1, "start"] <- "B"
  maze[maze[, 3] == 1 & maze[, 4] == 0, "start"] <- "C"
  maze[maze[, 3] == 0 & maze[, 4] == 1, "start"] <- "D"
  maze[(maze[, 5] == -1 & maze[, 6] == 0) & !is.na(maze[, 5]), "end"] <- "A"
  maze[(maze[, 5] == 0 & maze[, 6] == -1) & !is.na(maze[, 5]), "end"] <- "B"
  maze[(maze[, 5] == 1 & maze[, 6] == 0) & !is.na(maze[, 5]), "end"] <- "C"
  maze[(maze[, 5] == 0 & maze[, 6] == 1) & !is.na(maze[, 5]), "end"] <- "D"
  maze[["conec"]] <- paste0(maze[["start"]], maze[["end"]])
  maze <- maze[, c(1, 2, 9)]
  maze[maze[["conec"]] == "CD" | maze[["conec"]] == "DC" | maze[["conec"]] == "DNA" | maze[["conec"]] == "CNA" | maze[["conec"]] == "CC" | maze[["conec"]] == "DD", "cell"] <- 1
  maze[maze[["conec"]] == "AC" | maze[["conec"]] == "CA" | maze[["conec"]] == "AD" | maze[["conec"]] == "DA" | maze[["conec"]] == "ANA" | maze[["conec"]] == "AA", "cell"] <- 2
  maze[maze[["conec"]] == "BC" | maze[["conec"]] == "CB" | maze[["conec"]] == "BD" | maze[["conec"]] == "DB" | maze[["conec"]] == "BNA" | maze[["conec"]] == "BB", "cell"] <- 3
  maze[maze[["conec"]] == "AB" | maze[["conec"]] == "BA", "cell"] <- 4
  for (i in seq_len(nrow(maze))) {
    if (canvas[maze[["x"]][i], maze[["y"]][i]] == 0) {
      canvas[maze[["x"]][i], maze[["y"]][i]] <- maze[["cell"]][i]
    } else if (canvas[maze[["x"]][i], maze[["y"]][i]] == 4) {
      next
    } else if (canvas[maze[["x"]][i], maze[["y"]][i]] == 2 && maze[["cell"]][i] == 3) {
      canvas[maze[["x"]][i], maze[["y"]][i]] <- 4
    } else if (canvas[maze[["x"]][i], maze[["y"]][i]] == 3 && maze[["cell"]][i] == 2) {
      canvas[maze[["x"]][i], maze[["y"]][i]] <- 4
    } else if (canvas[maze[["x"]][i], maze[["y"]][i]] == 1) {
      canvas[maze[["x"]][i], maze[["y"]][i]] <- maze[["cell"]][i]
    }
  }
  cell_type_0 <- matrix(c(0, 0, 0, 0, 0, 0, 0, 0, 0), ncol = 3, byrow = TRUE)
  cell_type_1 <- matrix(c(0, 0, 1, 0, 0, 1, 1, 1, 1), ncol = 3, byrow = TRUE)
  cell_type_2 <- matrix(c(0, 0, 1, 0, 0, 1, 0, 0, 1), ncol = 3, byrow = TRUE)
  cell_type_3 <- matrix(c(0, 0, 0, 0, 0, 0, 1, 1, 1), ncol = 3, byrow = TRUE)
  cell_type_4 <- matrix(c(0, 0, 0, 0, 0, 0, 0, 0, 1), ncol = 3, byrow = TRUE)
  subCanvas <- as.list(canvas)
  dim(subCanvas) <- c(nrow(canvas), ncol(canvas))
  subCanvas[, ][subCanvas[, ] %in% 0] <- list(cell_type_0)
  subCanvas[, ][subCanvas[, ] %in% 1] <- list(cell_type_1)
  subCanvas[, ][subCanvas[, ] %in% 2] <- list(cell_type_2)
  subCanvas[, ][subCanvas[, ] %in% 3] <- list(cell_type_3)
  subCanvas[, ][subCanvas[, ] %in% 4] <- list(cell_type_4)
  b <- do.call(cbind, subCanvas[1, seq_len(ncol(canvas))])
  for (i in 2:nrow(canvas)) {
    a <- do.call(cbind, subCanvas[i, seq_len(ncol(canvas))])
    b <- rbind(b, a)
  }
  full_canvas <- t(rbind(1, cbind(1, b)))
  return(full_canvas)
}

# Copyright (C) 2021-2023 Koen Derks

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Draw Polygons and Lines
#'
#' @description This function draws many points on the canvas and connects these points into a polygon. After repeating this for all the colors, the edges of all polygons are drawn on top of the artwork.
#'
#' @usage canvas_polylines(
#'   colors,
#'   background = "#fafafa",
#'   ratio = 0.5,
#'   iterations = 1000,
#'   size = 0.1,
#'   resolution = 500
#' )
#'
#' @param colors      a string or character vector specifying the color(s) used for the artwork.
#' @param background  a character specifying the color used for the lines.
#' @param ratio       a positive value specifying the width of the polygons. Larger ratios cause more overlap.
#' @param iterations  a positive integer specifying the number of iterations of the algorithm.
#' @param size        a positive value specifying the size of the borders.
#' @param resolution  resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.
#'
#' @return A \code{ggplot} object containing the artwork.
#'
#' @author Koen Derks, \email{koen-derks@hotmail.com}
#'
#' @keywords artwork canvas
#'
#' @seealso \code{colorPalette}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#'
#' # Simple example
#' canvas_polylines(colors = colorPalette("retro1"))
#' }
#'
#' @export

canvas_polylines <- function(colors,
                             background = "#fafafa",
                             ratio = 0.5,
                             iterations = 1000,
                             size = 0.1,
                             resolution = 500) {
  .checkUserInput(resolution = resolution, iterations = iterations)
  alphas <- seq(from = 1, to = 0.1, length.out = length(colors))
  full_canvas <- data.frame(x = numeric(), y = numeric(), type = character())
  for (i in seq_along(colors)) {
    mat <- cpp_polylines(matrix(NA, nrow = iterations, ncol = 2), ratio, iterations, resolution, resolution)
    polygon <- data.frame(x = mat[, 1], y = mat[, 2], type = rep(colors[i], iterations))
    full_canvas <- rbind(full_canvas, polygon)
  }
  artwork <- ggplot2::ggplot(data = full_canvas, mapping = ggplot2::aes(x = x, y = y, fill = type)) +
    ggplot2::xlim(c(0, resolution)) +
    ggplot2::ylim(c(0, resolution)) +
    ggplot2::geom_polygon(color = NA, alpha = rep(alphas, each = iterations)) +
    ggplot2::geom_path(color = background, linewidth = size) +
    ggplot2::scale_fill_manual(values = colors)
  artwork <- theme_canvas(artwork, background)
  return(artwork)
}

## aRtsy version 1.0.0
This is a submission for aRtsy version 1.0.0. In this version I have:

* Fixed a bug in the c++ code.
* Improved documentation.

## Test environments
* Windows install, (on github actions) R release
* OS X install (on github actions), R release
* Linux install (on github actions), R release

## R CMD check results
There were no ERRORs or WARNINGs.

On linux, there was 1 NOTE:

* checking installed package size ... NOTE. installed size is 20.5Mb. sub-directories of 1Mb or more: libs 20.0Mb

## Downstream dependencies
There is currently 1 downstream dependency for this package.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_turmite.R
\name{canvas_turmite}
\alias{canvas_turmite}
\title{Draw Turmites}
\usage{
canvas_turmite(
  colors,
  background = "#fafafa",
  p = 0,
  iterations = 1000000,
  resolution = 500,
  noise = FALSE
)
}
\arguments{
\item{colors}{a character specifying the color used for the artwork. The number of colors determines the number of turmites.}

\item{background}{a character specifying the color used for the background.}

\item{p}{a value specifying the probability of a state switch within the turmite.}

\item{iterations}{a positive integer specifying the number of iterations of the algorithm.}

\item{resolution}{resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.}

\item{noise}{logical. Whether to add k-nn noise to the artwork. Note that adding noise increases computation time significantly in large dimensions.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function paints a turmite. A turmite is a Turing machine which has an orientation in addition to a current state and a "tape" that consists of a two-dimensional grid of cells.
}
\details{
The turmite algorithm consists of the following steps: 1) turn on the spot (left, right, up, down) 2) change the color of the square 3) move forward one square.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_turmite(colors = colorPalette("dark2"))
}

}
\references{
\url{https://en.wikipedia.org/wiki/Turmite}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_function.R
\name{canvas_function}
\alias{canvas_function}
\title{Draw Functions}
\usage{
canvas_function(
  colors,
  background = "#fafafa",
  by = 0.01,
  polar = TRUE,
  formula = NULL
)
}
\arguments{
\item{colors}{a string specifying the color used for the artwork.}

\item{background}{a character specifying the color used for the background.}

\item{by}{a value specifying the step size between consecutive points.}

\item{polar}{logical. Whether to draw the function with polar coordinates.}

\item{formula}{optional, a named list with 'x' and 'y' as structured in the example. If \code{NULL} (default), chooses a function with random parameters.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function paints functions with random parameters on a canvas.
}
\examples{
\donttest{
set.seed(10)

# Simple example
canvas_function(colors = colorPalette("tuscany1"))

# Advanced example
formula <- list(
  x = quote(x_i^2 - sin(y_i^2)),
  y = quote(y_i^3 - cos(x_i^2))
)
canvas_function(colors = "firebrick", formula = formula)
}

}
\references{
\url{https://github.com/cutterkom/generativeart}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_chladni.R
\name{canvas_chladni}
\alias{canvas_chladni}
\title{Draw Chladni Figures}
\usage{
canvas_chladni(
  colors,
  waves = 5,
  warp = 0,
  resolution = 500,
  angles = NULL,
  distances = NULL,
  flatten = FALSE
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used
for the artwork.}

\item{waves}{a character specifying the number of randomly sampled
waves, or an integer vector of waves to be summed.}

\item{warp}{a numeric value specifying the maximum warping distance
for each point. If \code{warp = 0} (the default), no warping is performed.}

\item{resolution}{resolution of the artwork in pixels per row/column.
Increasing the resolution increases the quality of the artwork but also
increases the computation time exponentially.}

\item{angles}{optional, a resolution x resolution matrix containing the
angles for the warp, or a character indicating the type of noise to use
(\code{svm}, \code{knn}, \code{rf}, \code{perlin}, \code{cubic},
\code{simplex}, or \code{worley}). If \code{NULL} (the default), the noise
type is chosen randomly.}

\item{distances}{optional, a resolution x resolution matrix containing the
distances for the warp, or a character indicating the type of noise to use
(\code{svm}, \code{knn}, \code{rf}, \code{perlin}, \code{cubic},
\code{simplex}, or \code{worley}). If \code{NULL} (the default), the noise
type is chosen randomly.}

\item{flatten}{logical, should colors be flattened after being assigned
to a point.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws Chladni figures on a canvas. Named after
  Ernst Chladni, an 18th century physicist who first discovered them, Chladni
  figures are patterns that arise from the vibrations of a two-dimensional
  plate, typically covered with a thin layer of sand or powder. The Chladni
  figures are created by varying the frequency of vibration applied to the
  plate. In this implementation, the grid underneath the plate can be
  transformed using a domain warping technique. The basic idea behind domain
  warping is to apply a series of transformations to the input grid to create
  a more complex and interesting output.
}
\examples{
\donttest{
set.seed(2)

# Simple example
canvas_chladni(colors = colorPalette("origami"))

# Advanced example
canvas_chladni(
  colors = colorPalette("lava"),
  waves = c(1, 2, 3, 9),
  warp = 1
)
}

}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_smoke.R
\name{canvas_smoke}
\alias{canvas_smoke}
\title{Draw Rainbow Smoke}
\usage{
canvas_smoke(
  colors,
  init = 1,
  shape = c("bursts", "clouds"),
  algorithm = c("minimum", "average"),
  resolution = 150
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{init}{an integer larger than zero and lower  than or equal to \code{resolution^2} specifying the initial number of pixels to color on the canvas.}

\item{shape}{a character indicating the shape of the smoke. Possible options are \code{burst} and \code{clouds}.}

\item{algorithm}{a character specifying how to select a new pixel. The default option \code{minimum} selects the pixel with the smallest color difference in a single neighbor and is relatively fast. The option \code{average} selects the pixel with the smallest average color difference in all the neighbors and is relatively slow.}

\item{resolution}{resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function implements the rainbow smoke algorithm.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_smoke(colors = "all", resolution = 500)

# Advanced example
reds <- colorRampPalette(c("red", "black"))
blues <- colorRampPalette(c("goldenrod", "navyblue"))
palette <- c(reds(100), blues(100))
canvas_smoke(colors = palette, init = 3, shape = "clouds", resolution = 500)
}

}
\references{
\url{http://rainbowsmoke.hu}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_watercolors.R
\name{canvas_watercolors}
\alias{canvas_watercolors}
\title{Draw Watercolors}
\usage{
canvas_watercolors(
  colors,
  background = "#fafafa",
  layers = 50,
  depth = 2,
  resolution = 250
)
}
\arguments{
\item{colors}{a string specifying the color used for the artwork.}

\item{background}{a character specifying the color used for the background.}

\item{layers}{the number of layers of each color.}

\item{depth}{the maximum depth of the recursive algorithm.}

\item{resolution}{resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function paints watercolors on a canvas.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_watercolors(colors = colorPalette("tuscany2"))
}

}
\references{
\url{https://tylerxhobbs.com/essays/2017/a-generative-approach-to-simulating-watercolor-paints}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_polylines.R
\name{canvas_polylines}
\alias{canvas_polylines}
\title{Draw Polygons and Lines}
\usage{
canvas_polylines(
  colors,
  background = "#fafafa",
  ratio = 0.5,
  iterations = 1000,
  size = 0.1,
  resolution = 500
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{background}{a character specifying the color used for the lines.}

\item{ratio}{a positive value specifying the width of the polygons. Larger ratios cause more overlap.}

\item{iterations}{a positive integer specifying the number of iterations of the algorithm.}

\item{size}{a positive value specifying the size of the borders.}

\item{resolution}{resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws many points on the canvas and connects these points into a polygon. After repeating this for all the colors, the edges of all polygons are drawn on top of the artwork.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_polylines(colors = colorPalette("retro1"))
}

}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_circlemap.R
\name{canvas_circlemap}
\alias{canvas_circlemap}
\title{Draw a Circle Map}
\usage{
canvas_circlemap(
  colors,
  left = 0,
  right = 12.56,
  bottom = 0,
  top = 1,
  iterations = 10,
  resolution = 1500
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used
for the artwork.}

\item{left}{a value specifying the minimum location on the x-axis.}

\item{right}{a value specifying the maximum location on the x-axis.}

\item{bottom}{a value specifying the minimum location on the y-axis.}

\item{top}{a value specifying the maximum location on the y-axis.}

\item{iterations}{a positive integer specifying the number of iterations of
the algorithm.}

\item{resolution}{resolution of the artwork in pixels per row/column.
Increasing the resolution increases the quality of the artwork but also
increases the computation time exponentially.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws a circle map on a canvas. A circle map is a
nonlinear dynamic system that can exhibit a phenomenon known as Arnold's
tongue: a visualization of the frequency-locking behavior of a nonlinear
oscillator with a periodic external force. The tongue is a region in the
parameter space of the oscillator where the frequency of the oscillator
matches the frequency of the external force. The tongue appears as a series
of tongues of varying widths and shapes that can extend into regions of the
parameter space where the frequency locking does not occur.
}
\examples{
\donttest{
canvas_circlemap(colors = colorPalette("dark2"))
}

}
\references{
\url{https://en.wikipedia.org/wiki/Arnold_tongue}

\url{https://linas.org/art-gallery/circle-map/circle-map.html}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_ribbons.R
\name{canvas_ribbons}
\alias{canvas_ribbons}
\title{Draw Ribbons}
\usage{
canvas_ribbons(
  colors,
  background = "#fdf5e6",
  triangle = TRUE
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork. The number of colors determines the number of ribbons.}

\item{background}{a character specifying the color of the background.}

\item{triangle}{logical. Whether to draw the triangle that breaks the ribbon polygons.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function paints random ribbons and (optionally) a triangle in the middle.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_ribbons(colors = colorPalette("retro1"))
}

}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_segments.R
\name{canvas_segments}
\alias{canvas_segments}
\title{Draw Segments}
\usage{
canvas_segments(
  colors,
  background = "#fafafa",
  n = 250,
  p = 0.5,
  H = 0.1,
  size = 0.2
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{background}{a character specifying the color used for the background.}

\item{n}{a positive integer specifying the number of line segments to draw.}

\item{p}{a value specifying the probability of drawing a vertical line segment.}

\item{H}{a positive value specifying the scaling factor for the line segments.}

\item{size}{a positive value specifying the size of the line segments.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws line segments on a canvas. The length and direction of the line segments is determined randomly.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_segments(colors = colorPalette("dark1"))
}

}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_forest.R
\name{canvas_forest}
\alias{canvas_forest}
\title{Draw a Random Forest}
\usage{
canvas_forest(
  colors,
  n = 1000,
  resolution = 500
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{n}{a positive integer specifying the number of random data points to generate.}

\item{resolution}{resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws the predictions from a random forest algorithm trained on randomly generated categorical data.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_forest(colors = colorPalette("jungle"))
}

}
\references{
\url{https://en.wikipedia.org/wiki/Random_forest}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_ant.R
\name{canvas_ant}
\alias{canvas_ant}
\title{Draw Langton's Ant}
\usage{
canvas_ant(
  colors,
  background = "#fafafa",
  iterations = 1000000,
  resolution = 500
)
}
\arguments{
\item{colors}{a character (vector) specifying the color(s) used for the
artwork.}

\item{background}{a character specifying the color used for the background.}

\item{iterations}{a positive integer specifying the number of iterations of
the algorithm.}

\item{resolution}{resolution of the artwork in pixels per row/column.
Increasing the resolution increases the quality of the artwork but also
increases the computation time exponentially.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws Langton's Ant on a canvas. Langton's Ant is
  a two-dimensional cellular automaton that is named after its creator, Chris
  Langton. See the \code{Details} section for more specific information about
  the algorithm used in this function.
}
\details{
The algorithm for Langton's Ant involves the following steps:

\itemize{
  \item{Set up a two-dimensional grid of cells, where each cell can either be
    "colored" or "non-colored." The initial state of the grid is usually a
    single non-colored cell in the center of the grid.}
  \item{Place an "ant" on the grid at the position of the initial non-colored
    cell. The ant can move in four directions: up, down, left, or right.}
  \item{At each step of the algorithm, the ant examines the color of the cell
    it is currently on. If the cell is non-colored, the ant turns 90 degrees
    clockwise, colors the cell, and moves forward one unit.}
  \item{If the cell is colored, the ant turns 90 degrees counterclockwise,
    uncolors the cell, and moves forward one unit.}
  \item{The ant continues to move around the grid, following these rules at
    each step. If a certain number of iterations has passed, the ant chooses
    a different color which corresponds to a different combination of these
    rules.}
}
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_ant(colors = colorPalette("house"))
}

}
\references{
\url{https://en.wikipedia.org/wiki/Langtons_ant}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_strokes.R
\name{canvas_strokes}
\alias{canvas_strokes}
\title{Draw Strokes}
\usage{
canvas_strokes(
  colors,
  neighbors = 1,
  p = 0.01,
  iterations = 1,
  resolution = 500,
  side = FALSE
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{neighbors}{a positive integer specifying the number of neighbors a block considers when taking over a color. More neighbors fades the artwork.}

\item{p}{a value specifying the probability of selecting a new color at each block. A higher probability adds more noise to the artwork.}

\item{iterations}{a positive integer specifying the number of iterations of the algorithm. More iterations generally apply more fade to the artwork.}

\item{resolution}{resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.}

\item{side}{logical. Whether to put the artwork on its side.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function creates an artwork that resembles paints strokes. The algorithm is based on the simple idea that each next point on the grid has a chance to take over the color of an adjacent colored point but also has a change of generating a new color.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_strokes(colors = colorPalette("tuscany1"))
}

}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_diamonds.R
\name{canvas_diamonds}
\alias{canvas_diamonds}
\title{Draw Diamonds}
\usage{
canvas_diamonds(
  colors,
  background = "#fafafa",
  col.line = "black",
  radius = 10,
  alpha = 1,
  p = 0.2,
  resolution = 500
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{background}{a character specifying the color used for the background.}

\item{col.line}{a character specifying the color of the diamond borders.}

\item{radius}{a positive value specifying the radius of the diamonds.}

\item{alpha}{a value specifying the transparency of the diamonds. If \code{NULL} (the default), added layers become increasingly more transparent.}

\item{p}{a value specifying the probability of drawing an empty diamond.}

\item{resolution}{resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws diamonds on a canvas and (optionally) places two lines behind them. The diamonds can be transparent or have a random color sampled from the input.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_diamonds(colors = colorPalette("tuscany1"))
}

}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_collatz.R
\name{canvas_collatz}
\alias{canvas_collatz}
\title{Draw Collatz Sequences}
\usage{
canvas_collatz(
  colors,
  background = "#fafafa",
  n = 200,
  angle.even = 0.0075,
  angle.odd = 0.0145,
  side = FALSE
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{background}{a character specifying the color used for the background.}

\item{n}{a positive integer specifying the number of random starting integers to use for the lines. Can also be a vector of numbers to use as starting numbers.}

\item{angle.even}{a value specifying the angle (in radials) to use in bending the sequence at each odd number.}

\item{angle.odd}{a value specifying the angle (in radials) to use in bending the sequence at each even number.}

\item{side}{logical. Whether to put the artwork on its side.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws the Collatz conjecture on a canvas. The
  conjecture of the Collatz sequence is that no matter what positive integer
  is chosen as the starting point of the sequence, the sequence will
  eventually reach the number 1. This conjecture has been verified for all
  starting integers up to very large numbers, but it has not been proven
  mathematically. Despite its simple rule, the sequence can produce long and
  complicated chains of numbers before eventually reaching 1. See the
  \code{Details} section for more specific information about the algorithm
  used in this function.
}
\details{
The Collatz sequence, also known as the \code{3n+1} problem, is a
  sequence of numbers generated by the following rule:

\itemize{
  \item{Start with any positive integer \code{n}.}
  \item{If \code{n} is even, divide it by 2.}
  \item{If \code{n} is odd, multiply it by 3 and add 1.}
  \item{Repeat this process with the new value of \code{n}, generating a new
    number in the sequence.}
}
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_collatz(colors = colorPalette("tuscany3"))
}

}
\references{
\url{https://nl.wikipedia.org/wiki/Collatz_Conjecture}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/saveCanvas.R
\name{saveCanvas}
\alias{saveCanvas}
\title{Save a Canvas to an External Device}
\usage{
saveCanvas(plot, filename, width = 7, height = 7, dpi = 300)
}
\arguments{
\item{plot}{a ggplot2 object to be saved.}

\item{filename}{the filename of the export.}

\item{width}{the width of the artwork in cm.}

\item{height}{the height of the artwork in cm.}

\item{dpi}{the \code{dpi} (dots per inch) of the file.}
}
\value{
No return value, called for saving plots.
}
\description{
This function is a wrapper around \code{ggplot2::ggsave}. It provides a suggested export with square dimensions for a canvas created using the \code{aRtsy} package.
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{canvas}
\keyword{save}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_nebula.R
\name{canvas_nebula}
\alias{canvas_nebula}
\title{Draw Nebulas}
\usage{
canvas_nebula(
  colors,
  k = 50,
  n = 500,
  resolution = 500
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{k}{a positive integer specifying the number of nearest neighbors to consider.}

\item{n}{a positive integer specifying the number of random data points to generate.}

\item{resolution}{resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function creates an artwork from randomly generated k-nearest neighbors noise.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_nebula(colors = colorPalette("tuscany1"))
}

}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_mandelbrot.R
\name{canvas_mandelbrot}
\alias{canvas_mandelbrot}
\title{Draw the Mandelbrot Set}
\usage{
canvas_mandelbrot(
  colors,
  iterations = 100,
  zoom = 1,
  set = c("mandelbrot", "multibrot", "julia", "ship"),
  left = -2.16,
  right = 1.16,
  bottom = -1.66,
  top = 1.66,
  resolution = 500
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{iterations}{a positive integer specifying the number of iterations of the algorithm.}

\item{zoom}{a positive value specifying the amount of zoom to apply.}

\item{set}{a character indicating which fractal set to draw. Possible options are \code{mandelbrot} for the Mandelbrot set, \code{multibrot} for variations of the Mandelbrot set (aka the Multibrot sets), \code{julia} for the Julia set and \code{ship} for the Burning ship set.}

\item{left}{a value specifying the minimum location on the x-axis.}

\item{right}{a value specifying the maximum location on the x-axis.}

\item{bottom}{a value specifying the minimum location on the y-axis.}

\item{top}{a value specifying the maximum location on the y-axis.}

\item{resolution}{resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws the Mandelbrot set and other related fractal sets on the canvas.
}
\examples{
\donttest{
canvas_mandelbrot(colors = colorPalette("tuscany1"), set = "mandelbrot")
canvas_mandelbrot(colors = colorPalette("flag"), set = "julia", zoom = 2)
}

}
\references{
\url{https://en.wikipedia.org/wiki/Mandelbrot_set}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_cobweb.R
\name{canvas_cobweb}
\alias{canvas_cobweb}
\title{Draw Cobwebs}
\usage{
canvas_cobweb(
  colors,
  background = "#fafafa",
  lines = 300,
  iterations = 100
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{background}{a character specifying the color used for the background.}

\item{lines}{the number of lines to draw.}

\item{iterations}{the number of iterations of the algorithm.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws a cobweb on the canvas. The cobweb consists
of many Fibonacci spirals shifted by random noise from a normal distribution.
A Fibonacci spiral is a logarithmic spiral that is derived from the
Fibonacci sequence, a mathematical sequence where each number is the sum of
the two preceding ones. The spiral is created by connecting the corners of
squares that are sized according to the Fibonacci sequence. Specifically, if
we draw a sequence of squares with side lengths of 1, 1, 2, 3, 5, 8, 13, and
so on, each square can be arranged so that it is tangent to the previous
square at a corner. When we connect these corners with a smooth curve, the
resulting shape is the Fibonacci spiral.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_cobweb(colors = colorPalette("neon1"), background = "black")
}

}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_splits.R
\name{canvas_splits}
\alias{canvas_splits}
\title{Draw Split Lines}
\usage{
canvas_splits(
  colors,
  background = "#fafafa",
  iterations = 6,
  sd = 0.2,
  lwd = 0.05,
  alpha = 0.5
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{background}{a character specifying the color used for the background (and the hole).}

\item{iterations}{a positive integer specifying the number of iterations of the algorithm.}

\item{sd}{a numeric value specifying the standard deviation of the angle noise.}

\item{lwd}{a numeric value specifying the width of the lines.}

\item{alpha}{a numeric value specifying the transparency of the lines.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws split lines.
}
\examples{
\donttest{
set.seed(2)

# Simple example
canvas_splits(colors = "black", sd = 0)

# Simple example
canvas_splits(colors = colorPalette("dark2"), background = "black", sd = 1)
}

}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_blacklight.R
\name{canvas_blacklight}
\alias{canvas_blacklight}
\title{Draw Blacklights}
\usage{
canvas_blacklight(
  colors,
  n = 1000,
  resolution = 500
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used
for the artwork.}

\item{n}{a positive integer specifying the number of random data
points to generate.}

\item{resolution}{resolution of the artwork in pixels per row/column.
Increasing the resolution increases the quality of the artwork but also
increases the computation time exponentially.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws Blacklights on a canvas using a Support
  Vector Machine (SVM) algorithm. SVM's are a type of supervised learning
  algorithm that can be used for classification and regression purposes. The
  main goal of the SVM technique is to find a hyperplane (decision boundary)
  that best separates the values in the training dataset. This function draws
  the predictions from the SVM algorithm fitted on a randomly generated
  continuous training data set.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_blacklight(colors = colorPalette("tuscany2"))
}

}
\references{
\url{https://en.wikipedia.org/wiki/Support-vector_machine}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_mesh.R
\name{canvas_mesh}
\alias{canvas_mesh}
\title{Draw Meshes}
\usage{
canvas_mesh(
  colors,
  background = "#fafafa",
  transform = c("perlin", "fbm", "simplex", "cubic",
                "worley", "knn", "rf", "svm"),
  lines = 500,
  iterations = 500,
  mixprob = 0
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{background}{a character specifying the color used for the background (and the hole).}

\item{transform}{a character specifying the type of transformation to use for the radius.}

\item{lines}{an integer specifying the number of lines to darw.}

\item{iterations}{a positive integer specifying the number of iterations of the algorithm.}

\item{mixprob}{a value between 0 and 1 specifying the probability of a line segment getting another color.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws one or more rotating circular morphing meshes on the canvas.
}
\examples{
\donttest{
set.seed(2)

# Simple example
canvas_mesh(colors = colorPalette("origami"))
}

}
\references{
\url{https://web.archive.org/web/20231130151102/http://rectangleworld.com/blog/archives/462}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_flow.R
\name{canvas_flow}
\alias{canvas_flow}
\title{Draw A Flow Field}
\usage{
canvas_flow(
  colors,
  background = "#fafafa",
  lines = 500,
  lwd = 0.05,
  iterations = 100,
  stepmax = 0.01,
  outline = c("none", "circle", "square"),
  polar = FALSE,
  angles = NULL
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{background}{a character specifying the color used for the background.}

\item{lines}{the number of lines to draw.}

\item{lwd}{expansion factor for the line width.}

\item{iterations}{the maximum number of iterations for each line.}

\item{stepmax}{the maximum proportion of the canvas covered in each iteration.}

\item{outline}{character. Which outline to use for the artwork. Possible options are \code{none} (default), \code{circle} or \code{square}.}

\item{polar}{logical. Whether to draw the flow field with polar coordinates.}

\item{angles}{optional, a 200 x 200 matrix containing the angles in the flow field, or a character indicating the type of noise to use (\code{svm}, \code{knn}, \code{rf}, \code{perlin}, \code{cubic}, \code{simplex}, or \code{worley}). If \code{NULL} (the default), the noise type is chosen randomly.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws flow fields on a canvas. The algorithm simulates the flow of points through a field of angles which can be set manually or generated from the predictions of a supervised learning method (i.e., knn, svm, random forest) trained on randomly generated data.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_flow(colors = colorPalette("dark2"))

# Outline example
canvas_flow(
  colors = colorPalette("vrolik1"), lines = 10000,
  outline = "circle", iterations = 10, angles = "svm"
)

# Polar example
canvas_flow(
  colors = colorPalette("vrolik2"), lines = 300,
  lwd = 0.5, polar = TRUE
)

# Advanced example
angles <- matrix(0, 200, 200)
angles[1:100, ] <- seq(from = 0, to = 2 * pi, length = 100)
angles[101:200, ] <- seq(from = 2 * pi, to = 0, length = 100)
angles <- angles + rnorm(200 * 200, sd = 0.1)
canvas_flow(
  colors = colorPalette("tuscany1"), background = "black",
  angles = angles, lwd = 0.4, lines = 1000, stepmax = 0.001
)
}

}
\references{
\url{https://tylerxhobbs.com/essays/2020/flow-fields}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_stripes.R
\name{canvas_stripes}
\alias{canvas_stripes}
\title{Draw Stripes}
\usage{
canvas_stripes(
  colors,
  n = 300,
  H = 1,
  burnin = 1
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{n}{a positive integer specifying the length of the brownian motion (effectively the width of the artwork).}

\item{H}{a positive value specifying the square of the standard deviation of each step in the motion.}

\item{burnin}{a positive integer specifying the number of steps to discard before filling each row.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function creates a brownian motion on each row of the artwork and colors it according to the height of the motion.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_stripes(colors = colorPalette("random", n = 10))
}

}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colorPalette.R
\name{colorPalette}
\alias{colorPalette}
\title{Color Palette Generator}
\usage{
colorPalette(
  name,
  n = NULL
)
}
\arguments{
\item{name}{name of the color palette. Can be \code{random} for random colors, \code{complement} for complementing colors, \code{divergent} for equally spaced colors, or \code{random-palette} for a random palette, but can also be the name of a pre-implemented palette. See the \code{details} section for a list of pre-implemented palettes.}

\item{n}{the number of colors to select from the palette. Required if \code{name = 'random'}, \code{name = 'complement'}, or \code{name = 'divergent'}. Otherwise, if \code{NULL}, automatically selects all colors from the chosen palette.}
}
\value{
A vector of colors.
}
\description{
This function creates a random color palette, or allows the user to select a pre-implemented palette.
}
\details{
The following color palettes are implemented:

\if{html}{\figure{colors.svg}{options: width=600 alt="colors"}}
\if{latex}{\figure{colors.pdf}{options: width=5in}}
}
\examples{
colorPalette("divergent", 5)
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{canvas}
\keyword{palette}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_squares.R
\name{canvas_squares}
\alias{canvas_squares}
\title{Draw Squares and Rectangles}
\usage{
canvas_squares(
  colors,
  background = "#000000",
  cuts = 50,
  ratio = 1.618,
  resolution = 200,
  noise = FALSE
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{background}{a character specifying the color used for the borders of the squares.}

\item{cuts}{a positive integer specifying the number of cuts to make.}

\item{ratio}{a value specifying the \code{1:1} ratio for each cut.}

\item{resolution}{resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.}

\item{noise}{logical. Whether to add k-nn noise to the artwork. Note that adding noise increases computation time significantly in large dimensions.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function paints random squares and rectangles. It works by repeatedly cutting into the canvas at random locations and coloring the area that these cuts create.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_squares(colors = colorPalette("retro2"))
}

}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

\name{aRtsy-package}
\alias{aRtsy}
\alias{aRtsy-package}
\docType{package}
\title{aRtsy --- Generative Art using \code{ggplot2}}
\description{
  \if{html}{\figure{logo.png}{options: alt='logo' width='120' style='float: right'}}
  \code{aRtsy} aims to make generative art accessible to the general public in a straightforward and standardized manner. 
  The package provides algorithms for creating artworks that incorporate some form of randomness and are dependent on the set \code{seed}. 
  Each algorithm is implemented in a separate function with its own set of parameters that can be tweaked.
  
  For documentation on \code{aRtsy} itself, including the manual and user guide
  for the package, worked examples, and other
  tutorial information visit the \href{https://koenderks.github.io/aRtsy/}{package website}.
  
}

\author{
  \tabular{ll}{
    Koen Derks (maintainer, author)\tab <koen-derks@hotmail.com> \cr
  }
 
  Please use the citation provided by \R when citing this package. 
  A BibTex entry is available from \code{citation("aRtsy")}.
}

\seealso{
Useful links:
\itemize{
  \item The \href{https://twitter.com/aRtsy_package}{twitter feed} to check the artwork of the day.
  \item The \href{https://github.com/koenderks/aRtsy/issues}{issue page} to submit a bug report or feature request.
}
}

\keyword{aRtsy} 
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_lissajous.R
\name{canvas_lissajous}
\alias{canvas_lissajous}
\title{Draw a Lissajous Curve}
\usage{
canvas_lissajous(
  colors,
  background = "#000000",
  iterations = 2,
  neighbors = 50,
  noise = FALSE
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{background}{a character specifying the color used for the background.}

\item{iterations}{a positive integer specifying the number of iterations of the algorithm.}

\item{neighbors}{a positive integer specifying the number of neighbors a block considers when drawing the connections.}

\item{noise}{logical. Whether to add perlin noise to the coordinates of the nodes.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws lissajous curves with points connected via a k-nearest neighbor approach.
}
\examples{
\donttest{
set.seed(13)

# Simple example
canvas_lissajous(colors = colorPalette("blossom"))
}

}
\references{
\url{https://en.wikipedia.org/wiki/Lissajous_curve}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_planet.R
\name{canvas_planet}
\alias{canvas_planet}
\title{Draw Planets}
\usage{
canvas_planet(
  colors,
  threshold = 4,
  iterations = 200,
  starprob = 0.01,
  fade = 0.2,
  radius = NULL,
  center.x = NULL,
  center.y = NULL,
  light.right = TRUE,
  resolution = 1500
)
}
\arguments{
\item{colors}{a character specifying the colors used for a single planet. Can also be a list where each entry is a vector of colors for a planet.}

\item{threshold}{a character specifying the threshold for a color take.}

\item{iterations}{a positive integer specifying the number of iterations of the algorithm.}

\item{starprob}{a value specifying the probability of drawing a star in outer space.}

\item{fade}{a value specifying the amount of fading to apply.}

\item{radius}{a numeric (vector) specifying the radius of the planet(s).}

\item{center.x}{the x-axis coordinate(s) for the center(s) of the planet(s).}

\item{center.y}{the y-axis coordinate(s) for the center(s) of the planet(s).}

\item{light.right}{whether to draw the light from the right or the left.}

\item{resolution}{resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function paints one or multiple planets and uses a cellular automata to fill their surfaces.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_planet(colors = colorPalette("lava"), threshold = 3)

# Advanced example
colors <- list(
  c("khaki1", "lightcoral", "lightsalmon"),
  c("dodgerblue", "forestgreen", "white"),
  c("gray", "darkgray", "beige")
)
canvas_planet(colors,
  radius = c(800, 400, 150),
  center.x = c(1, 500, 1100),
  center.y = c(1400, 500, 1000),
  starprob = 0.005
)
}

}
\references{
\url{https://fronkonstin.com/2021/01/02/neighborhoods-experimenting-with-cyclic-cellular-automata/}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_canvas.R
\name{theme_canvas}
\alias{theme_canvas}
\title{Canvas Theme for ggplot2 Objects}
\usage{
theme_canvas(x, background = NULL, margin = 0)
}
\arguments{
\item{x}{a ggplot2 object.}

\item{background}{a character specifying the color used for the empty canvas.}

\item{margin}{margins of the canvas.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
Add a canvas theme to the plot. The canvas theme by default has no margins and fills any empty canvas with a background color.
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{canvas}
\keyword{theme}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_maze.R
\name{canvas_maze}
\alias{canvas_maze}
\title{Draw Mazes}
\usage{
canvas_maze(
  color = "#fafafa",
  walls = "black",
  background = "#fafafa",
  resolution = 20,
  polar = FALSE
)
}
\arguments{
\item{color}{a character specifying the color used for the artwork.}

\item{walls}{a character specifying the color used for the walls of the maze.}

\item{background}{a character specifying the color used for the background.}

\item{resolution}{resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.}

\item{polar}{logical, whether to use polar coordinates. Warning, this increases display and saving time dramatically.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws a maze on a canvas.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_maze(color = "#fafafa")
}

}
\references{
\url{https://github.com/matfmc/mazegenerator}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_swirls.R
\name{canvas_swirls}
\alias{canvas_swirls}
\title{Draw Swirls}
\usage{
canvas_swirls(
  colors,
  background = "#fafafa",
  iterations = 250,
  n = 250,
  curvature = 0.005,
  lwd = 0.1,
  resolution = 500
)
}
\arguments{
\item{colors}{a character (vector) specifying the color(s) used for the
artwork.}

\item{background}{a character specifying the color used for the background.}

\item{iterations}{a positive integer specifying the number of iterations of
the algorithm.}

\item{n}{a positive integer specifying the number of particles.}

\item{curvature}{a positive number specifying the curvature of the lines.
Larger values imply relatively curved lines, while lower values produce
relatively straight lines.}

\item{lwd}{expansion factor for the line width.}

\item{resolution}{resolution of the artwork in pixels per row/column.
Increasing the resolution increases the quality of the artwork but also
increases the computation time exponentially.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws swirling stripes on a canvas by simulating a
  particle system.
}
\examples{
\donttest{
set.seed(2)

# Simple example
canvas_swirls(colors = colorPalette("origami"))
}

}
\references{
\url{https://mattdesl.svbtle.com/generative-art-with-nodejs-and-canvas}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_recaman.R
\name{canvas_recaman}
\alias{canvas_recaman}
\title{Draw Recaman's Sequence}
\usage{
canvas_recaman(
  colors,
  background = "#fafafa",
  iterations = 100,
  start = 0,
  increment = 1,
  curvature = 1,
  angle = 0,
  size = 0.1,
  closed = FALSE
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{background}{a character specifying the color used for the background.}

\item{iterations}{the number of iterations of the algorithm.}

\item{start}{the starting point of the algorithm.}

\item{increment}{the increment of each step.}

\item{curvature}{the curvature of each line.}

\item{angle}{the angle at which to place the artwork.}

\item{size}{the size of the lines.}

\item{closed}{logical. Whether to plot a curve from the end of the sequence back to the starting point.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws Recaman's sequence on a canvas. The algorithm takes increasingly large steps backward on the positive number line, but if it is unable to it takes a step forward.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_recaman(colors = colorPalette("tuscany1"))
}

}
\references{
\url{https://mathworld.wolfram.com/RecamansSequence.html}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_flame.R
\name{canvas_flame}
\alias{canvas_flame}
\title{Draw a Fractal Flame}
\usage{
canvas_flame(
  colors,
  background = "#000000",
  iterations = 1000000,
  variations = 0,
  symmetry = 0,
  blend = TRUE,
  weighted = FALSE,
  post = FALSE,
  final = FALSE,
  extra = FALSE,
  display = c("colored", "logdensity"),
  zoom = 1,
  resolution = 1000,
  gamma = 1
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{background}{a character specifying the color used for the background.}

\item{iterations}{a positive integer specifying the number of iterations of the algorithm. Using more iterations results in images of higher quality but also increases the computation time.}

\item{variations}{an integer (vector) with a minimum of 0 and a maximum of 48 specifying the variations to be included in the flame. The default \code{0} includes only a linear variation. Including multiple variations (e.g., \code{c(1, 2, 3)}) increases the computation time. See the details section for more information about possible variations.}

\item{symmetry}{an integer with a minimum of -6 and a maximum of 6 indicating the type of symmetry to include in the flame. The default \code{0} includes no symmetry. Including symmetry decreases the computation time as a function of the absolute \code{symmetry} value. See the details section for more information about possible symmetries.}

\item{blend}{logical. Whether to blend the variations (\code{TRUE}) or pick a unique variation in each iteration (\code{FALSE}). \code{blend = TRUE} increases computation time as a function of the number of included variations.}

\item{weighted}{logical. Whether to weigh the functions and the variations (\code{TRUE}) or pick a function at random and equally weigh all variations (\code{FALSE}). \code{weighted = TRUE} significantly increases the computation time.}

\item{post}{logical. Whether to apply a post transformation in each iteration.}

\item{final}{logical. Whether to apply a final transformation in each iteration.}

\item{extra}{logical. Whether to apply an additional post transformation after the final transformation. Only has an effect when \code{final = TRUE}.}

\item{display}{a character indicating how to display the flame. \code{colored} (the default) displays colors according to which function they originate from. \code{logdensity} plots a gradient using the log density of the pixel count.}

\item{zoom}{a positive value specifying the amount of zooming.}

\item{resolution}{resolution of the artwork in pixels per row/column. Increasing the resolution does not increases the computation time of this algorithm.}

\item{gamma}{a numeric value specifying the gamma correction (only used when \code{display = "colored"}). Larger values result in brighter images and vice versa.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function implements the fractal flame algorithm.
}
\details{
The \code{variation} argument can be used to include specific variations into the flame. See the appendix in the references for examples of all variations. Possible variations are:

\itemize{
 \item{\code{0}: Linear (default)}
 \item{\code{1}: Sinusoidal}
 \item{\code{2}: Spherical}
 \item{\code{3}: Swirl}
 \item{\code{4}: Horsehoe}
 \item{\code{5}: Polar}
 \item{\code{6}: Handkerchief}
 \item{\code{7}: Heart}
 \item{\code{8}: Disc}
 \item{\code{9}: Spiral}
 \item{\code{10}: Hyperbolic}
 \item{\code{11}: Diamond}
 \item{\code{12}: Ex}
 \item{\code{13}: Julia}
 \item{\code{14}: Bent}
 \item{\code{15}: Waves}
 \item{\code{16}: Fisheye}
 \item{\code{17}: Popcorn}
 \item{\code{18}: Exponential}
 \item{\code{19}: Power}
 \item{\code{20}: Cosine}
 \item{\code{21}: Rings}
 \item{\code{22}: Fan}
 \item{\code{23}: Blob}
 \item{\code{24}: PDJ}
 \item{\code{25}: Fan2}
 \item{\code{26}: Rings2}
 \item{\code{27}: Eyefish}
 \item{\code{28}: Bubble}
 \item{\code{29}: Cylinder}
 \item{\code{30}: Perspective}
 \item{\code{31}: Noise}
 \item{\code{32}: JuliaN}
 \item{\code{33}: JuliaScope}
 \item{\code{34}: Blur}
 \item{\code{35}: Gaussian}
 \item{\code{36}: RadialBlur}
 \item{\code{37}: Pie}
 \item{\code{38}: Ngon}
 \item{\code{39}: Curl}
 \item{\code{40}: Rectangles}
 \item{\code{41}: Arch}
 \item{\code{42}: Tangent}
 \item{\code{43}: Square}
 \item{\code{44}: Rays}
 \item{\code{45}: Blade}
 \item{\code{46}: Secant}
 \item{\code{47}: Twintrian}
 \item{\code{48}: Cross}
}

The \code{symmetry} argument can be used to include symmetry into the flame. Possible options are:

\itemize{
 \item{\code{0}: No symmetry (default)}
 \item{\code{-1}: Dihedral symmetry}
 \item{\code{1}: Two-way rotational symmetry}
 \item{\code{(-)2}: (Dihedral) Three-way rotational symmetry}
 \item{\code{(-)3}: (Dihedral) Four-way rotational symmetry}
 \item{\code{(-)4}: (Dihedral) Five-way rotational symmetry}
 \item{\code{(-)5}: (Dihedral) Six-way rotational symmetry}
 \item{\code{(-)6}: (Dihedral) Snowflake symmetry}
}
}
\examples{
\donttest{
set.seed(3)

# Simple example, linear variation, relatively few iterations
canvas_flame(colors = c("dodgerblue", "green"), variations = 0)

# Simple example, linear variation, dihedral symmetry
canvas_flame(colors = c("hotpink", "yellow"), variations = 0, symmetry = -1, iterations = 1e7)

# Advanced example (no-blend, weighted, sinusoidal and spherical variations)
canvas_flame(
  colors = colorPalette("origami"), variations = c(1, 2),
  blend = FALSE, weighted = TRUE, iterations = 1e8
)

# More iterations give much better images
set.seed(123)
canvas_flame(colors = c("red", "blue"), iterations = 1e8, variations = c(10, 17))
}

}
\references{
\url{https://flam3.com/flame_draves.pdf}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_phyllotaxis.R
\name{canvas_phyllotaxis}
\alias{canvas_phyllotaxis}
\title{Draw a Phyllotaxis}
\usage{
canvas_phyllotaxis(
  colors,
  background = "#fafafa",
  iterations = 10000,
  angle = 137.5,
  size = 0.01,
  alpha = 1,
  p = 0.5
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{background}{a character specifying the color used for the background.}

\item{iterations}{the number of iterations of the algorithm.}

\item{angle}{the angle at which to place the artwork.}

\item{size}{the size of the lines.}

\item{alpha}{transparency of the points.}

\item{p}{probability of drawing a point on each iteration.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function draws a phyllotaxis which resembles the arrangement of leaves on a plant stem.
}
\examples{
\donttest{
set.seed(1)

# Simple example
canvas_phyllotaxis(colors = colorPalette("tuscany1"))
}

}
\references{
\url{https://en.wikipedia.org/wiki/Phyllotaxis}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_tiles.R
\name{canvas_tiles}
\alias{canvas_tiles}
\title{Draw Portuguese Tiles}
\usage{
canvas_tiles(
  colors,
  background = "#ffffff",
  iterations = 1000,
  size = 5,
  col.line = "#000000",
  resolution = 100,
  layout = NULL
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork, or a list containing a set of colors for each unique tile on the wall.}

\item{background}{a character specifying the color of the background.}

\item{iterations}{a positive integer specifying the number of iterations of the algorithm.}

\item{size}{a positive integer specifying how many tiles should be in each row of the wall.}

\item{col.line}{a character specifying the color of the joints between the tiles.}

\item{resolution}{resolution of the artwork in pixels per row/column. Increasing the resolution increases the quality of the artwork but also increases the computation time exponentially.}

\item{layout}{optional. A matrix containing integers ranging from 1 to the maximum number of unique tiles (i.e., \code{length(colors)}) specifying the placement of the tiles on the wall.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function uses a reaction diffusion algorithm in an attempt to draw a Portuguese-styled tiling pattern.
}
\examples{
\donttest{
set.seed(3)

# Simple example
canvas_tiles(colors = colorPalette("azul"), iterations = 5000)

# Advanced example
canvas_tiles(colors = list(
  colorPalette("blossom"),
  colorPalette("neon1"),
  colorPalette("dark1")
))

# Custom layout
layout <- matrix(c(
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 2, 1, 1, 1, 2, 1, 1, 1,
  1, 1, 2, 2, 2, 1, 2, 2, 2, 1, 1,
  1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1,
  1, 1, 2, 2, 2, 2, 2, 2, 2, 1, 1,
  1, 1, 1, 2, 2, 2, 2, 2, 1, 1, 1,
  1, 1, 1, 1, 2, 2, 2, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
), nrow = 11, byrow = TRUE)
canvas_tiles(
  colors = list(colorPalette("azul"), colorPalette("blossom")),
  size = nrow(layout), layout = layout
)

# Another custom layout
set.seed(11)
layout <- matrix(c(
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
  7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
  2, 2, 1, 2, 2, 3, 3, 2, 2, 4, 4, 4, 2, 5, 5, 5, 2, 6, 2, 6, 2,
  2, 1, 2, 1, 2, 3, 2, 3, 2, 2, 4, 2, 2, 5, 2, 2, 2, 6, 2, 6, 2,
  2, 1, 1, 1, 2, 3, 3, 2, 2, 2, 4, 2, 2, 2, 5, 2, 2, 2, 6, 2, 2,
  2, 1, 2, 1, 2, 3, 2, 3, 2, 2, 4, 2, 5, 5, 5, 2, 2, 2, 6, 2, 2,
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
  7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2
), nrow = 21, byrow = TRUE)
canvas_tiles(
  colors = list(
    colorPalette("blossom"),
    colorPalette("azul"),
    colorPalette("neon1"),
    colorPalette("mixer4"),
    colorPalette("neon2"),
    colorPalette("vrolik1"),
    colorPalette("blackwhite")
  ),
  iterations = 2000,
  size = nrow(layout), layout = layout
)
}

}
\references{
\url{https://en.wikipedia.org/wiki/Reaction–diffusion_system}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canvas_petri.R
\name{canvas_petri}
\alias{canvas_petri}
\title{Draw Petri Dish Colonies}
\usage{
canvas_petri(
  colors,
  background = "#fafafa",
  dish = "black",
  attractors = 1000,
  iterations = 15,
  hole = 0
)
}
\arguments{
\item{colors}{a string or character vector specifying the color(s) used for the artwork.}

\item{background}{a character specifying the color used for the background (and the hole).}

\item{dish}{a character specifying the color used for the Petri dish.}

\item{attractors}{an integer specifying the number of attractors.}

\item{iterations}{a positive integer specifying the number of iterations of the algorithm.}

\item{hole}{a value between 0 and 0.9 specifying the hole size in proportion to the dish.}
}
\value{
A \code{ggplot} object containing the artwork.
}
\description{
This function uses a space colony algorithm to draw Petri dish colonies.
}
\examples{
\donttest{
set.seed(2)

# Simple example
canvas_petri(colors = colorPalette("origami"))

# Advanced example
canvas_petri(colors = "white", hole = 0.8, attractors = 5000)
}

}
\references{
\url{https://medium.com/@jason.webb/space-colonization-algorithm-in-javascript-6f683b743dc5}
}
\seealso{
\code{colorPalette}
}
\author{
Koen Derks, \email{koen-derks@hotmail.com}
}
\keyword{artwork}
\keyword{canvas}
